//
//  wbiff! mailbox flag for Windows
//	Copyright  1999, 2000 F. Sadaghiani (sada@guesswork.org)
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 
//
//  59 Temple Place - Suite 330
//  Boston, MA 
//  02111-1307, USA.
//
//

#include "std_wb.h"
#include "types.h"

#include "about.h"

#include "hyperlink.h"
#include "wb_util.h"
#include "res/resource.h"

const LPCTSTR ABOUT_WINDOWNAME = _T("About");
const LPCTSTR ABOUT_PROGRAM_INFO = 
	_T("wbiff! v2.02a is copyright  1999-2000 by F. Sadaghiani.\n")
	_T("All rights under copyright reserved.\n")
	_T("\nliquidninja mods v1.0 by Alexander Davidson"); //'ad'

extern HINSTANCE hInstance;
extern STATS stats;
extern DWORD dwState;

static int CALLBACK 
About_PropertySheetCallback(HWND hwnd, UINT message, LPARAM lParam)
{
	switch (message) {

		case PSCB_PRECREATE:
			// Remove the DS_CONTEXTHELP style from the dialog template
#ifdef LPDLGTEMPLATEEX
			if (((LPDLGTEMPLATEEX) lParam)->signature == 0xFFFF)
				((LPDLGTEMPLATEEX) lParam)->style &= ~DS_CONTEXTHELP;
			else
#endif
				((LPDLGTEMPLATE) lParam)->style &= ~DS_CONTEXTHELP;
	}

	return TRUE;
}

static BOOL _stdcall
ProcessWindowMessage_About(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static HYPERLINK hlHome;
	static HYPERLINK hlComments;
	static HYPERLINK hlGNU;

	switch (message) {

		case WM_INITDIALOG:
			hlHome.lpURL = _T("http://guesswork.org/software/wbiff/");
			hlComments.lpURL = _T("mailto:wbiff@guesswork.org");
			hlGNU.lpURL = _T("http://www.gnu.org/");

			hlGNU.clrVisited = hlComments.clrVisited = hlHome.clrVisited = RGB(240, 50, 10);
			hlGNU.clrLink = hlComments.clrLink = hlHome.clrLink = RGB(10, 50, 240);

			hyperlink_CreateHyperLink(GetDlgItem(hwnd, IDC_ABOUT_COMMENTS), &hlComments);
			hyperlink_CreateHyperLink(GetDlgItem(hwnd, IDC_ABOUT_WEB_PAGE), &hlHome);
			hyperlink_CreateHyperLink(GetDlgItem(hwnd, IDC_ABOUT_GNU), &hlGNU);

			SetDlgItemText(hwnd, IDC_ABOUT_PROG_INFO, ABOUT_PROGRAM_INFO);
			{
				HFONT hfont = GetStockObject(SYSTEM_FIXED_FONT); 
				SendDlgItemMessage(hwnd, IDC_STATIC_X, WM_SETFONT, (WPARAM)hfont, 0);
			}

			{
				TCHAR buffer[256];
				TCHAR dbuffer[256];

				GetDateFormat(LOCALE_USER_DEFAULT, DATE_LONGDATE | LOCALE_NOUSEROVERRIDE, &stats.stFirstRun, NULL, dbuffer, sizeof(dbuffer));
				wsprintf(buffer, "This program has been executed %d times since %s, and has notified you of %d new e-mail messages in %d checks for new e-mail.",
					stats.nTimesLoaded, dbuffer, stats.nMessages, stats.nTimesChecked);
		
				SetDlgItemText(hwnd, IDC_ABOUT_STATS, buffer);
			}
			break;

		case WM_CTLCOLORSTATIC:
			{
				WNDPROC p = (WNDPROC) GetWindowLong((HWND) lParam, GWL_WNDPROC);
				return CallWindowProc(p, (HWND) lParam, message, wParam, lParam);
			}
			break;
	}

	return 0;
}

void
about_DisplayDialog(HWND hwnd)
{
	PROPSHEETPAGE psp[1];
	PROPSHEETHEADER psh;

	psp[0].pszTemplate = MAKEINTRESOURCE(IDD_ABOUT);
	psp[0].dwSize = sizeof(PROPSHEETPAGE);
	psp[0].dwFlags = PSP_DEFAULT;
	psp[0].hInstance = hInstance;
	psp[0].pfnDlgProc = ProcessWindowMessage_About;
	
	psh.dwSize = sizeof(PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW | PSH_USECALLBACK;
	
	// which parent? choose best such that about window
	// is nicely cascaded over wbiff window or in the corner
	// where the the tray is
	if (!(dwState & WBIFF_CURRENT_STATE_MAINWND_VISIBLE))
		psh.hwndParent = util_GetTrayHwnd();
	else
		psh.hwndParent = hwnd;

	psh.hInstance = hInstance;
	psh.pszCaption = ABOUT_WINDOWNAME;
	psh.nPages = sizeof(psp) / sizeof(PROPSHEETPAGE);
	psh.ppsp = (LPPROPSHEETPAGE) &psp;
	psh.nStartPage = 0;
	psh.pfnCallback = About_PropertySheetCallback;

	hyperlink_Init(hInstance);
	PropertySheet(&psh);
	hyperlink_UnInit();
}
