//
//  wbiff! mailbox flag for Windows
//	Copyright  1999, 2000 F. Sadaghiani (sada@guesswork.org)
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 
//
//  59 Temple Place - Suite 330
//  Boston, MA 
//  02111-1307, USA.
//
//

#include "std_wb.h"
#include "crypto.h"

#pragma hdrstop

typedef _TUCHAR _UCHAR;
typedef unsigned int _UINT;

//
// I wrote these algorithms for my CS209 Assignment 3 chat server. Passwords
// were encrypted using a polyalphabetic substitution cipher as implemented by
// the crypto_Encrypt and crypto_Decrypt functions. In general, polyalphabetic
// substituion ciphers are no longer used to encrypt sensitive data because 
// of they're easily broken. (I guess that a Cray could break this algorithm
// in less than 5 minutes). But, hey, if you're trying to hack Windows passwords
// you can do it even less time without a Cray.
//
// Here's some further info on the scheme:
//	http://www.albany.net/~cybernet/Vigenere.html
//  http://www.cs.adfa.oz.au/teaching/studinfo/csc/lectures/classical.html
// 

/*
 * function: crypto_Encrypt
 *
 * Description:
 *
 *  Encrypts the string pointed to by `dst' using `key' as the
 *  key for the encryption process
 *
 * Parameters:
 *
 *  key -> [IN] encryption key to encode with
 *  dst -> [IN/OUT] buffer to (first read then) write encoded string to
 *
 * Return:
 *
 *  None
 */
void 
crypto_Encrypt(LPCTSTR key, LPTSTR dst)
{
	const int keylen = lstrlen(key);
	const int dstlen = lstrlen(dst);
	int i, keyp;
	
	for (keyp = i = 0; i < dstlen; i++) {
		
		dst[i] = (_UCHAR) (((_UINT) dst[i] + (_UINT) key[keyp]) % 256);
		if (++keyp > keylen)
			keyp = 0;
	}
}

/*
 * function: crypto_Decrypt
 *
 * Description:
 *
 *  Decrypts the string `dst' using `key' as the key to decrypt 
 *  with.
 *
 * Parameters:
 *
 *  key -> [IN] encryption key (used to encode the string)
 *  dst -> [IN/OUT] buffer to (first read then) store decoded string to
 * 
 * Return:
 *
 *  None
 */
void 
crypto_Decrypt(LPCTSTR key, LPTSTR dst)
{
	const int keylen = lstrlen(key);
	const int dstlen = lstrlen(dst);
	int i, keyp;
	
	for (keyp = i = 0; i < dstlen; i++) {
		
		dst[i] = (_UCHAR) (((_UINT) dst[i] + (_UINT) (256 - key[keyp])) % 256);
		if (++keyp > keylen)
			keyp = 0;
	}
}
