//
//  wbiff! mailbox flag for Windows
//	Copyright  1999, 2000 F. Sadaghiani (sada@guesswork.org)
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 
//
//  59 Temple Place - Suite 330
//  Boston, MA 
//  02111-1307, USA.
//
//

#include "std_wb.h"
#include "hyperlink.h"

#include "wb_util.h"

#include "res/resource.h"

static HCURSOR hCursor = NULL;
static WNDPROC WndProc_OldStaticProc = NULL;

static HFONT fntURL = NULL;

extern DWORD dwOsVersion;

#pragma intrinsic(memset)

// Forwards

LRESULT CALLBACK 
hyperlink_WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

static void
hyperlink_ViewObject(HWND hwnd, LPCTSTR lpsz);


void
hyperlink_Init(HINSTANCE hInstance)
{
	// On Windows2000 load the system cursor instead
	// WBIFF_VERSION_ENABLE_MOUSE_SHADOW

	if (dwOsVersion & WBIFF_VERSION_ENABLE_MOUSE_SHADOW) {

		hCursor = LoadCursor(NULL, MAKEINTRESOURCE(32649));
	}
	else {

		hCursor = LoadImage(hInstance, MAKEINTRESOURCE(IDC_CURSOR1), IMAGE_CURSOR, 32, 32, LR_DEFAULTCOLOR);
	}
}

void
hyperlink_UnInit()
{
	DestroyCursor(hCursor);	
	if (fntURL)
		DeleteObject(fntURL);
	fntURL = NULL;
}

void
hyperlink_CreateHyperLink(HWND hwnd, PHYPERLINK p)
{
	// subclass the window
	WndProc_OldStaticProc = (WNDPROC) SetWindowLong(hwnd, GWL_WNDPROC, (LONG) hyperlink_WndProc);

	p->bVisited = false;

	// set the window data to point to the hyperlink data
	SetWindowLong(hwnd, GWL_USERDATA, (LONG) p);

	if (!fntURL) {

		// only create the font once
		HFONT hFont;
		LOGFONT lf;

		hFont = (HFONT) SendMessage(hwnd, WM_GETFONT, 0, 0);
		GetObject(hFont, sizeof(LOGFONT), &lf);

		lf.lfUnderline = TRUE;

		fntURL = CreateFontIndirect(&lf);
	}

	SendMessage(hwnd, WM_SETFONT, (WPARAM) fntURL, TRUE);
}

LRESULT CALLBACK 
hyperlink_WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {

		case WM_SETCURSOR:
			SetCursor(hCursor);
			break;

		case WM_LBUTTONUP:
			{
				PHYPERLINK p = (PHYPERLINK) GetWindowLong(hwnd, GWL_USERDATA);

				p->bVisited = true;
				InvalidateRect(hwnd, NULL, TRUE);
				UpdateWindow(hwnd);

				hyperlink_ViewObject(hwnd, p->lpURL);
			}
			break;
		
		case WM_CTLCOLORSTATIC:
			{
				HDC hdc = (HDC) wParam;
				HWND hwndStatic = (HWND) lParam;

				PHYPERLINK p = (PHYPERLINK) GetWindowLong(hwnd, GWL_USERDATA);

				SetTextColor(hdc, p->bVisited ? p->clrVisited : p->clrLink);
				SetBkMode(hdc, TRANSPARENT);

				return (BOOL) GetStockObject(NULL_BRUSH);
			}
			break;

		default:
			return CallWindowProc(WndProc_OldStaticProc, hwnd, message, wParam, lParam);
	}

	return 0;
}

static void
hyperlink_ViewObject(HWND hwnd, LPCTSTR lpsz)
{
	util_OpenObject(hwnd, lpsz);
}


