//
//  wbiff! mailbox flag for Windows
//	Copyright  1999, 2000 F. Sadaghiani (sada@guesswork.org)
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 
//
//  59 Temple Place - Suite 330
//  Boston, MA 
//  02111-1307, USA.
//
//

#include "std_wb.h"
#include "types.h"

#include "impl.h"

#include "skin.h"
#include "summarywnd.h"
#include "settings.h"
#include "wb_util.h"
#include "impl.thread.h"
#include "res/resource.h"


extern HINSTANCE hInstance;
extern SKIN skin;
extern TCHAR szWorkingFolder[MAX_PATH + 1];
extern TCHAR ucButtonMask;
extern SETTINGS settings;
extern TCHAR szWindowText[64];
extern int nCount;
extern NOTIFYICONDATA notifyIcon;
extern HANDLE hThread_CheckNow;
extern HANDLE hThread_FlashTray;
extern HANDLE hThread_FlashWindow;
extern HANDLE hThread_CheckMailAnimation;
extern HANDLE hEvent_CheckMailAnimation;
extern HANDLE hEvent_ContinueAnimationTray;
extern HANDLE hEvent_ContinueAnimationWindow;

extern DWORD dwOsVersion;
extern DWORD dwState;

static HWND hwndMainTT = NULL; // tool tips for main window

/////////////////////////////////////////////////////////////////////////////
//
// Implementation handlers

void
impl_UpdateWindowText(LPCTSTR lpFormat, ...)
{
	// Updates window text field with the given
	// message text

	va_list args;
	va_start(args, lpFormat);

	// update window text
	szWindowText[0] = _T('\0');
	wvsprintf(szWindowText, lpFormat, args);
	va_end(args);
}

void
impl_UpdateToolTipText(LPCTSTR lpFormat, ...)
{
	// Updates tooltip text field with the given
	// message text

	TCHAR buffer[64];
	
	va_list args;
	va_start(args, lpFormat);

	notifyIcon.szTip[0] = _T('\0');
	wvsprintf(buffer, lpFormat, args);
	va_end(args);

	// we need to prefix the buffer with "wbiff! - "
	lstrcpy(notifyIcon.szTip, _T("wbiff! - "));
	lstrcat(notifyIcon.szTip, buffer);

	notifyIcon.uFlags = NIF_TIP; // only modify the tooltip text
	Shell_NotifyIcon(NIM_MODIFY, &notifyIcon);
}

void // pre: lpPathString is a pointer to a buffer of size = MAX_PATH
impl_CorrectPathString(LPTSTR lpPathString)
{
	TCHAR buffer[MAX_PATH + 1];

	lstrcpy(buffer, szWorkingFolder);
	lstrcat(buffer, lpPathString);
	lstrcpy(lpPathString, buffer);
}

void
impl_ClearFlag(HWND hwnd)
{
	if (dwState & WBIFF_CURRENT_STATE_FLAG_UP) {

		int nOldCount = nCount;
		
		impl_ResetFlagAndTrayState(hwnd);
		
		nCount = nOldCount; // reset count
		
		impl_UpdateWindowText(_T("%d waiting\n(flag cleared)"), nCount);
		impl_UpdateToolTipText(_T("%d waiting (flag cleared)"), nCount);
		
		InvalidateRect(hwnd, &skin.FontRect, TRUE);
		UpdateWindow(hwnd);

		TRACE("imp_ClearFlag called\r\n");
	}

	impl_SwapOutOfMemory();
}

void
impl_RunClient(HWND hwnd)
{
	if (!(dwState & WBIFF_CURRENT_STATE_CHECKING_MAIL) && settings.szEmailClient[0] != _T('\0')) {

		impl_ResetFlagAndTrayState(hwnd);

		if (!util_Exec(hwnd, settings.szEmailClient, settings.szEmailClientArgs))
			MessageBox(hwnd, _T("Error: could not launch e-mail client program (is the file missing?)"), WBIFF_WINDOWNAME, MB_OK | MB_ICONINFORMATION);

		TRACE("imp_RunClient called\r\n");
	}

	impl_SwapOutOfMemory();
}

void
impl_HandleHotKey(HWND hwnd, int nHotKey)
{
	TRACE("imp_HandleHotKey(nHotKey = %d) called\r\n", nHotKey);
	
	switch (nHotKey) {
		
		case WBIFF_IDHK_CLEAR_FLAG:
			impl_ClearFlag(hwnd);
			break;
		
		case WBIFF_IDHK_CHECK_NOW:
			impl_CheckNow(hwnd);
			break;
			
		case WBIFF_IDHK_RUN_CLIENT:
			impl_RunClient(hwnd);
			break;
			
		case WBIFF_IDHK_TOGGLE_MAIN:
			impl_ToggleMainWindow(hwnd);
			break;
			
		case WBIFF_IDHK_TOGGLE_SUMMARY:
			impl_ToggleSummaryWindow(hwnd);
			break;
	}
}

void
impl_DestroySkin(HWND hwnd)
{
	RECT rect;
	
	// save old x,y position
	GetWindowRect(hwnd, &rect);
	settings.x = rect.left;
	settings.y = rect.top;

	// free gdi objects from skin 
	if (skin.hBitmap)
		DeleteObject(skin.hBitmap);
	if (skin.hFont)
		DeleteObject(skin.hFont);

#if 1
	// Bounds checker reports an error while deleting regions. I don't understand
	// why that would be since regions are a system resource and ought to be 
	// relinquished to the system when they're no longer needed. 
	if (skin.hRegion)
		DeleteObject(skin.hRegion); 
#endif

	if (skin.hIconFlagDown)
		DestroyIcon(skin.hIconFlagDown);

	if (skin.hIconFlagUp)
		DestroyIcon(skin.hIconFlagUp);

	if (skin.hIconFlagDownDisabled)
		DestroyIcon(skin.hIconFlagDownDisabled);

	//'ad'
	if (skin.hIconBroken)
		DestroyIcon(skin.hIconBroken);

	skin.hBitmap = NULL;
	skin.hFont = NULL;
	skin.hRegion = NULL;
	skin.hIconFlagDown = NULL;
	skin.hIconFlagUp = NULL;
	skin.hIconBroken = NULL;
	skin.hIconFlagDownDisabled = NULL;
}

HICON
impl_LoadIcon(LPTSTR lpIconImage, LPCTSTR lpDefault)
{
	UINT flags;
	LPCTSTR lpsz;
	
	if (lpIconImage[0] != _T('\0')) {
		
		// load icon from a specified icon file
		if (lpIconImage[1] != _T(':'))
			impl_CorrectPathString(lpIconImage);
		
		flags = LR_DEFAULTCOLOR | LR_LOADFROMFILE;
		lpsz = lpIconImage;
	}
	else {

		// load icon from resource
		flags = LR_DEFAULTCOLOR;
		lpsz = lpDefault;
	}
	
	return (HICON) LoadImage(hInstance, lpsz, IMAGE_ICON, 16, 16, flags);
}

bool
impl_InitalizeSkin(HWND hwnd)
{
	if (!skin_Load(&skin, settings.szSelectedSkin)) {

		// skin load will load the specified skin settings.szSelectedSkin. If that
		// skin does not exist, it tries to load "default.wbs". If "default.wbs" does
		// not exist, it tries to load the first *.wbs file in the directory. If no
		// such file exists, it returns false in which case the program terminates.

		TCHAR buffer[256];

		wsprintf(buffer, _T("The file \"%s\" does not exist and there are no other skins in the program directory. Download one or more skins and re-run the program."), settings.szSelectedSkin);
		MessageBox(hwnd, buffer, WBIFF_WINDOWNAME, MB_OK | MB_ICONSTOP);
		PostQuitMessage(0);
		return false;
	}
	else {

		//
		// Adjust the rectangle to 1-based indicies
		
		skin.FlagRect.right++;
		skin.FlagRect.bottom++;

		skin.SysButtonRect.right++;
		skin.SysButtonRect.bottom++;

		skin.MinButtonRect.right++;
		skin.MinButtonRect.bottom++;

		skin.ClsButtonRect.right++;
		skin.ClsButtonRect.bottom++;

		skin.CaptionRect.right++;
		skin.CaptionRect.bottom++;

		//
		// Load the icons: flag up, flag down, flag down disabled
		skin.hIconFlagDown = impl_LoadIcon(skin.Icon_FlagDown, MAKEINTRESOURCE(IDI_ICON8));
		skin.hIconFlagUp = impl_LoadIcon(skin.Icon_FlagUp, MAKEINTRESOURCE(IDI_ICON7));
		skin.hIconFlagDownDisabled = impl_LoadIcon(skin.Icon_FlagDownDisabled, MAKEINTRESOURCE(IDI_ICON9));

		//'ad'
		skin.hIconBroken = impl_LoadIcon(skin.Icon_Broken, MAKEINTRESOURCE(IDI_ICON11));


		//
		// Load the skin bitmap image
		if (skin.Image[1] != _T(':'))
			impl_CorrectPathString(skin.Image);
		skin.hBitmap = (HBITMAP) LoadImage(NULL, skin.Image, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_LOADFROMFILE | LR_VGACOLOR);
		if (!skin.hBitmap) {
			
			TCHAR szMessage[128 + MAX_PATH];
			szMessage[0] = _T('\0');

			wsprintf(szMessage, _T("Error: couldn't load image file \"%s\" in skin initialization file \"%s\"\n\nPress OK to modify the program settings."), skin.Image, settings.szSelectedSkin);
			MessageBox(hwnd, szMessage, WBIFF_WINDOWNAME, MB_OK | MB_ICONSTOP);
			return false;
		}

		// begin 2.01 (200-06-03) 
		//
		// Create the tray icon when the skin is being initialized. This should
		// occur after the bitmap is successfully loaded.

		notifyIcon.uFlags = NIF_ICON | NIF_TIP;

		if (settings.bShowEnabledStatus && !(dwState & WBIFF_CURRENT_STATE_ENABLED))
			notifyIcon.hIcon = skin.hIconFlagDownDisabled;
		else
			notifyIcon.hIcon = skin.hIconFlagDown;

		// Note: if the window text is non-empty, use it instead of 
		//       just the window name
		lstrcpy(notifyIcon.szTip, szWindowText[0] == _T('\0') ? WBIFF_WINDOWNAME : szWindowText);
		Shell_NotifyIcon(NIM_MODIFY, &notifyIcon);

		// end 2.01 update

		// set window position (top most or not top most)
		SetWindowPos(hwnd, settings.bAOT ? HWND_TOPMOST : HWND_NOTOPMOST, settings.x, settings.y, skin.Width, skin.Height, 0);
		if (settings.bAOT)
			SetActiveWindow(hwnd);

		// Create the region
		skin.hRegion = skin_BitmapToRegion(skin.hBitmap, RGB(255,0,255), 0, 0, 0);
		SetWindowRgn(hwnd, skin.hRegion, TRUE);
	}
	return true;
}

RECT*
impl_GetRectAndSetButtonMask(HWND hwnd, const POINT* point)
{
	// Sets the button based on the position of the
	// cursor and returns a pointer to the RECT that
	// must be invalidated
	RECT* rect = NULL;

	if (PtInRect(&skin.SysButtonRect, *point) && (skin.SysButtonDown_X != -1) && (skin.SysButtonDown_X != -1) && (skin.SysButtonUp_X != -1) && (skin.SysButtonUp_Y != -1)) {
		
		ucButtonMask = WBIFF_BUTTON_STATE_SYSBUTTON | WBIFF_BUTTON_STATE_DOWN;
		rect = &skin.SysButtonRect;
	}
	else if (PtInRect(&skin.MinButtonRect, *point) && (skin.MinButtonDown_X != -1) && (skin.MinButtonDown_X != -1) && (skin.MinButtonUp_X != -1) && (skin.MinButtonUp_Y != -1)) {
		
		ucButtonMask = WBIFF_BUTTON_STATE_MINBUTTON | WBIFF_BUTTON_STATE_DOWN;
		rect = &skin.MinButtonRect;
	}
	else if (PtInRect(&skin.ClsButtonRect, *point) && (skin.ClsButtonDown_X != -1) && (skin.ClsButtonDown_X != -1) && (skin.ClsButtonUp_X != -1) && (skin.ClsButtonUp_Y != -1)) {
		
		ucButtonMask = WBIFF_BUTTON_STATE_CLSBUTTON | WBIFF_BUTTON_STATE_DOWN;
		rect = &skin.ClsButtonRect;
	}

	return rect;
}

void
impl_GetSummaryWindowDisplayPoint(HWND hwnd, POINT* point)
{
	const int offset = 5;
	RECT rect;
	
	SystemParametersInfo(SPI_GETWORKAREA, 0, &rect, 0);
	
	if (dwState & WBIFF_CURRENT_STATE_MAINWND_VISIBLE) {
		
		GetCursorPos(point);

		// offset it a bit
		point->x -= offset;
		point->y -= offset;
	}
	else {
		
		RECT rcTray;
		HWND hwndTray = util_GetTrayHwnd();
		
		GetWindowRect(hwndTray, &rcTray);
		
		point->x = rcTray.left;
		point->y = rcTray.top;
	}
	
	// ensure window is visible
	if (point->x + summarywnd_CX + offset > rect.right)
		point->x = rect.right - (summarywnd_CX + offset);
	else if (point->x < rect.left)
		point->x = rect.left + offset;
	
	if ((point->y + summarywnd_CY + offset > rect.bottom))
		point->y = rect.bottom - (summarywnd_CY + offset);
	else if (point->y < rect.top)
		point->y = rect.top + offset;
}

void
impl_ToggleSummaryWindow(HWND hwnd)
{
	if (settings.bObtainEmailHeaders && settings.nAccounts && !(dwState & WBIFF_CURRENT_STATE_CHECKING_MAIL)) {

		if (summarywnd_IsWindowVisible())
			summarywnd_Destroy();
		else {

			POINT point;
			impl_GetSummaryWindowDisplayPoint(hwnd, &point);		
			summarywnd_Create(hwnd, point.x, point.y, settings.accounts, settings.nAccounts);
		}

		TRACE("impl_ToggleSummaryWindow called\r\n");
	}

	impl_SwapOutOfMemory();
}

void
impl_ResetFlagAndTrayState(HWND hwnd)
{
	nCount = 0;
	dwState &= ~WBIFF_CURRENT_STATE_FLAG_UP;
	szWindowText[0] = _T('\0');
	
	InvalidateRect(hwnd, &skin.FlagRect, TRUE);
	InvalidateRect(hwnd, &skin.FontRect, TRUE);

	notifyIcon.uFlags = NIF_ICON | NIF_TIP;
	
	if (settings.bShowEnabledStatus && !(dwState & WBIFF_CURRENT_STATE_ENABLED))
		notifyIcon.hIcon = skin.hIconFlagDownDisabled;
	else { //'ad'
		BOOL bBroken = FALSE;
		int i;

		for (i = 0; i < settings.nAccounts; ++i) {
			if (settings.accounts[i].nCount < 0) {
				bBroken = TRUE;
				break;
			}
		}
		notifyIcon.hIcon = (bBroken ? skin.hIconBroken : skin.hIconFlagDown);
	}

	lstrcpy(notifyIcon.szTip, WBIFF_WINDOWNAME);
	Shell_NotifyIcon(NIM_MODIFY, &notifyIcon);
	UpdateWindow(hwnd);

	if (summarywnd_IsWindowVisible())
		summarywnd_Destroy();

	TRACE("impl_ResetFlagAndTrayState called\r\n");
}

void
impl_ClearAnimations()
{
	HANDLE handles[] = { hThread_FlashWindow, hThread_FlashTray };

	if (hEvent_ContinueAnimationTray)
		SetEvent(hEvent_ContinueAnimationTray);

	if (hEvent_ContinueAnimationWindow)
		SetEvent(hEvent_ContinueAnimationWindow);

	WaitForMultipleObjects(2, handles, TRUE, INFINITE);

	TRACE("impl_ClearAnimations() called\r\n");
}

void
impl_SetFlagAndTrayState(HWND hwnd)
{
	dwState |= WBIFF_CURRENT_STATE_FLAG_UP;
	InvalidateRect(hwnd, &skin.FlagRect, TRUE);
	InvalidateRect(hwnd, &skin.FontRect, TRUE);

	notifyIcon.uFlags = NIF_ICON;
	notifyIcon.hIcon = skin.hIconFlagUp;
	Shell_NotifyIcon(NIM_MODIFY, &notifyIcon);

	UpdateWindow(hwnd);
}

void
impl_UpdateSummaryWindow(HWND hwnd, bool bDisplay)
{
	// bDisplay is only a hint.
	// If the user has chosen not to receive summary window notifications
	// they will not be sent
		
	if (summarywnd_IsWindowVisible())
		summarywnd_ReInit(settings.accounts, settings.nAccounts);
	else if (settings.bObtainEmailHeaders && settings.bAutoDisplaySummary) {

		// bDisplay is our hint of whether there is e-mail or not
		if (settings.bForceSummaryDisplay || bDisplay) {

			// force the display
			POINT point;
			impl_GetSummaryWindowDisplayPoint(hwnd, &point);		
			summarywnd_Create(hwnd, point.x, point.y, settings.accounts, settings.nAccounts);
		}
	}
}

void
impl_RenderSettings(HWND hwnd)
{
	// register hotkeys
	WORD w;

	if (w = LOWORD(settings.hk_ClearFlag))
		RegisterHotKey(hwnd, WBIFF_IDHK_CLEAR_FLAG, HIBYTE(w), LOBYTE(w));
	
	if (w = LOWORD(settings.hk_CheckNow))
		RegisterHotKey(hwnd, WBIFF_IDHK_CHECK_NOW, HIBYTE(w), LOBYTE(w));

	if (w = LOWORD(settings.hk_RunClient))
		RegisterHotKey(hwnd, WBIFF_IDHK_RUN_CLIENT, HIBYTE(w), LOBYTE(w));

	if (w = LOWORD(settings.hk_ToggleMain))
		RegisterHotKey(hwnd, WBIFF_IDHK_TOGGLE_MAIN, HIBYTE(w), LOBYTE(w));

	if (w = LOWORD(settings.hk_ToggleSummary))
		RegisterHotKey(hwnd, WBIFF_IDHK_TOGGLE_SUMMARY, HIBYTE(w), LOBYTE(w));

	// set process priority based on user setting
	SetPriorityClass(GetCurrentProcess(), settings.nPriority == 0 ? NORMAL_PRIORITY_CLASS : IDLE_PRIORITY_CLASS);

	// Create tool tips on the window
	if (settings.bEnableToolTips_Main) {

		hwndMainTT = util_TT_CreateToolTip(hInstance, hwnd);

		if (!IsRectEmpty(&skin.FlagRect))
			util_TT_AddTool(hwndMainTT, hwnd, &skin.FlagRect, NULL, WBIFF_TOOLTIP_MAIN_FLAG);

		if (!IsRectEmpty(&skin.ClsButtonRect))
			util_TT_AddTool(hwndMainTT, hwnd, &skin.ClsButtonRect, _T("Close"), WBIFF_TOOLTIP_MAIN_CLOSE);
	
		if (!IsRectEmpty(&skin.MinButtonRect))
			util_TT_AddTool(hwndMainTT, hwnd, &skin.MinButtonRect, _T("Minimze"), WBIFF_TOOLTIP_MAIN_MINIMIZE);
		
		if (!IsRectEmpty(&skin.SysButtonRect))
			util_TT_AddTool(hwndMainTT, hwnd, &skin.SysButtonRect, _T("wbiff! Menu"), WBIFF_TOOLTIP_MAIN_MENU);
	}
}

void
impl_UnrenderSettings(HWND hwnd)
{
	// unregister hot keys
	if (settings.hk_ClearFlag)
		UnregisterHotKey(hwnd, WBIFF_IDHK_CLEAR_FLAG);

	if (settings.hk_CheckNow)
		UnregisterHotKey(hwnd, WBIFF_IDHK_CHECK_NOW);

	if (settings.hk_RunClient)
		UnregisterHotKey(hwnd, WBIFF_IDHK_RUN_CLIENT);

	if (settings.hk_ToggleMain)
		UnregisterHotKey(hwnd, WBIFF_IDHK_TOGGLE_MAIN);

	if (settings.hk_ToggleSummary)
		UnregisterHotKey(hwnd, WBIFF_IDHK_TOGGLE_SUMMARY);

	// kill tool tips
	if (settings.bEnableToolTips_Main) {

		/*if (!IsRectEmpty(&skin.FlagRect))
			util_TT_DelTool(hwndMainTT, hwnd, WBIFF_TOOLTIP_MAIN_FLAG);

		if (!IsRectEmpty(&skin.ClsButtonRect))
			util_TT_DelTool(hwndMainTT, hwnd, WBIFF_TOOLTIP_MAIN_CLOSE);
	
		if (!IsRectEmpty(&skin.MinButtonRect))
			util_TT_DelTool(hwndMainTT, hwnd, WBIFF_TOOLTIP_MAIN_MINIMIZE);
		
		if (!IsRectEmpty(&skin.SysButtonRect))
			util_TT_DelTool(hwndMainTT, hwnd, WBIFF_TOOLTIP_MAIN_MENU);*/

		if (hwndMainTT) {
		
			DestroyWindow(hwndMainTT); // bounds checker reports an erorr while destroying TT windows
			hwndMainTT = NULL;
		}
	}
}

void
impl_SetTimer(HWND hwnd, bool bSetTimer)
{
	if (bSetTimer)
		SetTimer(hwnd, WBIFF_TIMER_CHECK_MAIL, 60000 * settings.nUpdateInterval, NULL);
	else
		KillTimer(hwnd, WBIFF_TIMER_CHECK_MAIL);

#ifdef _DEBUG
	if (bSetTimer)
		TRACE("impl_SetTimer(bSetTimer = true): timeout = %d\r\n", settings.nUpdateInterval);
	else
		TRACE("impl_SetTimer(bSetTimer = false): timeout = %d\r\n", settings.nUpdateInterval);
#endif
}

void 
impl_ShowMenu(HWND hwnd, const POINT* point)
{
	// "point" is expected to be in screen coordiates
	HMENU hMenu = LoadMenu(hInstance, MAKEINTRESOURCE(IDR_MENU1));

	if (hMenu) {

		HMENU hSubMenu = GetSubMenu(hMenu, 0);
		if (hSubMenu) {

			// Add custom menu item (specified in the skin)
			if (skin.HyperLinkText[0] != _T('\0') && skin.HyperLink[0] != _T('\0')) {
				
				HMENU hwbiffMenu = GetSubMenu(hSubMenu, 0);

				// Use obsolete API since `InsertMenuItem' does not work
				// on Windows '95
				AppendMenu(hwbiffMenu, MF_SEPARATOR, 0, NULL);
				AppendMenu(hwbiffMenu, MF_STRING, IDM_GOTO_USERHYPERLINK, skin.HyperLinkText);
			}			
			
			// minimize/restore
			EnableMenuItem(hSubMenu, IDM_MINIMIZE, MF_BYCOMMAND | (dwState & WBIFF_CURRENT_STATE_MAINWND_VISIBLE) ? MF_ENABLED : MF_GRAYED);
			EnableMenuItem(hSubMenu, IDM_RESTORE, MF_BYCOMMAND | (dwState & WBIFF_CURRENT_STATE_MAINWND_VISIBLE) ? MF_GRAYED : MF_ENABLED);

			// check now
			EnableMenuItem(hSubMenu, IDM_CHECK_NOW, MF_BYCOMMAND | (settings.nAccounts && !(dwState & WBIFF_CURRENT_STATE_CHECKING_MAIL) ? MF_ENABLED : MF_GRAYED));

			// run client
			EnableMenuItem(hSubMenu, IDM_RUN_CLIENT, MF_BYCOMMAND | ((dwState & WBIFF_CURRENT_STATE_CHECKING_MAIL) || settings.szEmailClient[0] == _T('\0') ? MF_GRAYED : MF_ENABLED));

			// clear flag
			EnableMenuItem(hSubMenu, IDM_CLEAR_FLAG, MF_BYCOMMAND | ((!(dwState & WBIFF_CURRENT_STATE_CHECKING_MAIL) && (dwState & WBIFF_CURRENT_STATE_FLAG_UP)) ? MF_ENABLED : MF_GRAYED));

			// settings
			EnableMenuItem(hSubMenu, IDM_SETTINGS, MF_BYCOMMAND | (!(dwState & WBIFF_CURRENT_STATE_CHECKING_MAIL) ? MF_ENABLED : MF_GRAYED));

			// enabled mode
			EnableMenuItem(hSubMenu, IDM_ENABLED,	MF_BYCOMMAND | (settings.nAccounts && !(dwState & WBIFF_CURRENT_STATE_CHECKING_MAIL) ? MF_ENABLED : MF_GRAYED));
			CheckMenuItem(hSubMenu, IDM_ENABLED, MF_BYCOMMAND | (dwState & WBIFF_CURRENT_STATE_ENABLED) ? MF_CHECKED : MF_UNCHECKED);

			// summary window
			EnableMenuItem(hSubMenu, IDM_SUMMARY, MF_BYCOMMAND | (settings.bObtainEmailHeaders && settings.nAccounts && !(dwState & WBIFF_CURRENT_STATE_CHECKING_MAIL) ? MF_ENABLED : MF_GRAYED));
			CheckMenuItem(hSubMenu, IDM_SUMMARY, MF_BYCOMMAND | (summarywnd_IsWindowVisible() ? MF_CHECKED : MF_UNCHECKED));
			SetMenuDefaultItem(hSubMenu, IDM_SUMMARY, FALSE);

			SetForegroundWindow(hwnd);
			TrackPopupMenu(hSubMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, point->x, point->y, 0, hwnd, NULL);
			PostMessage(hwnd, WM_USER, 0, 0); // fixes Windows '95 bug with menu
		}

		DestroyMenu(hMenu);
	}

	impl_SwapOutOfMemory();
}

// Platform SDK 2000 does not defined these symbolic 
// constants for some reason. They're defined in all
// previous Win32 Platform SDKs.

#ifndef IDANI_OPEN
#define IDANI_OPEN          1
#endif

#ifndef IDANI_CLOSE
#define IDANI_CLOSE         2
#endif

#ifndef IDANI_CAPTION
#define IDANI_CAPTION       3
#endif

void
impl_ToggleMainWindow(HWND hwnd)
{
	const bool bShow = !(dwState & WBIFF_CURRENT_STATE_MAINWND_VISIBLE);
	const HWND hwndTray = util_GetTrayHwnd();

	// show/hide the windows
	summarywnd_ShowWindow(bShow);
	ShowWindow(hwnd, bShow ? SW_SHOW : SW_HIDE);

	dwState ^= WBIFF_CURRENT_STATE_MAINWND_VISIBLE;

	if (bShow)
		SetForegroundWindow(hwnd);
	
	if (hwndTray) {

		LPRECT pFrom, pTo;
		RECT rc1, rc2;
		int flag = 0;

		GetWindowRect(hwnd, &rc1);
		GetWindowRect(hwndTray, &rc2);

		if (bShow) {

			pTo = &rc1;
			pFrom = &rc2;
			pFrom->left = (pFrom->right - (pFrom->right - pFrom->left) / 2);
			pFrom->right = pFrom->left + 1;
			flag = IDANI_OPEN;
		}
		else {

			pTo = &rc2;
			pFrom = &rc1;
			pTo->left = (pTo->right - (pTo->right - pTo->left) / 2);
			pTo->right = pTo->left + 1;
			flag = IDANI_CLOSE;
		}

		DrawAnimatedRects(hwnd, flag | IDANI_CAPTION, pFrom, pTo);
	}

	impl_SwapOutOfMemory();
}

void
impl_CheckNow(HWND hwnd)
{
	if (!(dwState & WBIFF_CURRENT_STATE_CHECKING_MAIL) && settings.nAccounts) {

		DWORD tid;

		dwState |= WBIFF_CURRENT_STATE_CHECKING_MAIL;

		// Kill the current timer; this timer is reset in the "thread done"
		// handler
		impl_SetTimer(hwnd, false);

		impl_UpdateWindowText(_T("Checking e-mail..."));
		impl_UpdateToolTipText(_T("Checking e-mail..."));

		InvalidateRect(hwnd, &skin.FontRect, TRUE);
		UpdateWindow(hwnd);

		if (settings.bCheckMailAnimation) {

			hEvent_CheckMailAnimation = CreateEvent(NULL, FALSE, FALSE, NULL);
			hThread_CheckMailAnimation = wb_thread(impl_ThreadProc_CheckMailAnimation, hwnd, &tid)
		}

		hThread_CheckNow = wb_thread(impl_ThreadProc_CheckNow, hwnd, &tid)

		TRACE("impl_CheckNow: threadid = %d\r\n", tid);
	}
}

void
impl_Settings(HWND hwnd)
{
	bool bAccountsChanged;
	bool bWasEnabled = dwState & WBIFF_CURRENT_STATE_ENABLED;
	bool bWasShowStatus = settings.bShowEnabledStatus;
	bool bWasObtainHeaders = settings.bObtainEmailHeaders;
	
	TRACE("impl_Settings - Enter\r\n");

	// Kill the timer
	impl_SetTimer(hwnd, false);

	impl_UnrenderSettings(hwnd);

	if (settings_Modify(hwnd, &bAccountsChanged) == IDOK) {

		if (bAccountsChanged)
			settings_UnLoad(&settings);
		
		settings_Load(&settings, bAccountsChanged);

		impl_DestroySkin(hwnd);
		impl_InitalizeSkin(hwnd);

		InvalidateRect(hwnd, NULL, TRUE);

		// if there were no changes to the accounts AND
		// the user didn't have ObtainHeaders then toggled it Off
		// just do a normal update
		if (!bAccountsChanged && !(bWasObtainHeaders && !settings.bObtainEmailHeaders)) {

			// no change to accounts
			summarywnd_UpdateWindow();
			UpdateWindow(hwnd);
		}
		else {

			if (bAccountsChanged)
				impl_ResetFlagAndTrayState(hwnd);

			// destroy summary window because user changed a setting OR
			// because the accounts changed
			if (bAccountsChanged || (bWasObtainHeaders && !settings.bObtainEmailHeaders && summarywnd_IsWindowVisible()))
				summarywnd_Destroy();
		}
	}

	impl_RenderSettings(hwnd);
	
	if (!bWasEnabled && !(dwState & WBIFF_CURRENT_STATE_FLAG_UP)) {
		
		bool bChange = false;
		
		// If the tray icon was "flag down disabled" and the user had "show enabled status" on
		// and has now turned it off and the the flag is down, then reset the image to 
		// the default "flag down" icon
		
		if (bWasShowStatus && (!settings.bShowEnabledStatus)) {

			notifyIcon.hIcon = skin.hIconFlagDown;
			bChange = true;
		}

		// Otherwise, If the state was disabled, but no idicator was on but now should be
		// on, change the icon to the "flag down disabled" image
		else if (!bWasShowStatus && settings.bShowEnabledStatus) {

			notifyIcon.hIcon = skin.hIconFlagDownDisabled;
			bChange = true;
		}

		notifyIcon.uFlags = NIF_ICON;
		Shell_NotifyIcon(NIM_MODIFY, &notifyIcon);
	}

	if (dwState & WBIFF_CURRENT_STATE_ENABLED)
		impl_SetTimer(hwnd, true);

	TRACE("impl_Settings - Leave\r\n");

	impl_SwapOutOfMemory();
}

void
impl_BeginNotificationAnimations(HWND hwnd)
{
	DWORD tid;

	if (settings.bFlashTray) {

		hEvent_ContinueAnimationTray = CreateEvent(NULL, FALSE, FALSE, NULL);
		hThread_FlashTray = wb_thread(impl_ThreadProc_FlashTray, hwnd, &tid)
	}

	if ((dwState & WBIFF_CURRENT_STATE_MAINWND_VISIBLE) && settings.bFlashWindow) {

		hEvent_ContinueAnimationWindow = CreateEvent(NULL, FALSE, FALSE, NULL);
		hThread_FlashWindow = wb_thread(impl_ThreadProc_FlashWindow, hwnd, &tid);
	}
}


void
impl_DoubleClickAction(HWND hwnd)
{
	// Double click action enumeration:
	//
	//  0: Run client
	//  1: Check for new mail (immediately)
	//  2: Minimize the window
	//  3: Do nothing
	
	switch (settings.eDca) {

		case 0: // run client
			impl_RunClient(hwnd);
			break;

		case 1:
			impl_ToggleSummaryWindow(hwnd);
			break;

		case 2: // check for new e-mail
			impl_CheckNow(hwnd);
			break;

		case 3:
			impl_ToggleMainWindow(hwnd);
			break;

#ifdef _DEBUG
		case 4:
			// Do nothing
			break;

		default:
			MessageBox(hwnd, _T("Invalid double click action specified."), WBIFF_WINDOWNAME, MB_OK);
			break;
#endif
	}
}

void
impl_ShiftClickAction(HWND hwnd)
{
	// SHIFT + Click action enumeration:
	//
	//  0: Run client
	//  1: Toggle summary window
	//  2: Clear Flag
	//  3: Check for new mail (immediately)
	//  4: Do nothing

	switch (settings.eSDca) {

		case 0:
			impl_RunClient(hwnd);
			break;

		case 1:
			impl_ToggleSummaryWindow(hwnd);
			break;

		case 2:
			impl_ClearFlag(hwnd);
			break;

		case 3:
			impl_CheckNow(hwnd);
			break;

#ifdef _DEBUG
		case 4: // The "Do nothing" case
			break;

		default:
			MessageBox(hwnd, _T("Invalid SHIFT + double click action specified."), WBIFF_WINDOWNAME, MB_OK);
			break;
#endif
	}
}

void
impl_GetVersion()
{
	OSVERSIONINFO osvi;

	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

	// assume OS has no capabilites
	dwOsVersion = WBIFF_VERSION_NULL;
		
	if (GetVersionEx(&osvi)) {

		switch (osvi.dwPlatformId) {

			case VER_PLATFORM_WIN32_NT:
				dwOsVersion |= WBIFF_VERSION_ENABLE_SWAPPING;

				// Windows 2000 allows for special effects
				if (osvi.dwMajorVersion >= 5)
					dwOsVersion |= WBIFF_VERSION_ENABLE_EFFECTS | WBIFF_VERSION_ENABLE_MOUSE_SHADOW;
				break;

			case VER_PLATFORM_WIN32_WINDOWS:
				// Windows 98 allows for special effects
				if ((osvi.dwMajorVersion > 4) || ((osvi.dwMajorVersion == 4) && (osvi.dwMinorVersion > 0)))
					dwOsVersion |= WBIFF_VERSION_ENABLE_EFFECTS;
			break;
		}
	}
}

void
impl_SwapOutOfMemory()
{
#ifndef _DEBUG
	if (dwOsVersion & WBIFF_VERSION_ENABLE_SWAPPING) {

		typedef BOOL (WINAPI *SPWSSF)(HANDLE, SIZE_T, SIZE_T);
			
		static SPWSSF spwssf = NULL;

		if (!spwssf)
			spwssf = (SPWSSF) GetProcAddress(GetModuleHandle(_T("kernel32.dll")), _T("SetProcessWorkingSetSize"));

		if (spwssf)
			spwssf(GetCurrentProcess(), -1, -1);
	}
#endif
}

void
impl_SetToolTipText(LPNMTTDISPINFO tooltip)
{
	SendMessage(tooltip->hdr.hwndFrom, TTM_SETMAXTIPWIDTH, 0, 300);
	tooltip->lpszText = szWindowText;
}

void
impl_ToggleEnabledStatus(HWND hwnd)
{
	dwState ^= WBIFF_CURRENT_STATE_ENABLED;
	impl_SetTimer(hwnd, dwState & WBIFF_CURRENT_STATE_ENABLED);

	if (settings.bShowEnabledStatus) {

		if (dwState & WBIFF_CURRENT_STATE_FLAG_UP) {

			// ClearFlag call takes care of the setting of the tray icon
			// automatically
			impl_ClearFlag(hwnd);
		}
		else {

			// flag is down. Determine which image to show
			if (dwState & WBIFF_CURRENT_STATE_ENABLED)
				notifyIcon.hIcon = skin.hIconFlagDown;
			else
				notifyIcon.hIcon = skin.hIconFlagDownDisabled;

			notifyIcon.uFlags = NIF_ICON;
			Shell_NotifyIcon(NIM_MODIFY, &notifyIcon);
		}
	}

	impl_SwapOutOfMemory();
}

