//
//  wbiff! mailbox flag for Windows
//	Copyright  1999, 2000 F. Sadaghiani (sada@guesswork.org)
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 
//
//  59 Temple Place - Suite 330
//  Boston, MA 
//  02111-1307, USA.
//
//

#include "std_wb.h"
#include "types.h"

#include "mail.imap4.h"

#include "mail.general.h"
#include "wb_util.h"

#pragma intrinsic(memcpy, memset)
#pragma hdrstop

#define buffer_size 1024

static void
imap4_SetErrorText(LPTSTR lpErrorText, LPCTSTR lpServerResponse)
{
	lstrcpyn(lpErrorText, lpServerResponse + 0x5, MAIL_ACCOUNT_FIELD_LENGTH);
	lpErrorText[lstrlen(lpErrorText) - 2] = _T('\0');
}

static 
int mail_imap4_PositiveReply(LPCTSTR lpBuffer, int nCmdIndex)
{
  TCHAR buffer[32];
	int length;
	
  // assume untagged response if nCmdIndex is negative
  if (nCmdIndex < 0)
		lstrcpy(buffer, _T("* OK"));
  else
		wsprintf(buffer, _T("A%03d OK"), nCmdIndex);
	
  length = lstrlen(buffer);
	if (lstrlen(lpBuffer) >= length) {
		
		if (CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, lpBuffer, length, buffer, length) == CSTR_EQUAL) 
			return true;
	}
	
	return false;
}

#define RECEIVESINGLELINE(hSocket, buffer, buffer_length) \
	do { if (!mail_general_ReceiveSingleLine(hSocket, buffer, buffer_length)) goto LABEL_TIMEDOUT; } while (0);

#define SENDSINGLELINE(hSocket, buffer, buffer_length) \
	do { if (!mail_general_SendSingleLine(hSocket, buffer, buffer_length)) goto LABEL_TIMEDOUT; } while (0);

void 
mail_imap4_GetCount(MAIL_PACCOUNT pAccount, bool bObtainHeaders)
{
	SOCKET hSocket;	
	
	TRACE(_T("mail_imap4_GetCount -- ENTER\r\nnAccount information:\r\n\thost: %s (port %d)\r\n\tlogin: %s\r\n\tpassword: %s\r\n\r\n"),
		pAccount->szHost, pAccount->nPort, pAccount->szLogin, pAccount->szPassword);
	
	if ((hSocket = mail_general_ConnectTo(pAccount)) == INVALID_SOCKET) {
		
		// failed to create socket or connect
		switch (WSAGetLastError()) {
			
		case WSAETIMEDOUT:
			pAccount->nCount = errorConnectionTimedout;
			break;
			
		default:
			pAccount->nCount = errorUnableToConnect;
			break;
		}
	}
	else {
		
		int nCmdIndex = -1;
		TCHAR szSendBuffer[buffer_size];
		TCHAR szRecvBuffer[buffer_size];
		TCHAR szNewMessages[buffer_size];

		// Receive welcome message
		RECEIVESINGLELINE(hSocket, szRecvBuffer, buffer_size)
		
		// ---->		
		
		// logon to system
		wsprintf(szSendBuffer, _T("A%03d LOGIN %s %s\r\n"), ++nCmdIndex, pAccount->szLogin, pAccount->szPassword);
		SENDSINGLELINE(hSocket, szSendBuffer, lstrlen(szSendBuffer))
		RECEIVESINGLELINE(hSocket, szRecvBuffer, buffer_size)
		if (!mail_imap4_PositiveReply(szRecvBuffer, nCmdIndex)) {
			
			// logon attempt failed
			imap4_SetErrorText(pAccount->szErrorText, szRecvBuffer);
			pAccount->nCount = errorBadLogon; // couldn't login
			goto LABEL_END;
		}
		
		// ---->
		
		// select inbox folder
		wsprintf(szSendBuffer, _T("A%03i SELECT %s\r\n"), ++nCmdIndex, _T("INBOX"));
		SENDSINGLELINE(hSocket, szSendBuffer, lstrlen(szSendBuffer))
		do {
			
			// receive all lines from server
			RECEIVESINGLELINE(hSocket, szRecvBuffer, buffer_size)
			
		} while (CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, szRecvBuffer, 4, szSendBuffer, 4) != CSTR_EQUAL);
		
		// determine if the last received line contained an error message
		if (!mail_imap4_PositiveReply(szRecvBuffer, nCmdIndex)) {
			
			// SELECT inbox folder attempt failed
			imap4_SetErrorText(pAccount->szErrorText, szRecvBuffer);
			pAccount->nCount = errorImap4_SelectError; // couldn't select INBOX
			goto LABEL_END;
		}
		
		// ---->
		
		// CHECK
		wsprintf(szSendBuffer, _T("A%03i CHECK\r\n"), ++nCmdIndex);
		SENDSINGLELINE(hSocket, szSendBuffer, lstrlen(szSendBuffer))
		do {
			
			// receive all lines from server
			RECEIVESINGLELINE(hSocket, szRecvBuffer, buffer_size)
			
		} while (CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, szRecvBuffer, 4, szSendBuffer, 4) != CSTR_EQUAL);
		
		// determine if the last received line contained an error message
		if (!mail_imap4_PositiveReply(szRecvBuffer, nCmdIndex)) {
			
			// CHECK command attempt failed
			imap4_SetErrorText(pAccount->szErrorText, szRecvBuffer);
			pAccount->nCount = errorImap4_CheckError; // check method failed
			goto LABEL_END;
		}
		
		// ---->

		// The command SEARCH UNSEEN isn't accepted from Domino servers.
		// This problem needs to be resolved.
		
		// SEARCH UNSEEN
		szNewMessages[0] = _T('\0');
		wsprintf(szSendBuffer, _T("A%03d SEARCH UNSEEN\r\n"), ++nCmdIndex);
		SENDSINGLELINE(hSocket, szSendBuffer, lstrlen(szSendBuffer))
		
		do {
			
			// receive all lines from server
			RECEIVESINGLELINE(hSocket, szRecvBuffer, buffer_size)
			
			// copy the line with the search results
			// Responses:  REQUIRED untagged response: SEARCH
			// (i.e. a 8 byte offset)
			if (CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, szRecvBuffer, 8, _T("* SEARCH"), 8) == CSTR_EQUAL)
				lstrcpy(szNewMessages, szRecvBuffer + 8);
			
		} while (CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, szRecvBuffer, 4, szSendBuffer, 4) != CSTR_EQUAL);
		
		// determine if the last received line contained an error message
		if (!mail_imap4_PositiveReply(szRecvBuffer, nCmdIndex)) {
			
			// SEARCH command attempt failed
			imap4_SetErrorText(pAccount->szErrorText, szRecvBuffer);
			pAccount->nCount = errorImap4_SearchError; // couldn't search unseen messages
			goto LABEL_END;
		}
		else {
			
			pAccount->nCount = 0;
			
			// check if there is actually some unseen mail
			if (lstrcmp(szNewMessages, _T("\r\n")) != 0) {
				
				TCHAR* p = szNewMessages + 1;
				do {
					// count number of new messages
					++pAccount->nCount;
					
					// skip over current number in buffer
					while (*p != _T(' ') && *p != _T('\r')) p++;
					
					// skip over any in-between spaces
					while (*p == _T(' ') && *p != _T('\r')) p++;
				}
				while (*p != _T('\r'));
				
				// get headers if needed
				if (bObtainHeaders && mail_general_AllocMessages(pAccount)) {
					
					const LPCTSTR fields_From = _T("From:");
					const LPCTSTR fields_Subject = _T("Subject:");
					const LPCTSTR fields_Date = _T("Date:");
					
					const int fields_FromLength = lstrlen(fields_From);
					const int fields_SubjectLength = lstrlen(fields_Subject);
					const int fields_DateLength = lstrlen(fields_Date);
					
					TCHAR* q;
					int nIndex = 0;
					p = szNewMessages + 1;				
					
					do {
						
						pAccount->messages[nIndex].szDate[0] =
						pAccount->messages[nIndex].szFrom[0] =
						pAccount->messages[nIndex].szSubject[0] = _T('\0');
						
						// fetch the message header
						wsprintf(szSendBuffer, _T("A%03i FETCH %d (FLAGS RFC822.HEADER.LINES (SUBJECT DATE FROM))\r\n"), ++nCmdIndex, util_atoi(p));
						
						// skip over current number in buffer
						while (*p != _T(' ') && *p != _T('\r')) p++;
						
						// skip over any in-between spaces
						while (*p == _T(' ') && *p != _T('\r')) p++;
						
						// send request for message headers and allocate a new message for
						// the account
						SENDSINGLELINE(hSocket, szSendBuffer, lstrlen(szSendBuffer))
						do {
							
							RECEIVESINGLELINE(hSocket, szRecvBuffer, buffer_size)
							
							// parse the output here
							if (pAccount->messages[nIndex].szFrom[0] == _T('\0') && CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, szRecvBuffer, fields_FromLength, fields_From, fields_FromLength) == CSTR_EQUAL) {
								
								q = util_TrimBoth(szRecvBuffer + fields_FromLength);
								lstrcpy(pAccount->messages[nIndex].szFrom, q);
							}
							else if (pAccount->messages[nIndex].szSubject[0] == _T('\0') && CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, szRecvBuffer, fields_SubjectLength, fields_Subject, fields_SubjectLength) == CSTR_EQUAL) {
								
								q = util_TrimBoth(szRecvBuffer + fields_SubjectLength);
								lstrcpy(pAccount->messages[nIndex].szSubject, q);
							}
							else if (pAccount->messages[nIndex].szDate[0] == _T('\0') && CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, szRecvBuffer, fields_DateLength, fields_Date, fields_DateLength) == CSTR_EQUAL) {
								
								q = util_TrimBoth(szRecvBuffer + fields_DateLength);
								lstrcpy(pAccount->messages[nIndex].szDate, q);
							}
							
						} while (CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, szRecvBuffer, 4, szSendBuffer, 4) != CSTR_EQUAL);
						
						++nIndex;
					}
					while (*p != _T('\r'));
				}
			}
		}

		// if we get here, everything is OK
		goto LABEL_END;

LABEL_TIMEDOUT:
		pAccount->nCount = errorSendRecvTimedout;
		goto LABEL_EXIT;

LABEL_END:
		wsprintf(szSendBuffer, _T("A%03d LOGOUT"), nCmdIndex++);
		mail_general_SendSingleLine(hSocket, szSendBuffer, lstrlen(szSendBuffer)); // no need for time-out check
	}

LABEL_EXIT:
	closesocket(hSocket);
	
	TRACE(_T("-- mail_imap4_GetCount -- LEAVE\r\n"));
}

/**
		}
		__except (EXCEPTION_EXECUTE_HANDLER) {
			
			DWORD e = GetExceptionCode();
			switch (e) {

				case WSAETIMEDOUT:
					pAccount->nCount = errorSendRecvTimedout;
					break;

#ifdef _DEBUG
				default:
					MessageBox(NULL, "Exception @ mail_pop3_GetCount", NULL, MB_OK);
					break;
#endif
			}
		}

**/



