//
//  wbiff! mailbox flag for Windows
//	Copyright  1999, 2000 F. Sadaghiani (sada@guesswork.org)
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 
//
//  59 Temple Place - Suite 330
//  Boston, MA 
//  02111-1307, USA.
//
//

#include "std_wb.h"
#include "types.h"

#include "mail.pop3.h"

#include "mail.general.h"
#include "wb_util.h"

#pragma intrinsic(memcpy)
#pragma hdrstop


// If the server returned a negative response, the error response
// is copied into lpBuffer and the function returns TRUE; otherwise
// the lpResult is a valid response line and the call succeeded
static bool
mail_pop3_NegativeReply(LPCTSTR lpResult, LPTSTR lpBuffer, int nLength)
{
	int nResultLength = lstrlen(lpResult) + 1; // include the space needed for null terminator
	
	if (nResultLength >= 4) { // error string is "-ERR "

		if (lpResult[0] == _T('-') && lpResult[1] == _T('E')) { // must be an error code
			
			// lstrcpyn is funny: it always subtracts 1 from the specified count 
			lstrcpyn(lpBuffer, lpResult, (nResultLength > nLength) ? nLength: nResultLength - 2); // don't add \r\n
			return true;
		}
	}

	return false;
}

// returns true iff the end of the message has been reached
static bool
mail_pop3_EndOfMessage(LPCTSTR lpBuffer)
{
	return lstrlen(lpBuffer) == 3 && lpBuffer[0] == _T('.') && lpBuffer[1] == _T('\r') && lpBuffer[2] == _T('\n');
}

/**
static bool
mail_pop3_ExtractHeaders(SOCKET hSocket, MAIL_PACCOUNT pAccount, LPTSTR lpBuffer, int nLength, int nIndex)
{
	const LPCTSTR fields_From = _T("From:");
	const LPCTSTR fields_Subject = _T("Subject:");
	const LPCTSTR fields_Date = _T("Date:");

	const int fields_FromLength = lstrlen(fields_From);
	const int fields_SubjectLength = lstrlen(fields_Subject);
	const int fields_DateLength = lstrlen(fields_Date);
	
	bool collected, done;
	
	TRACE("-- mail_ExtractHeaders --\r\n");
	
	// initialize the message buffer with NULLS
	pAccount->messages[nIndex].szDate[0] =
	pAccount->messages[nIndex].szFrom[0] =
	pAccount->messages[nIndex].szSubject[0] = _T('\0');

	done =             // not done yet
	collected = false; // haven't collected the information yet

	while (!done) {

		// receive a line from the server
		RECEIVESINGLELINE(hSocket, lpBuffer, nLength);
		if (mail_pop3_NegativeReply(lpBuffer, pAccount->szErrorText, MAIL_ACCOUNT_FIELD_LENGTH))
			return false;
		else if (!collected) {

			TCHAR* p; // pointer in lpBuffer

			if (pAccount->messages[nIndex].szFrom[0] == _T('\0') && CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, lpBuffer, fields_FromLength, fields_From, fields_FromLength) == CSTR_EQUAL) {

				p = util_TrimBoth(lpBuffer + fields_FromLength);
				lstrcpy(pAccount->messages[nIndex].szFrom, p);
			}
			else if (pAccount->messages[nIndex].szSubject[0] == _T('\0') && CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, lpBuffer, fields_SubjectLength, fields_Subject, fields_SubjectLength) == CSTR_EQUAL) {

				p = util_TrimBoth(lpBuffer + fields_SubjectLength);
				lstrcpy(pAccount->messages[nIndex].szSubject, p);
			}
			else if (pAccount->messages[nIndex].szDate[0] == _T('\0') && CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, lpBuffer, fields_DateLength, fields_Date, fields_DateLength) == CSTR_EQUAL) {

				p = util_TrimBoth(lpBuffer + fields_DateLength);
				lstrcpy(pAccount->messages[nIndex].szDate, p);
			}

			// update collected flag
			collected = pAccount->messages[nIndex].szDate[0] != _T('\0') && pAccount->messages[nIndex].szFrom[0] != _T('\0') &&	pAccount->messages[nIndex].szSubject[0] != _T('\0');
		}
		
		// are we done parsing?
		done = mail_pop3_EndOfMessage(lpBuffer);
	}

	TRACE(_T("nIndex: %d\r\nFrom: %s\r\nDate: %s\r\nSubject: %s\r\n-- Leaving mail_ExtractHeaders --\r\n"), 
		nIndex,
		pAccount->messages[nIndex].szFrom,
		pAccount->messages[nIndex].szDate,
		pAccount->messages[nIndex].szSubject);

	return true;
}
**/

/////////////////////////////////////////////////////////////////////////////

#define buffer_size 1024

#define RECEIVESINGLELINE(hSocket, buffer, buffer_length) \
	do { if (!mail_general_ReceiveSingleLine(hSocket, buffer, buffer_length)) goto LABEL_TIMEDOUT; } while (0);

#define SENDSINGLELINE(hSocket, buffer, buffer_length) \
	do { if (!mail_general_SendSingleLine(hSocket, buffer, buffer_length)) goto LABEL_TIMEDOUT; } while (0);

void
mail_pop3_GetCount(MAIL_PACCOUNT pAccount, bool bObtainHeaders)
{
	SOCKET hSocket = INVALID_SOCKET;

	TRACE(_T("-- mail_pop3_GetCount -- ENTER\r\n\r\nAccount information:\r\n\thost: %s (port %d)\r\n\tlogin: %s\r\n\tpassword: %s\r\n"),
		pAccount->szHost, pAccount->nPort, pAccount->szLogin, pAccount->szPassword);

	if ((hSocket = mail_general_ConnectTo(pAccount)) == INVALID_SOCKET) {

		// failed to create socket or connect
		switch (WSAGetLastError()) {

			case WSAETIMEDOUT:
				pAccount->nCount = errorConnectionTimedout;
				break;

			default:
				pAccount->nCount = errorUnableToConnect;
				break;
		}
	}
	else {

		TCHAR buffer[buffer_size]; // POP3 RFC specifies a maximum of 512 (but allocate more)

		buffer[0] = _T('\0'); // init buffer
		pAccount->nCount = 0;

		// receive initial welcome message
		RECEIVESINGLELINE(hSocket, buffer, buffer_size)

		// AUTHORIZATION STATE
	
		// USER
		wsprintf(buffer, _T("USER %s\r\n"), pAccount->szLogin);
		SENDSINGLELINE(hSocket, buffer, lstrlen(buffer))
		RECEIVESINGLELINE(hSocket, buffer, buffer_size)
		if (mail_pop3_NegativeReply(buffer, pAccount->szErrorText, MAIL_ACCOUNT_FIELD_LENGTH)) {

			pAccount->nCount = errorBadLogon;
			goto LABEL_END;
		}

		// PASS
		wsprintf(buffer, _T("PASS %s\r\n"), pAccount->szPassword);
		SENDSINGLELINE(hSocket, buffer, lstrlen(buffer))
		RECEIVESINGLELINE(hSocket, buffer, buffer_size)
		if (mail_pop3_NegativeReply(buffer, pAccount->szErrorText, MAIL_ACCOUNT_FIELD_LENGTH)) {
			
			pAccount->nCount = errorBadLogon;
			goto LABEL_END;
		}

		// STAT
		lstrcpy(buffer, _T("STAT\r\n"));
		SENDSINGLELINE(hSocket, buffer, lstrlen(buffer))
		RECEIVESINGLELINE(hSocket, buffer, buffer_size)
		if (mail_pop3_NegativeReply(buffer, pAccount->szErrorText, MAIL_ACCOUNT_FIELD_LENGTH)) {

			pAccount->nCount = errorPop3_StatError;
			goto LABEL_END;
		}
		else {

			// parse the response: number of waiting messages
			// positive response format is well-known, space parsing is OK
			LPTSTR p = buffer;
			while (*p != _T(' ')) p++;
			pAccount->nCount = util_atoi(++p);
		}

		// allocate memory for needed headers
		if (bObtainHeaders && (pAccount->nCount > 0) && mail_general_AllocMessages(pAccount)) {
		
			const LPCTSTR fields_From = _T("From:");
			const LPCTSTR fields_Subject = _T("Subject:");
			const LPCTSTR fields_Date = _T("Date:");

			const int fields_FromLength = lstrlen(fields_From);
			const int fields_SubjectLength = lstrlen(fields_Subject);
			const int fields_DateLength = lstrlen(fields_Date);
			
			bool collected, done;
			
			int i;

			for (i = 0; i < pAccount->nCount; ++i) {

				wsprintf(buffer, _T("TOP %d 0\r\n"), i + 1);
				SENDSINGLELINE(hSocket, buffer, lstrlen(buffer))

				pAccount->messages[i].szDate[0] =
				pAccount->messages[i].szFrom[0] =
				pAccount->messages[i].szSubject[0] = _T('\0');

				done =             // not done yet
				collected = false; // haven't collected the information yet

				while (!done) {

					RECEIVESINGLELINE(hSocket, buffer, buffer_size)
					if (mail_pop3_NegativeReply(buffer, pAccount->szErrorText, MAIL_ACCOUNT_FIELD_LENGTH)) {

						pAccount->nCount = errorPop3_TopError;
						goto LABEL_END;
					}
					else if (!collected) {

						TCHAR* p; // pointer in lpBuffer

						if (pAccount->messages[i].szFrom[0] == _T('\0') && CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, buffer, fields_FromLength, fields_From, fields_FromLength) == CSTR_EQUAL) {

							p = util_TrimBoth(buffer + fields_FromLength);
							lstrcpy(pAccount->messages[i].szFrom, p);
						}
						else if (pAccount->messages[i].szSubject[0] == _T('\0') && CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, buffer, fields_SubjectLength, fields_Subject, fields_SubjectLength) == CSTR_EQUAL) {

							p = util_TrimBoth(buffer + fields_SubjectLength);
							lstrcpy(pAccount->messages[i].szSubject, p);
						}
						else if (pAccount->messages[i].szDate[0] == _T('\0') && CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE, buffer, fields_DateLength, fields_Date, fields_DateLength) == CSTR_EQUAL) {

							p = util_TrimBoth(buffer + fields_DateLength);
							lstrcpy(pAccount->messages[i].szDate, p);
						}

						collected = pAccount->messages[i].szDate[0] != _T('\0') && pAccount->messages[i].szFrom[0] != _T('\0') &&	pAccount->messages[i].szSubject[0] != _T('\0');
					}
					
					done = mail_pop3_EndOfMessage(buffer);
				}

				TRACE(_T("nIndex: %d\r\nFrom: %s\r\nDate: %s\r\nSubject: %s\r\n-- Leaving mail_ExtractHeaders --\r\n\r\n"), 
					i,
					pAccount->messages[i].szFrom,
					pAccount->messages[i].szDate,
					pAccount->messages[i].szSubject);
			}
		}

		// if we get here, everthing is OK
		goto LABEL_END;

LABEL_TIMEDOUT:
		pAccount->nCount = errorSendRecvTimedout;
		goto LABEL_EXIT;

LABEL_END:	
		lstrcpy(buffer, _T("QUIT\r\n"));
		mail_general_SendSingleLine(hSocket, buffer, lstrlen(buffer));
		mail_general_ReceiveSingleLine(hSocket, buffer, buffer_size);
	}

LABEL_EXIT:
	closesocket(hSocket);

	TRACE(_T("-- mail_pop3_GetCount -- LEAVE\r\n"));
}

/**
		}
		__except (EXCEPTION_EXECUTE_HANDLER) {

			DWORD e = GetExceptionCode();
			switch (e) {

				case WSAETIMEDOUT:
					pAccount->nCount = errorSendRecvTimedout;
					break;

#ifdef _DEBUG
				default:
					MessageBox(NULL, "Exception @ mail_pop3_GetCount", NULL, MB_OK);
					break;
#endif
			}
		}
**/
