//
//  wbiff! mailbox flag for Windows
//	Copyright  1999, 2000 F. Sadaghiani (sada@guesswork.org)
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 
//
//  59 Temple Place - Suite 330
//  Boston, MA 
//  02111-1307, USA.
//
//

/*
TODO
----
-generalized framework
*/

#include "std_wb.h"

#include "crypto.h"
#include "types.h"
#include "res/resource.h"
#include "hyperlink.h"
#include "reg.h"
#include "wb_util.h"

#pragma intrinsic(memset)

const LPCTSTR SETTINGS_WINDOWNAME = _T("Settings");

static bool bAccountsChanged = false;
static SETTINGS temp_settings;    // global temporary settings
static MAIL_ACCOUNT account; // global temporary account

extern HINSTANCE hInstance;
extern DWORD dwState;

static enum { // NOTE: This enum defines the order of pages in propsheet
	
	pageAccounts = 0,
	pageGeneral,
	pageHeaders,
	pageHotKeys,
	pageMouse,
	pageAnimation,
	pageSkin,
	pageAdvanced,

	// count of pages (last item)
	pageCount
};

/*
===========================
GetCheckedRadioButton
===========================
*/
static int 
GetCheckedRadioButton(HWND hwnd, int nIDFirstButton, int nIDLastButton)
{
	int nID;
	
	for (nID = nIDFirstButton; nID <= nIDLastButton; ++nID)
		if (IsDlgButtonChecked(hwnd, nID))
			return nID; // id that matched
		
		return 0; // invalid ID
}

/*
===========================
BrowseForFileName
===========================
*/
static BOOL
BrowseForFileName(HWND hwnd, LPTSTR lpFileName, int nLength, LPCTSTR lpFilter)
{
	OPENFILENAME of;
	TCHAR szFolder[MAX_PATH], szFile[MAX_PATH], *p;
	
	if (lpFileName[0] == _T('\0')) {
		
		// no default file specified
		szFolder[0] = szFile[0] = _T('\0');
	}
	else {
		
		// default file specified
		lstrcpy(szFolder, lpFileName);
		p = szFolder + nLength;
		while (p > szFolder && *p != _T('\\'))
			p--;
		
		lstrcpyn(szFile, ++p, lstrlen(p));
		
		*p = lpFileName[0] =  _T('\0');
	}
	
	of.lStructSize = sizeof(OPENFILENAME);
	of.hwndOwner = hwnd;
	of.hInstance = NULL;
	of.lpstrFilter = lpFilter;
	of.lpstrCustomFilter = (LPTSTR) NULL;
	of.nMaxCustFilter = 0L;
	of.nFilterIndex = 1L;
	of.lpstrFile = szFile;
	of.nMaxFile = nLength;
	of.lpstrFileTitle = NULL;
	of.nMaxFileTitle = 0;
	of.lpstrInitialDir = szFolder;
	of.lpstrTitle = _T("Select File");
	of.nFileOffset = 0;
	of.nFileExtension = 0;
	of.lpstrDefExt = NULL;
	of.lCustData = 0;
	of.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_EXPLORER | OFN_LONGNAMES;
	
	if (GetOpenFileName(&of)) {
		
		// copy buffer back
		lstrcpy(lpFileName, szFile);
		return TRUE;
	}
	else 
		return FALSE;
}

static void
SetLoadOnWindowsStartupFlagInRegistry(bool bEnabled)
{
	HKEY hKey;
	static const LPCTSTR lpKeyName = _T("wbiff_autoload");
	
	if (RegOpenKeyEx(HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\CurrentVersion\\Run"),	0, KEY_ALL_ACCESS, &hKey) == ERROR_SUCCESS) {
		
		if (!bEnabled) {
			
			// removed the key
			RegDeleteValue(hKey, lpKeyName);
		}
		else {
			
			// set the key
			TCHAR buffer[MAX_PATH + 1], *p;
			
			buffer[0] = _T('\0');
			p = buffer + GetModuleFileName(NULL, buffer, sizeof(buffer));
			while (p >= buffer && *p != _T('\\')) p--;
			*(++p) = _T('\0');
			
			lstrcat(p, _T("wbiff.exe"));
			RegSetValueEx(hKey, lpKeyName, 0, REG_SZ, buffer, lstrlen(buffer));
		}
	}
}

// --------------------------------------------------------------------------

/*
===========================
Accounts Page
===========================
*/
static void
Accounts_CopyAccount(MAIL_PACCOUNT p, const MAIL_PACCOUNT account)
{
	ZeroMemory(p, sizeof(MAIL_ACCOUNT));
	lstrcpy(p->szDisplayName, account->szDisplayName);
	lstrcpy(p->szHost, account->szHost);
	lstrcpy(p->szLogin, account->szLogin);
	lstrcpy(p->szPassword, account->szPassword);
	p->protocol = account->protocol;
	p->nPort = account->nPort;
	p->nTimeout = account->nTimeout;
}

static void
Accounts_InsertAccount(HWND hwnd, MAIL_PACCOUNT account)
{
	MAIL_PACCOUNT p = (MAIL_PACCOUNT) wb_malloc(sizeof(MAIL_ACCOUNT));
	LV_ITEM lvi;
	int item;
	
	// copy existing account into temporary memory for lParam pointer
	Accounts_CopyAccount(p, account);
	
	lvi.mask = LVIF_TEXT | LVIF_PARAM;
	lvi.iSubItem = 0;
	lvi.pszText = account->szDisplayName;
	lvi.cchTextMax = MAIL_ACCOUNT_FIELD_LENGTH;
	lvi.lParam = (LPARAM) p;
	lvi.iItem = SendDlgItemMessage(hwnd, IDC_ACCOUNTS_LIST, LVM_GETITEMCOUNT, 0, 0);
	
	item = SendDlgItemMessage(hwnd, IDC_ACCOUNTS_LIST, LVM_INSERTITEM, 0, (LPARAM) &lvi);
	
	lvi.mask = LVIF_TEXT;
	lvi.iSubItem = 1;
	lvi.pszText = account->protocol == p_POP3 ? _T("POP3") : _T("IMAP4");
	lvi.iItem = item;
	SendDlgItemMessage(hwnd, IDC_ACCOUNTS_LIST, LVM_SETITEM, 0, (LPARAM) &lvi);
}

static void
Accounts_PopulateList(HWND hwnd)
{
	const DWORD style = LVS_EX_FULLROWSELECT /*| LVS_EX_CHECKBOXES*/;
	int i;
	LVCOLUMN lvc;
	
	// set extended style
	SendDlgItemMessage(hwnd, IDC_ACCOUNTS_LIST, LVM_SETEXTENDEDLISTVIEWSTYLE, style, style);
	
	// insert columms
	lvc.mask = LVCF_FMT | LVCF_TEXT | LVCF_WIDTH;
	lvc.fmt = LVCFMT_LEFT;
	lvc.iSubItem = 0;
	lvc.pszText = _T("Account");
	lvc.cx = 160;
	
	SendDlgItemMessage(hwnd, IDC_ACCOUNTS_LIST, LVM_INSERTCOLUMN, 0, (LPARAM) &lvc);
	
	lvc.cx = 100;
	lvc.pszText = _T("Protocol Type");
	SendDlgItemMessage(hwnd, IDC_ACCOUNTS_LIST, LVM_INSERTCOLUMN, 1, (LPARAM) &lvc);
	
	// insert all existing accounts
	for (i = 0; i < temp_settings.nAccounts; i++)
		Accounts_InsertAccount(hwnd, &temp_settings.accounts[i]);
}

static void
Accounts_Update(HWND hwnd)
{
	if (bAccountsChanged) {
		
		int i;
		LV_ITEM lvi;
		MAIL_PACCOUNT p;
		
		temp_settings.nAccounts = SendDlgItemMessage(hwnd, IDC_ACCOUNTS_LIST, LVM_GETITEMCOUNT, 0, 0);
		
		wb_free(temp_settings.accounts);
		temp_settings.accounts = (MAIL_PACCOUNT) wb_malloc(temp_settings.nAccounts * sizeof(MAIL_ACCOUNT));
		
		for (i = 0; i < temp_settings.nAccounts; i++) {
			
			lvi.mask = LVIF_PARAM;
			lvi.iItem = i;
			lvi.iSubItem = 0;
			
			SendDlgItemMessage(hwnd, IDC_ACCOUNTS_LIST, LVM_GETITEM, 0, (LPARAM) &lvi);
			p = (MAIL_PACCOUNT) lvi.lParam;
			Accounts_CopyAccount(&temp_settings.accounts[i], p);
		}
	}
	
	temp_settings.nUpdateInterval = GetDlgItemInt(hwnd, IDC_ACCOUNTS_UPDATE_INTERVAL, NULL, FALSE);
	temp_settings.nMinLimit = GetDlgItemInt(hwnd, IDC_ACCOUNTS_MIN_LIMIT, NULL, FALSE);
}

static BOOL _stdcall
Accounts_AccountPropertiesDlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		
		case WM_INITDIALOG:
			CheckRadioButton(hwnd, IDC_ACCOUNT_PROPERTIES_PROTOCOL_POP3, IDC_ACCOUNT_PROPERTIES_PROTOCOL_IMAP4, IDC_ACCOUNT_PROPERTIES_PROTOCOL_POP3 + account.protocol);
			SetDlgItemText(hwnd, IDC_ACCOUNT_PROPERTIES_DISPLAY_NAME, account.szDisplayName);
			SetDlgItemText(hwnd, IDC_ACCOUNT_PROPERTIES_HOST, account.szHost);
			SetDlgItemText(hwnd, IDC_ACCOUNT_PROPERTIES_LOGIN, account.szLogin);
			SetDlgItemText(hwnd, IDC_ACCOUNT_PROPERTIES_PASSWORD, account.szPassword);
			return 0;
			break;
			
		case WM_NOTIFY:
			{
				LPNMHDR hdr = (LPNMHDR) lParam;
				if (hdr->code == PSN_KILLACTIVE) {
					
					account.protocol = GetCheckedRadioButton(hwnd, IDC_ACCOUNT_PROPERTIES_PROTOCOL_POP3, IDC_ACCOUNT_PROPERTIES_PROTOCOL_IMAP4) - IDC_ACCOUNT_PROPERTIES_PROTOCOL_POP3;
					GetDlgItemText(hwnd, IDC_ACCOUNT_PROPERTIES_DISPLAY_NAME, account.szDisplayName, MAIL_ACCOUNT_FIELD_LENGTH);
					GetDlgItemText(hwnd, IDC_ACCOUNT_PROPERTIES_HOST, account.szHost, MAIL_ACCOUNT_FIELD_LENGTH);
					GetDlgItemText(hwnd, IDC_ACCOUNT_PROPERTIES_LOGIN, account.szLogin, MAIL_ACCOUNT_FIELD_LENGTH);
					GetDlgItemText(hwnd, IDC_ACCOUNT_PROPERTIES_PASSWORD, account.szPassword, MAIL_ACCOUNT_FIELD_LENGTH);
				}
			}
			break;
			
		case WM_COMMAND:
			if (HIWORD(wParam) == EN_CHANGE) {
				
				// all text fields must be specified
				EnableWindow(GetDlgItem(GetParent(hwnd), IDOK),
					GetWindowTextLength(GetDlgItem(hwnd, IDC_ACCOUNT_PROPERTIES_DISPLAY_NAME)) &&
					GetWindowTextLength(GetDlgItem(hwnd, IDC_ACCOUNT_PROPERTIES_HOST)) && 
					GetWindowTextLength(GetDlgItem(hwnd, IDC_ACCOUNT_PROPERTIES_LOGIN)) && 
					GetWindowTextLength(GetDlgItem(hwnd, IDC_ACCOUNT_PROPERTIES_PASSWORD))
					);
			}
			else if (HIWORD(wParam) == BN_CLICKED) {
				
				if (LOWORD(wParam) == IDC_ACCOUNT_PROPERTIES_PROTOCOL_POP3)
					account.nPort = WBIFF_DEFAULT_PROTOCOL_POP3_PORT;
				else if (LOWORD(wParam) == IDC_ACCOUNT_PROPERTIES_PROTOCOL_IMAP4)
					account.nPort = WBIFF_DEFAULT_PROTOCOL_IMAP4_PORT;
			}
			break;			
	}
	return 0;
}

static void
Accounts_AccountPropertiesAdvDlgProc_UpdateTimeoutInfo(HWND hwnd, int position)
{
	TCHAR buffer[32];
	
	SendDlgItemMessage(hwnd, IDC_ACCOUNT_PROPERTIES_SOCKET_TIMEOUT, TBM_SETPOS, TRUE, position);
	
	account.nTimeout = position * 30;
	if (account.nTimeout > 30) {
		
		int mins = account.nTimeout / 60;
		int secs = account.nTimeout - mins * 60;
		
		if (secs)
			wsprintf(buffer, _T("%d mins. %d seconds"), mins, secs);
		else
			wsprintf(buffer, _T("%d minute%s"), mins, mins > 1 ? _T("s") : _T(""));
	}
	else 
		wsprintf(buffer, _T("%d seconds"), account.nTimeout);
	
	SetWindowText(GetDlgItem(hwnd, IDC_ACCOUNT_PROPERTIES_SOCKET_TIMEOUT_HINT), buffer);
}

static BOOL _stdcall
Accounts_AccountPropertiesAdvDlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		
		case WM_INITDIALOG:
			SendDlgItemMessage(hwnd, IDC_SPIN1, UDM_SETRANGE, 0, MAKELONG(9999, 1));
			SendDlgItemMessage(hwnd, IDC_ACCOUNT_PROPERTIES_SOCKET_TIMEOUT, TBM_SETRANGE, FALSE, MAKELONG(1, 5));
			Accounts_AccountPropertiesAdvDlgProc_UpdateTimeoutInfo(hwnd, account.nTimeout / 30);
			return 0;
			break;
			
		case WM_NOTIFY:
			{
				LPNMHDR hdr = (LPNMHDR) lParam;
				if (hdr->code == PSN_KILLACTIVE) {
					
					account.nPort = GetDlgItemInt(hwnd, IDC_ACCOUNT_PROPERTIES_PORT, NULL, FALSE);
				}
				else if (hdr->code == PSN_SETACTIVE) {
					
					SetDlgItemInt(hwnd, IDC_ACCOUNT_PROPERTIES_PORT, account.nPort, FALSE);
				}
			}
			break;
			
		case WM_HSCROLL:
			Accounts_AccountPropertiesAdvDlgProc_UpdateTimeoutInfo(hwnd, SendDlgItemMessage(hwnd, IDC_ACCOUNT_PROPERTIES_SOCKET_TIMEOUT, TBM_GETPOS, 0, 0));
			break;
			
		case WM_COMMAND:
			if (HIWORD(wParam) == BN_CLICKED && LOWORD(wParam) == IDC_ACCOUNT_PROPERTIES_PORT_DEFAULT) {
				
				SetDlgItemInt(
					hwnd, 
					IDC_ACCOUNT_PROPERTIES_PORT, 
					account.nPort = (account.protocol == p_POP3 ?	WBIFF_DEFAULT_PROTOCOL_POP3_PORT : WBIFF_DEFAULT_PROTOCOL_IMAP4_PORT),
					FALSE
					);
			}
			else if (HIWORD(wParam) == EN_CHANGE) {
				
				// all text fields must be specified
				EnableWindow(GetDlgItem(GetParent(hwnd), IDOK),
					
					account.szDisplayName[0] != _T('\0') &&
					account.szHost[0] != _T('\0') &&
					account.szLogin[0] != _T('\0') &&
					account.szPassword[0] != _T('\0')
					);
			}
			break;			
	}
	return 0;
}

static bool
Accounts_AddAccountDlg(HWND hwnd)
{
	PROPSHEETPAGE psp[2];
	PROPSHEETHEADER psh;
	
	// init new account information
	ZeroMemory(&account, sizeof(MAIL_ACCOUNT));
	account.protocol = p_POP3;
	account.nPort = WBIFF_DEFAULT_PROTOCOL_POP3_PORT;
	account.nTimeout = 90;
	
	psp[0].pszTemplate = MAKEINTRESOURCE(IDD_ACCOUNT_PROPERTIES);
	psp[0].dwSize = sizeof(PROPSHEETPAGE);
	psp[0].dwFlags = PSP_DEFAULT | PSP_HASHELP;
	psp[0].hInstance = hInstance;
	psp[0].pfnDlgProc = Accounts_AccountPropertiesDlgProc;
	
	psp[1].pszTemplate = MAKEINTRESOURCE(IDD_ACCOUNT_PROPERTIES_ADV);
	psp[1].dwSize = sizeof(PROPSHEETPAGE);
	psp[1].dwFlags = PSP_DEFAULT | PSP_HASHELP;
	psp[1].hInstance = hInstance;
	psp[1].pfnDlgProc = Accounts_AccountPropertiesAdvDlgProc;
	
	psh.dwSize = sizeof(PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW;
	psh.hwndParent = hwnd;
	psh.hInstance = hInstance;
	psh.pszCaption = _T("Add Account");
	psh.nPages = sizeof(psp) / sizeof(PROPSHEETPAGE);
	psh.ppsp = (LPPROPSHEETPAGE) &psp;
	psh.nStartPage = 0;
	
	switch (PropertySheet(&psh)) {
		
		case IDOK:
			Accounts_InsertAccount(hwnd, &account);
			return true;
			break;
	}
	
	return false;
}

static bool
Accounts_RemoveAccountDlg(HWND hwnd)
{
	const HWND hwndList = GetDlgItem(hwnd, IDC_ACCOUNTS_LIST);
	int i;
	
	while ((i = ListView_GetNextItem(hwndList, -1, LVNI_SELECTED)) != -1) {
		
		ListView_DeleteItem(hwndList, i);
	}
	return true;
}

static bool
Accounts_ModifyAccountDlg(HWND hwnd)
{
	// PRE-CONDITION: Only one item in the list may be selected
	const HWND hwndList = GetDlgItem(hwnd, IDC_ACCOUNTS_LIST);
	int result = ListView_GetNextItem(hwndList, -1, LVNI_SELECTED);
	
	if (result != -1) {
		
		LV_ITEM lvi;
		MAIL_PACCOUNT p;
		int item = result;
		PROPSHEETPAGE psp[2];
		PROPSHEETHEADER psh;
		
		lvi.mask = LVIF_PARAM;
		lvi.iItem = item;
		lvi.iSubItem = 0;
		
		SendDlgItemMessage(hwnd, IDC_ACCOUNTS_LIST, LVM_GETITEM, 0, (LPARAM) &lvi);
		p = (MAIL_PACCOUNT) lvi.lParam;
		
		// copy the account information into temporary buffer
		Accounts_CopyAccount(&account, p);
		
		// setup property sheet
		psp[0].pszTemplate = MAKEINTRESOURCE(IDD_ACCOUNT_PROPERTIES);
		psp[0].dwSize = sizeof(PROPSHEETPAGE);
		psp[0].dwFlags = PSP_DEFAULT | PSP_HASHELP;
		psp[0].hInstance = hInstance;
		psp[0].pfnDlgProc = Accounts_AccountPropertiesDlgProc;
		
		psp[1].pszTemplate = MAKEINTRESOURCE(IDD_ACCOUNT_PROPERTIES_ADV);
		psp[1].dwSize = sizeof(PROPSHEETPAGE);
		psp[1].dwFlags = PSP_DEFAULT | PSP_HASHELP;
		psp[1].hInstance = hInstance;
		psp[1].pfnDlgProc = Accounts_AccountPropertiesAdvDlgProc;
		
		psh.dwSize = sizeof(PROPSHEETHEADER);
		psh.dwFlags = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW;
		psh.hwndParent = hwnd;
		psh.hInstance = hInstance;
		psh.pszCaption = _T("Modify Account");
		psh.nPages = sizeof(psp) / sizeof(PROPSHEETPAGE);
		psh.ppsp = (LPPROPSHEETPAGE) &psp;
		psh.nStartPage = 0;
		
		switch (PropertySheet(&psh)) {
			
		case IDOK:
			// copy the information back to the 'p' pointer
			Accounts_CopyAccount(p, &account);
			
			lvi.mask = LVIF_TEXT;
			lvi.iItem = item;
			lvi.pszText = p->szDisplayName;
			lvi.cchTextMax = MAIL_ACCOUNT_FIELD_LENGTH;
			lvi.iSubItem = 0;
			
			SendMessage(hwndList, LVM_SETITEM, 0, (LPARAM) &lvi);
			
			lvi.iSubItem = 1;
			lvi.pszText = p->protocol == p_POP3 ? _T("POP3") : _T("IMAP4");
			
			SendMessage(hwndList, LVM_SETITEM, 0, (LPARAM) &lvi);
			return true;
			break;
		}
	}
	
	return false;
}

static BOOL _stdcall
ProcessWindowMessage_Accounts(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		
		case WM_INITDIALOG:
			SendDlgItemMessage(hwnd, IDC_SPIN1, UDM_SETRANGE, 0, MAKELONG(9999, 1));
			SendDlgItemMessage(hwnd, IDC_SPIN2, UDM_SETRANGE, 0, MAKELONG(9999, 1));
			SetDlgItemInt(hwnd, IDC_ACCOUNTS_UPDATE_INTERVAL, temp_settings.nUpdateInterval, FALSE);
			SetDlgItemInt(hwnd, IDC_ACCOUNTS_MIN_LIMIT, temp_settings.nMinLimit, FALSE);
			EnableWindow(GetDlgItem(hwnd, IDC_ACCOUNTS_REMOVE), FALSE);
			EnableWindow(GetDlgItem(hwnd, IDC_ACCOUNTS_MODIFY), FALSE);
			Accounts_PopulateList(hwnd);
			return TRUE;
			break;
			
		case WM_NOTIFY:
			{
				NMHDR* pNMHDR = (NMHDR*) lParam;
				
				if (pNMHDR->code == PSN_SETACTIVE)
					temp_settings.nLastActivePage = pageAccounts;
				else if (pNMHDR->idFrom == IDC_ACCOUNTS_LIST) {
					
					if (pNMHDR->code == LVN_DELETEITEM) {
						
						LPNMLISTVIEW p = (LPNMLISTVIEW) lParam;
						MAIL_PACCOUNT pp = (MAIL_PACCOUNT) p->lParam;
						wb_free(pp);
					}
					else {
						
						int selected;
						
						selected = SendDlgItemMessage(hwnd, IDC_ACCOUNTS_LIST, LVM_GETSELECTEDCOUNT, 0, 0);
						EnableWindow(GetDlgItem(hwnd, IDC_ACCOUNTS_REMOVE), selected);
						EnableWindow(GetDlgItem(hwnd, IDC_ACCOUNTS_MODIFY), selected == 1);
					}
				}
			}
			break;
			
		case WM_DESTROY:
			Accounts_Update(hwnd);
			break;
			
		case WM_COMMAND:
			if (HIWORD(wParam) == BN_CLICKED) {
				
				switch (LOWORD(wParam)) {
					
				case IDC_ACCOUNTS_ADD:
					if (Accounts_AddAccountDlg(hwnd))
						bAccountsChanged = true;
					break;
					
				case IDC_ACCOUNTS_REMOVE:
					if (Accounts_RemoveAccountDlg(hwnd))
						bAccountsChanged = true;
					break;
					
				case IDC_ACCOUNTS_MODIFY:
					if (Accounts_ModifyAccountDlg(hwnd))
						bAccountsChanged = true;
					break;
				}
			}
			break;
	}
	
	return 0;
}

/*
===========================
General Page
===========================
*/
static void
General_Update(HWND hwnd, bool bFinalUpdate)
{
	temp_settings.bPopupWindow = IsDlgButtonChecked(hwnd, IDC_GENERAL_POPUP_WINDOW) == BST_CHECKED;
	temp_settings.bFlashWindow = IsDlgButtonChecked(hwnd, IDC_GENERAL_FLASH_WINDOW) == BST_CHECKED;
	temp_settings.bFlashTray = IsDlgButtonChecked(hwnd, IDC_GENERAL_FLASH_TRAY) == BST_CHECKED;
	temp_settings.bRunClient = IsDlgButtonChecked(hwnd, IDC_GENERAL_RUN_CLIENT) == BST_CHECKED;
	temp_settings.bPlayWave = IsDlgButtonChecked(hwnd, IDC_GENERAL_PLAY_WAVE) == BST_CHECKED;
	temp_settings.bStartMinimized = IsDlgButtonChecked(hwnd, IDC_GENERAL_START_MINIMIZED) == BST_CHECKED;
	temp_settings.bEnableToolTips_Main = IsDlgButtonChecked(hwnd, IDC_GENERAL_ENABLE_TOOLTIPS) == BST_CHECKED;
	
	if (bFinalUpdate) {
		
		GetDlgItemText(hwnd, IDC_GENERAL_WAVE_FILE, temp_settings.szWaveFile, MAX_PATH);
		GetDlgItemText(hwnd, IDC_GENERAL_EMAIL_CLIENT, temp_settings.szEmailClient, MAX_PATH);
		GetDlgItemText(hwnd, IDC_GENERAL_EMAIL_CLIENT_ARGS, temp_settings.szEmailClientArgs, MAX_PATH);
	}
}

static void
General_Enable(HWND hwnd)
{
	EnableWindow(GetDlgItem(hwnd, IDC_GENERAL_RUN_CLIENT), GetWindowTextLength(GetDlgItem(hwnd, IDC_GENERAL_EMAIL_CLIENT)));
	EnableWindow(GetDlgItem(hwnd, IDC_GENERAL_WAVE_FILE), temp_settings.bPlayWave);
	EnableWindow(GetDlgItem(hwnd, IDC_GENERAL_PLAY_WAVE_BROWSE), temp_settings.bPlayWave);
}

static BOOL _stdcall
ProcessWindowMessage_General(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		
		case WM_INITDIALOG:
			if (temp_settings.bPopupWindow)
				CheckDlgButton(hwnd, IDC_GENERAL_POPUP_WINDOW, BST_CHECKED);
			if (temp_settings.bFlashWindow)
				CheckDlgButton(hwnd, IDC_GENERAL_FLASH_WINDOW, BST_CHECKED);
			if (temp_settings.bFlashTray)
				CheckDlgButton(hwnd, IDC_GENERAL_FLASH_TRAY, BST_CHECKED);
			if (temp_settings.bRunClient)
				CheckDlgButton(hwnd, IDC_GENERAL_RUN_CLIENT, BST_CHECKED);
			if (temp_settings.bPlayWave && temp_settings.szWaveFile[0] != _T('\0'))
				CheckDlgButton(hwnd, IDC_GENERAL_PLAY_WAVE, BST_CHECKED);
			if (temp_settings.bStartMinimized)
				CheckDlgButton(hwnd, IDC_GENERAL_START_MINIMIZED, BST_CHECKED);
			if (temp_settings.bEnableToolTips_Main)
				CheckDlgButton(hwnd, IDC_GENERAL_ENABLE_TOOLTIPS, BST_CHECKED);
			SetDlgItemText(hwnd, IDC_GENERAL_EMAIL_CLIENT, temp_settings.szEmailClient);
			SetDlgItemText(hwnd, IDC_GENERAL_EMAIL_CLIENT_ARGS, temp_settings.szEmailClientArgs);
			SetDlgItemText(hwnd, IDC_GENERAL_WAVE_FILE, temp_settings.szWaveFile);
			return TRUE;
			break;
			
		case WM_NOTIFY:
			{
				NMHDR* pNMHDR = (NMHDR*) lParam;
				if (pNMHDR->code == PSN_SETACTIVE) {
					
					General_Enable(hwnd);
					temp_settings.nLastActivePage = pageGeneral;
				}
				else if (pNMHDR->code == PSN_KILLACTIVE)
					General_Update(hwnd, TRUE);
			}
			break;
			
		case WM_DESTROY:
			General_Update(hwnd, TRUE);
			break;
			
		case WM_COMMAND:
			if (HIWORD(wParam) == BN_CLICKED || HIWORD(wParam) == EN_CHANGE) {
				
				switch (LOWORD(wParam)) {
					
				case IDC_GENERAL_PLAY_WAVE_BROWSE:
					{
						static TCHAR szFilter[] = _T("WAVE Files (*.wav)\0*.wav\0All Files (*.*)\0*.*\0\0");
						TCHAR szFile[MAX_PATH + 1];
						
						GetDlgItemText(hwnd, IDC_GENERAL_WAVE_FILE, szFile, MAX_PATH);
						if (BrowseForFileName(hwnd, szFile, sizeof(szFile), szFilter)) {
							
							SetDlgItemText(hwnd, IDC_GENERAL_WAVE_FILE, szFile);
							SetFocus(GetDlgItem(hwnd, IDC_GENERAL_WAVE_FILE));
							SendDlgItemMessage(hwnd, IDC_GENERAL_WAVE_FILE, EM_SETSEL, 0, -1);
							
							lstrcpy(temp_settings.szWaveFile, szFile);
						}
					}
					break;
					
				case IDC_GENERAL_EMAIL_CLIENT_BROWSE:
					{
						static TCHAR szFilter[] = _T("Program Files (*.exe)\0*.exe\0All Files (*.*)\0*.*\0\0");
						TCHAR szFile[MAX_PATH + 1];
						
						GetDlgItemText(hwnd, IDC_GENERAL_EMAIL_CLIENT, szFile, MAX_PATH);
						if (BrowseForFileName(hwnd, szFile, sizeof(szFile), szFilter)) {
							
							SetDlgItemText(hwnd, IDC_GENERAL_EMAIL_CLIENT, szFile);
							SetFocus(GetDlgItem(hwnd, IDC_GENERAL_EMAIL_CLIENT));
							SendDlgItemMessage(hwnd, IDC_GENERAL_EMAIL_CLIENT, EM_SETSEL, 0, -1);
							
							lstrcpy(temp_settings.szEmailClient, szFile);
						}
					}
					break;
					
				default:
					General_Update(hwnd, FALSE);
					break;
				}
				General_Enable(hwnd);
			}
			break;
	}
	
	return 0;
}

/*
===========================
Advanced Page
===========================
*/
static void 
Advanced_Update(HWND hwnd)
{
	int nRadio = GetCheckedRadioButton(hwnd, IDC_ADV_PROCESS_NORMAL, IDC_ADV_PROCESS_LOW);
	temp_settings.nPriority = nRadio - IDC_ADV_PROCESS_NORMAL;
	
	temp_settings.bStartupCheck = IsDlgButtonChecked(hwnd, IDC_ADV_CHECK_ON_STARTUP) == BST_CHECKED;
	temp_settings.bStartEnabled = IsDlgButtonChecked(hwnd, IDC_ADV_START_ENABLED) == BST_CHECKED;
	temp_settings.bShowEnabledStatus = IsDlgButtonChecked(hwnd, IDC_ADV_SHOW_ENABLED_STATUS) == BST_CHECKED;
	temp_settings.bAOT = IsDlgButtonChecked(hwnd, IDC_ADV_AOT) == BST_CHECKED;
	temp_settings.bCheckInParallel = IsDlgButtonChecked(hwnd, IDC_ADV_CHECK_IN_PARALLEL) == BST_CHECKED;
	temp_settings.bIgnoreErrors = IsDlgButtonChecked(hwnd, IDC_ADV_IGNORE_ERRORS) == BST_CHECKED;
	temp_settings.bLoadOnWindowsStartup = IsDlgButtonChecked(hwnd, IDC_ADV_LOAD_ON_STARTUP) == BST_CHECKED;
}

static BOOL _stdcall
ProcessWindowMessage_Advanced(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		
		case WM_INITDIALOG:
			CheckRadioButton(hwnd, IDC_ADV_PROCESS_NORMAL, IDC_ADV_PROCESS_LOW, IDC_ADV_PROCESS_NORMAL + temp_settings.nPriority);
			
			if (temp_settings.bStartupCheck)
				CheckDlgButton(hwnd, IDC_ADV_CHECK_ON_STARTUP, BST_CHECKED);
			if (temp_settings.bStartEnabled)
				CheckDlgButton(hwnd, IDC_ADV_START_ENABLED, BST_CHECKED);
			if (temp_settings.bShowEnabledStatus)
				CheckDlgButton(hwnd, IDC_ADV_SHOW_ENABLED_STATUS, BST_CHECKED);
			if (temp_settings.bAOT)
				CheckDlgButton(hwnd, IDC_ADV_AOT, BST_CHECKED);
			if (temp_settings.bCheckInParallel)
				CheckDlgButton(hwnd, IDC_ADV_CHECK_IN_PARALLEL, BST_CHECKED);
			if (temp_settings.bIgnoreErrors)
				CheckDlgButton(hwnd, IDC_ADV_IGNORE_ERRORS, BST_CHECKED);	
			if (temp_settings.bLoadOnWindowsStartup)
				CheckDlgButton(hwnd, IDC_ADV_LOAD_ON_STARTUP, BST_CHECKED);
			return TRUE;
			break;
			
		case WM_DESTROY:
			Advanced_Update(hwnd);
			break;
			
		case WM_NOTIFY:
			{
				NMHDR* pNMHDR = (NMHDR*) lParam;
				if (pNMHDR->code == PSN_SETACTIVE)
					temp_settings.nLastActivePage = pageAdvanced;
			}
			break;
			
		case WM_COMMAND:
			if (HIWORD(wParam) == BN_CLICKED)
				Advanced_Update(hwnd);
			break;
	}
	
	return 0;
}

/*
===========================
Headers Page
===========================
*/
static void 
Headers_Update(HWND hwnd)
{
	temp_settings.bObtainEmailHeaders = IsDlgButtonChecked(hwnd, IDC_HDR_OBTAIN) == BST_CHECKED;
	temp_settings.bAutoDisplaySummary = IsDlgButtonChecked(hwnd, IDC_HDR_AUTO_DISPLAY) == BST_CHECKED;
	temp_settings.bForceSummaryDisplay = IsDlgButtonChecked(hwnd, IDC_HDR_FORCE_AUTO_DISPLAY) == BST_CHECKED;
	temp_settings.bAutoCloseSummary = IsDlgButtonChecked(hwnd, IDC_HDR_AUTO_CLOSE_SUMMARY) == BST_CHECKED;
	temp_settings.bEnableToolTips_Summary = IsDlgButtonChecked(hwnd, IDC_HDR_ENABLE_TOOLTIPS) == BST_CHECKED;
	temp_settings.bUseWin32Colors = IsDlgButtonChecked(hwnd, IDC_HDR_WIN32_COLORS) == BST_CHECKED;
	temp_settings.bDisplayNonZeroAccountFirst = IsDlgButtonChecked(hwnd, IDC_HDR_JUMP_TO_FIRST_NZ) == BST_CHECKED;
}

static void 
Headers_Enable(HWND hwnd)
{
	EnableWindow(GetDlgItem(hwnd, IDC_HDR_AUTO_DISPLAY), temp_settings.bObtainEmailHeaders);
	EnableWindow(GetDlgItem(hwnd, IDC_HDR_FORCE_AUTO_DISPLAY), temp_settings.bObtainEmailHeaders && temp_settings.bAutoDisplaySummary);
	EnableWindow(GetDlgItem(hwnd, IDC_HDR_AUTO_CLOSE_SUMMARY), temp_settings.bObtainEmailHeaders);
	EnableWindow(GetDlgItem(hwnd, IDC_HDR_ENABLE_TOOLTIPS), temp_settings.bObtainEmailHeaders);
	EnableWindow(GetDlgItem(hwnd, IDC_HDR_JUMP_TO_FIRST_NZ), temp_settings.bObtainEmailHeaders);
	EnableWindow(GetDlgItem(hwnd, IDC_HDR_WIN32_COLORS), temp_settings.bObtainEmailHeaders);
	EnableWindow(GetDlgItem(hwnd, IDC_HDR_FONT_COLOR), temp_settings.bObtainEmailHeaders && !temp_settings.bUseWin32Colors);
	EnableWindow(GetDlgItem(hwnd, IDC_HDR_BACK_COLOR), temp_settings.bObtainEmailHeaders && !temp_settings.bUseWin32Colors);
	EnableWindow(GetDlgItem(hwnd, IDC_HDR_WIN32_COLORS_FONT), temp_settings.bObtainEmailHeaders && !temp_settings.bUseWin32Colors);
	EnableWindow(GetDlgItem(hwnd, IDC_HDR_WIN32_COLORS_BACKGROUND), temp_settings.bObtainEmailHeaders && !temp_settings.bUseWin32Colors);
}

static BOOL _stdcall
ProcessWindowMessage_Headers(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		
		case WM_INITDIALOG:
			if (temp_settings.bObtainEmailHeaders)
				CheckDlgButton(hwnd, IDC_HDR_OBTAIN, BST_CHECKED);
			if (temp_settings.bAutoDisplaySummary)
				CheckDlgButton(hwnd, IDC_HDR_AUTO_DISPLAY, BST_CHECKED);
			if (temp_settings.bForceSummaryDisplay)
				CheckDlgButton(hwnd, IDC_HDR_FORCE_AUTO_DISPLAY, BST_CHECKED);
			if (temp_settings.bAutoCloseSummary)
				CheckDlgButton(hwnd, IDC_HDR_AUTO_CLOSE_SUMMARY, BST_CHECKED);
			if (temp_settings.bEnableToolTips_Summary)
				CheckDlgButton(hwnd, IDC_HDR_ENABLE_TOOLTIPS, BST_CHECKED);
			if (temp_settings.bUseWin32Colors)
				CheckDlgButton(hwnd, IDC_HDR_WIN32_COLORS, BST_CHECKED);
			if (temp_settings.bDisplayNonZeroAccountFirst)
				CheckDlgButton(hwnd, IDC_HDR_JUMP_TO_FIRST_NZ, BST_CHECKED);
			Headers_Update(hwnd);
			return TRUE;
			break;
			
		case WM_NOTIFY:
			{
				NMHDR* pNMHDR = (NMHDR*) lParam;
				if (pNMHDR->code == PSN_SETACTIVE)  {
					
					Headers_Enable(hwnd);
					temp_settings.nLastActivePage = pageHeaders;
				}
			}
			break;
			
		case WM_DESTROY:
			Headers_Update(hwnd);
			break;
			
		case WM_DRAWITEM:
			{
				UINT idCtl = (UINT) wParam;
				LPDRAWITEMSTRUCT lpdis = (LPDRAWITEMSTRUCT) lParam;
				
				switch (idCtl) {
					
				case IDC_HDR_FONT_COLOR:
					if (lpdis->itemAction == ODA_DRAWENTIRE) {
						
						HBRUSH brush = CreateSolidBrush(temp_settings.clrFontColor);
						FillRect(lpdis->hDC, &lpdis->rcItem, brush);
						DeleteObject(brush);
					}
					break;
					
				case IDC_HDR_BACK_COLOR:
					if (lpdis->itemAction == ODA_DRAWENTIRE) {
						
						HBRUSH brush = CreateSolidBrush(temp_settings.clrBackColor);
						FillRect(lpdis->hDC, &lpdis->rcItem, brush);
						DeleteObject(brush);
					}
					break;
				}
			}
			break;
			
		case WM_COMMAND:
			if (HIWORD(wParam) == BN_CLICKED) {
				
				switch (LOWORD(wParam)) {
					
				case IDC_HDR_BACK_COLOR:
				case IDC_HDR_FONT_COLOR:
					{
						CHOOSECOLOR cc;
						COLORREF* clr = LOWORD(wParam) == IDC_HDR_BACK_COLOR ? &temp_settings.clrBackColor : &temp_settings.clrFontColor;
						
						static COLORREF clrCustomData[16] = { 0, };
						
						int i;
						for (i = 0; i < 16; i++)
							clrCustomData[i] = (RGB(255 - i * 10, i * 10, i * 10));
						
						ZeroMemory(&cc, sizeof(CHOOSECOLOR));
						cc.lStructSize = sizeof(CHOOSECOLOR);
						cc.hwndOwner = hwnd;
						cc.Flags = CC_RGBINIT | CC_ANYCOLOR;
						cc.rgbResult = *clr;
						cc.lpCustColors = clrCustomData;
						
						if (ChooseColor(&cc)) {
							
							HWND hwndCC = GetDlgItem(hwnd, LOWORD(wParam) == IDC_HDR_BACK_COLOR ? IDC_HDR_BACK_COLOR : IDC_HDR_FONT_COLOR);
							
							*clr = cc.rgbResult;
							InvalidateRect(hwndCC, NULL, TRUE);
							UpdateWindow(hwndCC);
						}
					}
					break;					
					
				default:
					Headers_Update(hwnd);
					Headers_Enable(hwnd);
					break;
					
				}
			}
			break;
	}
	
	return 0;
}

/*
===========================
Animation Page
===========================
*/
static void 
Animation_Update(HWND hwnd)
{
	temp_settings.bCheckMailAnimation = IsDlgButtonChecked(hwnd, IDC_ANIMATION_CHECK_MAIL) == BST_CHECKED;
	temp_settings.bContinueAnimationTray = IsDlgButtonChecked(hwnd, IDC_ANIMATION_CONTINUE_FLASHING_TRAY) == BST_CHECKED;
	temp_settings.bContinueAnimationWindow = IsDlgButtonChecked(hwnd, IDC_ANIMATION_CONTINUE_FLASHING_WINDOW) == BST_CHECKED;
	temp_settings.bFlashFlagDownOnly = IsDlgButtonChecked(hwnd, IDC_ANIMATION_FLASH_FLAG_DOWN_ONLY) == BST_CHECKED;
	temp_settings.nAnimationIterations = GetDlgItemInt(hwnd, IDC_ANIMATION_ITERATIONS, NULL, FALSE);
	temp_settings.nAnimationDelay = GetDlgItemInt(hwnd, IDC_ANIMATION_DELAY, NULL, FALSE);
	temp_settings.bHurryCheckMailAnimation = IsDlgButtonChecked(hwnd, IDC_ANIMATION_CHECK_MAIL_HURRY) == BST_CHECKED;
	
	if (temp_settings.nAnimationIterations > 400)
		temp_settings.nAnimationIterations = 400;
	if (temp_settings.nAnimationDelay > 5000)
		temp_settings.nAnimationDelay = 5000;
}

static void
Animation_Enable(HWND hwnd)
{
	EnableWindow(GetDlgItem(hwnd, IDC_ANIMATION_CONTINUE_FLASHING_TRAY), temp_settings.bFlashTray);
	EnableWindow(GetDlgItem(hwnd, IDC_ANIMATION_FLASH_FLAG_DOWN_ONLY), temp_settings.bFlashTray);
	EnableWindow(GetDlgItem(hwnd, IDC_ANIMATION_CONTINUE_FLASHING_WINDOW), temp_settings.bFlashWindow);
	EnableWindow(GetDlgItem(hwnd, IDC_ANIMATION_CHECK_MAIL_HURRY), temp_settings.bCheckMailAnimation);
	
	EnableWindow(GetDlgItem(hwnd, IDC_ANIMATION_ITERATIONS), !(temp_settings.bContinueAnimationTray && temp_settings.bContinueAnimationWindow) && (temp_settings.bFlashWindow || temp_settings.bFlashTray));
	EnableWindow(GetDlgItem(hwnd, IDC_SPIN1), !(temp_settings.bContinueAnimationTray && temp_settings.bContinueAnimationWindow));
	
}

static BOOL _stdcall
ProcessWindowMessage_Animation(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		
		case WM_INITDIALOG:
			{
				UDACCEL uda;
				
				uda.nSec = 0;
				uda.nInc = 100;
				
				SendDlgItemMessage(hwnd, IDC_SPIN1, UDM_SETRANGE, 0, MAKELONG(500, 1));
				SendDlgItemMessage(hwnd, IDC_SPIN2, UDM_SETRANGE, 0, MAKELONG(5000, 100));
				SendDlgItemMessage(hwnd, IDC_SPIN2, UDM_SETACCEL, 1, (LPARAM) &uda);
				if (temp_settings.bCheckMailAnimation)
					CheckDlgButton(hwnd, IDC_ANIMATION_CHECK_MAIL, BST_CHECKED);
				if (temp_settings.bContinueAnimationTray)
					CheckDlgButton(hwnd, IDC_ANIMATION_CONTINUE_FLASHING_TRAY, BST_CHECKED);
				if (temp_settings.bContinueAnimationWindow)
					CheckDlgButton(hwnd, IDC_ANIMATION_CONTINUE_FLASHING_WINDOW, BST_CHECKED);
				if (temp_settings.bHurryCheckMailAnimation)
					CheckDlgButton(hwnd, IDC_ANIMATION_CHECK_MAIL_HURRY, BST_CHECKED);
				if (temp_settings.bFlashFlagDownOnly)
					CheckDlgButton(hwnd, IDC_ANIMATION_FLASH_FLAG_DOWN_ONLY, BST_CHECKED);
				
				SetDlgItemInt(hwnd, IDC_ANIMATION_ITERATIONS, temp_settings.nAnimationIterations, FALSE);
				SetDlgItemInt(hwnd, IDC_ANIMATION_DELAY, temp_settings.nAnimationDelay, FALSE);
				Animation_Enable(hwnd);
				return TRUE;
			}
			break;
			
		case WM_NOTIFY:
			{
				NMHDR* pNMHDR = (NMHDR*) lParam;
				if (pNMHDR->code == PSN_SETACTIVE) {
					
					Animation_Enable(hwnd);
					temp_settings.nLastActivePage = pageAnimation;
				}
			}
			break;
			
		case WM_COMMAND:
			if (HIWORD(wParam) == BN_CLICKED) {
				
				if (LOWORD(wParam) == IDC_ANIMATION_RESTORE_DEFAULTS) {
					
					// set some default values
					temp_settings.nAnimationIterations = WBIFF_DEFAULT_ANIMATION_ITERATIONS;
					temp_settings.nAnimationDelay = WBIFF_DEFAULT_ANIMATION_DELAY;
					
					SetDlgItemInt(hwnd, IDC_ANIMATION_ITERATIONS, temp_settings.nAnimationIterations, FALSE);
					SetDlgItemInt(hwnd, IDC_ANIMATION_DELAY, temp_settings.nAnimationDelay, FALSE);
					CheckDlgButton(hwnd, IDC_ANIMATION_CHECK_MAIL_HURRY, BST_CHECKED);
				}
				else {
					
					Animation_Update(hwnd);
					Animation_Enable(hwnd);
				}
			}
			break;
			
		case WM_DESTROY:
			Animation_Update(hwnd);
			break;
	}
	
	return 0;
}

/*
===========================
HotKeys Page
===========================

	Hotkeys are stored in the LOWORD portion of a DWORD value (that is,
	in the lower 16 bits) such that the VIRTUAL-KEY code
	is stored in the low byte and the MODIFIERS are stored in the
	high byte of that LOWORD. Bits 16-31 are unused.
*/
static void
HotKeys_ToSDKFormat(WORD* bad)
{
	// convert from bad common control format to good
	// SDK format
	
	BYTE bad_byte = HIBYTE(*bad);
	BYTE good_byte = 0;
	
	if (bad_byte & HOTKEYF_SHIFT)
		good_byte |= MOD_SHIFT;
	if (bad_byte & HOTKEYF_CONTROL)
		good_byte |= MOD_CONTROL;
	if (bad_byte & HOTKEYF_ALT)
		good_byte |= MOD_ALT;
	if (bad_byte & HOTKEYF_EXT)
		good_byte |= MOD_WIN;
	
	*bad = LOBYTE(*bad) + (good_byte << 8);
}

static void
HotKeys_ToCommCtrlFormat(WORD* good)
{
	// convert good api format to bad common control format
	// we only need to modify the HIBYTE of the word-value
	
	BYTE good_byte = HIBYTE(*good);
	BYTE bad_byte = 0;
	
	if (good_byte & MOD_ALT)
		bad_byte |= HOTKEYF_ALT;
	if (good_byte & MOD_CONTROL)
		bad_byte |= HOTKEYF_CONTROL;
	if (good_byte & MOD_SHIFT)
		bad_byte |= HOTKEYF_SHIFT;
	if (good_byte & MOD_WIN)
		bad_byte |= HOTKEYF_EXT;
	
	*good = LOBYTE(*good) + (bad_byte << 8);
}

static void 
HotKeys_SetHotKey(HWND hwnd, DWORD wValue, int nID)
{
	WORD w = LOWORD(wValue);
	
	// convert this value to the screwed up Common Control format
	HotKeys_ToCommCtrlFormat(&w);
	
	SendDlgItemMessage(hwnd, nID, HKM_SETHOTKEY, w, 0);
}

static void
HotKeys_GetHotKey(HWND hwnd, DWORD* wValue, int nID)
{
	WORD w = (WORD) SendDlgItemMessage(hwnd, nID, HKM_GETHOTKEY, 0, 0);
	
	HotKeys_ToSDKFormat(&w);
	
	*wValue = 0;
	*wValue = w;	
}

static void
HotKeys_Update(HWND hwnd)
{
	HotKeys_GetHotKey(hwnd, &temp_settings.hk_CheckNow, IDC_HOTKEY_CHECK_NOW);
	HotKeys_GetHotKey(hwnd, &temp_settings.hk_ToggleMain, IDC_HOTKEY_TOGGLE_MAIN);
	HotKeys_GetHotKey(hwnd, &temp_settings.hk_ToggleSummary, IDC_HOTKEY_TOGGLE_SUMMARY);
	HotKeys_GetHotKey(hwnd, &temp_settings.hk_RunClient, IDC_HOTKEY_RUN_CLIENT);
	HotKeys_GetHotKey(hwnd, &temp_settings.hk_ClearFlag, IDC_HOTKEY_CLEAR_FLAG);
	temp_settings.bQuickMinimize = IsDlgButtonChecked(hwnd, IDC_HOTKEY_QUICK_MINIMIZE) == BST_CHECKED;
	temp_settings.bEnableSummaryArrowKeys = IsDlgButtonChecked(hwnd, IDC_HOTKEY_SUMMARY_ARROW_KEYS) == BST_CHECKED;
}

static void
HotKeys_Enable(HWND hwnd)
{
	EnableWindow(GetDlgItem(hwnd, IDC_HOTKEY_RUN_CLIENT), temp_settings.szEmailClient[0] != 0);
	EnableWindow(GetDlgItem(hwnd, IDC_HOTKEY_TOGGLE_SUMMARY), temp_settings.bObtainEmailHeaders);
	EnableWindow(GetDlgItem(hwnd, IDC_HOTKEY_SUMMARY_ARROW_KEYS), temp_settings.bObtainEmailHeaders);
}

static BOOL _stdcall
ProcessWindowMessage_HotKeys(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		
		case WM_INITDIALOG:
			return TRUE;
			break;
			
		case WM_NOTIFY:
			{
				NMHDR* pNMHDR = (NMHDR*) lParam;
				if (pNMHDR->code == PSN_SETACTIVE) {
					
					HotKeys_SetHotKey(hwnd, temp_settings.hk_CheckNow, IDC_HOTKEY_CHECK_NOW);
					HotKeys_SetHotKey(hwnd, temp_settings.hk_ToggleMain, IDC_HOTKEY_TOGGLE_MAIN);
					HotKeys_SetHotKey(hwnd, temp_settings.hk_ToggleSummary, IDC_HOTKEY_TOGGLE_SUMMARY);
					HotKeys_SetHotKey(hwnd, temp_settings.hk_RunClient, IDC_HOTKEY_RUN_CLIENT);
					HotKeys_SetHotKey(hwnd, temp_settings.hk_ClearFlag, IDC_HOTKEY_CLEAR_FLAG);
					if (temp_settings.bQuickMinimize)
						CheckDlgButton(hwnd, IDC_HOTKEY_QUICK_MINIMIZE, BST_CHECKED);
					if (temp_settings.bEnableSummaryArrowKeys)
						CheckDlgButton(hwnd, IDC_HOTKEY_SUMMARY_ARROW_KEYS, BST_CHECKED);
					
					HotKeys_Enable(hwnd);
					temp_settings.nLastActivePage = pageHotKeys;
				}
				else if (pNMHDR->code == PSN_KILLACTIVE) {
					
					HotKeys_Update(hwnd);
				}
			}
			break;
	}
	
	return 0;
}

/*
===========================
Mouse Page
===========================
*/
static void
Mouse_Enable(HWND hwnd)
{
	EnableWindow(GetDlgItem(hwnd, IDC_SMOUSE_RUN_CLIENT), temp_settings.szEmailClient[0] != 0);
	EnableWindow(GetDlgItem(hwnd, IDC_SMOUSE_TOGGLE_SUMMARY), temp_settings.bObtainEmailHeaders);
	EnableWindow(GetDlgItem(hwnd, IDC_MOUSE_EDCA_RUN_CLIENT), temp_settings.szEmailClient[0] != 0);
	EnableWindow(GetDlgItem(hwnd, IDC_MOUSE_EDCA_TOGGLE_SUMMARY), temp_settings.bObtainEmailHeaders);
	
	// if the selected option is invalid (i.e. "Run client" is selected but 
	// no client is specified, revert to "Do nothing" option)
	if ((IsDlgButtonChecked(hwnd, IDC_SMOUSE_RUN_CLIENT) && (temp_settings.szEmailClient[0] == _T('\0'))) || (IsDlgButtonChecked(hwnd, IDC_SMOUSE_TOGGLE_SUMMARY) && !temp_settings.bObtainEmailHeaders))
		CheckRadioButton(hwnd, IDC_SMOUSE_RUN_CLIENT, IDC_SMOUSE_DO_NOTHING, IDC_SMOUSE_DO_NOTHING);
	
	if ((IsDlgButtonChecked(hwnd, IDC_MOUSE_EDCA_RUN_CLIENT) && (temp_settings.szEmailClient[0] == _T('\0')) || (IsDlgButtonChecked(hwnd, IDC_MOUSE_EDCA_TOGGLE_SUMMARY) && !temp_settings.bObtainEmailHeaders)))
		CheckRadioButton(hwnd, IDC_MOUSE_EDCA_RUN_CLIENT, IDC_MOUSE_EDCA_DO_NOTHING, IDC_MOUSE_EDCA_DO_NOTHING);
}

static void
Mouse_Update(HWND hwnd)
{
	temp_settings.eDca = GetCheckedRadioButton(hwnd, IDC_MOUSE_EDCA_RUN_CLIENT, IDC_MOUSE_EDCA_DO_NOTHING);
	temp_settings.eDca -= IDC_MOUSE_EDCA_RUN_CLIENT;
	temp_settings.eSDca = GetCheckedRadioButton(hwnd, IDC_SMOUSE_RUN_CLIENT, IDC_SMOUSE_DO_NOTHING);
	temp_settings.eSDca -= IDC_SMOUSE_RUN_CLIENT;
}

static BOOL _stdcall
ProcessWindowMessage_Mouse(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
		
		case WM_INITDIALOG:
			CheckRadioButton(hwnd, IDC_SMOUSE_RUN_CLIENT, IDC_SMOUSE_DO_NOTHING, temp_settings.eSDca + IDC_SMOUSE_RUN_CLIENT);
			CheckRadioButton(hwnd, IDC_MOUSE_EDCA_RUN_CLIENT, IDC_MOUSE_EDCA_DO_NOTHING, temp_settings.eDca + IDC_MOUSE_EDCA_RUN_CLIENT);
			Mouse_Update(hwnd);
			return TRUE;
			break;
			
		case WM_NOTIFY:
			{
				NMHDR* pNMHDR = (NMHDR*) lParam;
				if (pNMHDR->code == PSN_SETACTIVE) {
					
					Mouse_Enable(hwnd);
					temp_settings.nLastActivePage = pageMouse;
				}
			}
			break;
			
		case WM_DESTROY:
			Mouse_Update(hwnd);
			break;
	}
	
	return 0;
}

/*
===========================
Skin Page
===========================
*/
typedef struct tagSKININFO
{
	bool bFontFaceSpecified;
	TCHAR szFullFileName[MAX_PATH + 1];
	TCHAR szFileName[MAX_PATH + 1];
	
} SKININFO, *PSKININFO;

static int 
Skin_ReadInt(LPCTSTR lpSectionName, LPCTSTR lpKeyName, int nDefault, LPCTSTR lpFileName)
{
	return GetPrivateProfileInt(lpSectionName, lpKeyName, nDefault, lpFileName);
}

static void 
Skin_ReadString(LPTSTR lpText, int nCount, LPCTSTR lpSectionName, LPCTSTR lpKeyName, LPCTSTR lpFileName)
{
	lpText[0] = _T('\0');
	GetPrivateProfileString(lpSectionName, lpKeyName, lpText, lpText, nCount, lpFileName);
}

static void
Skin_PopulateList(HWND hwnd)
{
	// the current working directory for the DLL is the same
	// as for the EXE. All skins are in that same dir.
	TCHAR cwd[MAX_PATH + 1], buffer[MAX_PATH + 1], *p;
	WIN32_FIND_DATA fd;
	HANDLE hFile;
	LRESULT lResult;
	int defaultPointer = -1;
	
	cwd[0] = _T('\0');
	p = cwd + GetModuleFileName(NULL, cwd, MAX_PATH);
	while (p >= cwd && *p != _T('\\')) p--;
	*(++p) = _T('\0');
	
	lstrcpy(buffer, cwd);
	lstrcat(buffer, _T("*.wbs"));
	
	EnableWindow(GetDlgItem(hwnd, IDC_SKIN_FONT_SIZE), FALSE);
	if ((hFile = FindFirstFile(buffer, &fd)) != INVALID_HANDLE_VALUE) {
		
		PSKININFO ps;
		int item, selectedItem = -1;
		TCHAR temp[64], addstring[128];
		
		do {
			
			// get data from file (Author, Skin name, font face)
			// then insert into list
			ps = (PSKININFO) wb_malloc(sizeof(SKININFO));
			
			lstrcpy(ps->szFullFileName, cwd);
			lstrcat(ps->szFullFileName, fd.cFileName);
			lstrcpy(ps->szFileName, fd.cFileName);
			
			Skin_ReadString(temp, sizeof(temp), _T("Skin"), _T("FontFaceName"), ps->szFullFileName);
			ps->bFontFaceSpecified = temp[0] != _T('\0');
			
			// get author and skin name -- format these into addstring
			Skin_ReadString(temp, sizeof(temp), _T("Skin"), _T("Name"), ps->szFullFileName);
			lstrcpy(addstring, temp);
			lstrcat(addstring, _T(" - "));
			Skin_ReadString(temp, sizeof(temp), _T("Skin"), _T("Author"), ps->szFullFileName);
			lstrcat(addstring, temp);
			
			item = SendDlgItemMessage(hwnd, IDC_SKIN_LIST, LB_ADDSTRING, 0, (LPARAM) addstring);
			SendDlgItemMessage(hwnd, IDC_SKIN_LIST, LB_SETITEMDATA, item, (LPARAM) ps);
			
			// if current item is the "current skin" then update index pointer
			if (selectedItem == -1) {
				
				if (lstrcmpi(ps->szFileName, _T("default.wbs")) == 0) {
					
					defaultPointer = item;
					if (temp_settings.szSelectedSkin[0] == _T('\0'))
						selectedItem = item;
				}
				else if (lstrcmpi(temp_settings.szSelectedSkin, ps->szFileName) == 0)
					selectedItem = item;
			}
		}
		while (FindNextFile(hFile, &fd));
		
		FindClose(hFile);
		
		ps = NULL;
		if ((lResult = SendDlgItemMessage(hwnd, IDC_SKIN_LIST, LB_GETITEMDATA, (WPARAM) selectedItem, 0)) != LB_ERR) {
			
			// The specified skin (temp_settings.szSelectedSkin) does exist, choose it
			ps = (PSKININFO) lResult;
		}
		else {
			
			// This skin specified in the temp_settings is invalid, just select the first
			// skin in the list
			if (defaultPointer != -1)
				selectedItem = defaultPointer;
			else
				selectedItem = 0;
			
			if ((lResult = SendDlgItemMessage(hwnd, IDC_SKIN_LIST, LB_GETITEMDATA, (WPARAM) selectedItem, 0)) != LB_ERR)
				ps = (PSKININFO) lResult;
		}
		
		if (ps)
			EnableWindow(GetDlgItem(hwnd, IDC_SKIN_FONT_SIZE), ps->bFontFaceSpecified);
		
		SendDlgItemMessage(hwnd, IDC_SKIN_LIST, LB_SETCURSEL, (WPARAM) selectedItem, 0);
	}
}

static BOOL _stdcall
ProcessWindowMessage_Skin(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static HYPERLINK hlTutorial;
	static HYPERLINK hlDownload;
	
	switch (message) {
		
		case WM_INITDIALOG:
			SendDlgItemMessage(hwnd, IDC_SPIN1, UDM_SETRANGE, 0, MAKELONG(64, 6));
			if (temp_settings.bEasyMove)
				CheckDlgButton(hwnd, IDC_SKIN_EASY_MOVE, BST_CHECKED);
			SetDlgItemInt(hwnd, IDC_SKIN_FONT_SIZE, temp_settings.nFontSize, FALSE);
			Skin_PopulateList(hwnd);
			
			// subclass the hyperlinks (both static controls use the same [old] window proc)
			hlTutorial.lpURL = _T("http://guesswork.org/software/wbiff/skindevelop.html");
			hlDownload.lpURL = _T("http://guesswork.org/software/wbiff/skins.html");
			
			hlTutorial.clrVisited =
			hlDownload.clrVisited = RGB(240, 50, 10);
			hlDownload.clrLink =
			hlTutorial.clrLink = RGB(10, 50, 240);
			
			hyperlink_Init(hInstance);
			hyperlink_CreateHyperLink(GetDlgItem(hwnd, IDC_SKIN_TUTORIAL), &hlTutorial);
			hyperlink_CreateHyperLink(GetDlgItem(hwnd, IDC_SKIN_DOWNLOAD), &hlDownload);
			return TRUE;
			break;
			
		case WM_CTLCOLORSTATIC:
			{
				WNDPROC p = (WNDPROC) GetWindowLong((HWND) lParam, GWL_WNDPROC);
				return CallWindowProc(p, (HWND) lParam, message, wParam, lParam);
			}
			break;
			
		case WM_NOTIFY:
			{
				NMHDR* pNMHDR = (NMHDR*) lParam;
				if (pNMHDR->code == PSN_SETACTIVE)
					temp_settings.nLastActivePage = pageSkin;
			}
			break;
			
		case WM_COMMAND:
			if (HIWORD(wParam) == LBN_SELCHANGE) {
				
				// get item data and update the font size selection box
				// and the temp_settings.szSelectedSkin member
				
				int cur = SendMessage((HWND) lParam, LB_GETCURSEL, 0, 0);
				PSKININFO p = (PSKININFO) SendMessage((HWND) lParam, LB_GETITEMDATA, (WPARAM) cur, 0);
				
				lstrcpy(temp_settings.szSelectedSkin, p->szFileName);
				EnableWindow(GetDlgItem(hwnd, IDC_SKIN_FONT_SIZE), p->bFontFaceSpecified);
				InvalidateRect(GetDlgItem(hwnd, IDC_SPIN1), NULL, TRUE);
				UpdateWindow(GetDlgItem(hwnd, IDC_SPIN1));
			}
			break;
			
		case WM_DESTROY:
			{
				int i, count;
				HWND hwndLB = GetDlgItem(hwnd, IDC_SKIN_LIST);
				
				temp_settings.nFontSize = GetDlgItemInt(hwnd, IDC_SKIN_FONT_SIZE, NULL, FALSE);
				temp_settings.bEasyMove = IsDlgButtonChecked(hwnd, IDC_SKIN_EASY_MOVE) == BST_CHECKED;
				
				for (i = 0, count = SendMessage(hwndLB, LB_GETCOUNT, 0, 0); i < count; i++) {
					
					DWORD dw = SendMessage(hwndLB, LB_GETITEMDATA, (WPARAM) i, 0);
					wb_free((LPVOID) dw);
				}
				
				hyperlink_UnInit();
			}
			break;
	}
	
	return 0;
}

// **************************************************************************

void
settings_UnLoad(SETTINGS* pSettings)
{
	if (pSettings->accounts) {
		
		wb_free(pSettings->accounts);
		pSettings->accounts = NULL;
		pSettings->nAccounts = 0;
	}
}

void
settings_Save(SETTINGS* pSettings)
{
	HKEY hKey;
	
	if (RegOpenKeyEx(HKEY_CURRENT_USER, _T("Software\\wbiff!"), 0, KEY_ALL_ACCESS, &hKey) != ERROR_SUCCESS) {
		
		// opening the key failed, create it.
    DWORD dwDisposition;
		
    // create the wbiff! key for future reference
		RegOpenKeyEx(HKEY_CURRENT_USER, _T("Software"), 0, KEY_ALL_ACCESS, &hKey);
    RegCreateKeyEx(hKey, _T("wbiff!"), 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, &dwDisposition);
	}
	
	// ------ NON USER SETTINGS -------------------------------------
	
	regWI(hKey, (BYTE*) &pSettings->nLastActivePage, _T("nLastActivePage"));
	regWI(hKey, (BYTE*) &pSettings->x, _T("x"));
	regWI(hKey, (BYTE*) &pSettings->y, _T("y"));
	
	// ------ ACCOUNTS PAGE -----------------------------------------
	
	regWI(hKey, (BYTE*) &pSettings->nMinLimit, _T("a_nMinLimit"));
	regWI(hKey, (BYTE*) &pSettings->nUpdateInterval, _T("a_nUpdateInterval"));
	regWI(hKey, (BYTE*) &pSettings->nAccounts, _T("a_nAccounts"));
	
	if (pSettings->nAccounts && pSettings->accounts) {
		
		TCHAR szHost[32], szLogin[32], szPassword[32], szProtocol[32], szDisplayName[32], szPort[32], szTimeout[32];
		int i;
		
		for (i = 0; i < pSettings->nAccounts; i++) {
			
			wsprintf(szDisplayName, _T("a_szDisplayName%d"), i);
			wsprintf(szHost, _T("a_szHost%d"), i);
			wsprintf(szLogin, _T("a_szLogin%d"), i);
			wsprintf(szPassword, _T("a_szPassword%d"), i);
			wsprintf(szProtocol, _T("a_nType%d"), i);
			wsprintf(szPort, _T("a_nPort%d"), i);
			wsprintf(szTimeout, _T("a_nTimeout%d"), i);
			
			crypto_Encrypt(WBIFF_WINDOWNAME, pSettings->accounts[i].szPassword);
			
			regWS(hKey, pSettings->accounts[i].szDisplayName, szDisplayName);
			regWS(hKey, pSettings->accounts[i].szHost, szHost);
			regWS(hKey, pSettings->accounts[i].szLogin, szLogin);
			regWS(hKey, pSettings->accounts[i].szPassword, szPassword);
			regWI(hKey, (BYTE*) &pSettings->accounts[i].protocol, szProtocol);
			regWI(hKey, (BYTE*) &pSettings->accounts[i].nPort, szPort);
			regWI(hKey, (BYTE*) &pSettings->accounts[i].nTimeout, szTimeout);
		}
	}
	
	// ------ GENERAL PAGE ------------------------------------------
	
	regWI(hKey, (BYTE*) &pSettings->bFlashTray, _T("bFlashTray"));
	regWI(hKey, (BYTE*) &pSettings->bFlashWindow, _T("bFlashWindow"));
	regWI(hKey, (BYTE*) &pSettings->bRunClient, _T("bRunClient"));
	regWI(hKey, (BYTE*) &pSettings->bPlayWave, _T("bPlayWave"));
	regWI(hKey, (BYTE*) &pSettings->bPopupWindow, _T("bPopupWindow"));
	regWI(hKey, (BYTE*) &pSettings->bStartMinimized, _T("bStartMinimized"));
	regWI(hKey, (BYTE*) &pSettings->bEnableToolTips_Main, _T("bEnableToolTips_Main"));
	regWS(hKey, pSettings->szEmailClient, _T("szEmailClient"));
	regWS(hKey, pSettings->szEmailClientArgs, _T("szEmailClientArgs"));
	regWS(hKey, pSettings->szWaveFile, _T("szWaveFile"));
	
	// ------ ADVANCED PAGE -----------------------------------------
	
	regWI(hKey, (BYTE*) &pSettings->nPriority, _T("nPriority"));
	regWI(hKey, (BYTE*) &pSettings->bIgnoreErrors, _T("bIgnoreErrors"));
	regWI(hKey, (BYTE*) &pSettings->bCheckInParallel, _T("bCheckInParallel"));
	regWI(hKey, (BYTE*) &pSettings->bStartupCheck, _T("bStartupCheck"));
	regWI(hKey, (BYTE*) &pSettings->bStartEnabled, _T("bStartEnabled"));
	regWI(hKey, (BYTE*) &pSettings->bShowEnabledStatus, _T("bShowEnabledStatus"));
	regWI(hKey, (BYTE*) &pSettings->bAOT, _T("bAOT"));
	regWI(hKey, (BYTE*) &pSettings->bLoadOnWindowsStartup, _T("bLoadOnWindowsStartup"));
	
	// ------ HEADERS PAGE -------------------------------------------
	
	regWI(hKey, (BYTE*) &pSettings->bObtainEmailHeaders, _T("bObtainEmailHeaders"));
	regWI(hKey, (BYTE*) &pSettings->bAutoDisplaySummary, _T("bAutoDisplaySummary"));
	regWI(hKey, (BYTE*) &pSettings->bForceSummaryDisplay, _T("bForceSummaryDisplay"));
	regWI(hKey, (BYTE*) &pSettings->bAutoCloseSummary, _T("bAutoCloseSummary"));
	regWI(hKey, (BYTE*) &pSettings->bEnableToolTips_Summary, _T("bEnableToolTips_Summary"));
	regWI(hKey, (BYTE*) &pSettings->bDisplayNonZeroAccountFirst, _T("bDisplayNonZeroAccountFirst"));
	regWI(hKey, (BYTE*) &pSettings->bUseWin32Colors, _T("bUseWin32Colors"));
	regWI(hKey, (BYTE*) &pSettings->clrBackColor, _T("clrBackColor"));
	regWI(hKey, (BYTE*) &pSettings->clrFontColor, _T("clrFontColor"));
	
	// ------ ANIMATION PAGE -----------------------------------------
	
	regWI(hKey, (BYTE*) &pSettings->bCheckMailAnimation, _T("bCheckMailAnimation"));
	regWI(hKey, (BYTE*) &pSettings->bHurryCheckMailAnimation, _T("bHurryCheckMailAnimation"));
	regWI(hKey, (BYTE*) &pSettings->nAnimationIterations, _T("nAnimationIterations"));
	regWI(hKey, (BYTE*) &pSettings->nAnimationDelay, _T("nAnimationDelay"));
	regWI(hKey, (BYTE*) &pSettings->bContinueAnimationTray, _T("bContinueAnimationTray"));
	regWI(hKey, (BYTE*) &pSettings->bContinueAnimationWindow, _T("bContinueAnimationWindow"));
	regWI(hKey, (BYTE*) &pSettings->bFlashFlagDownOnly, _T("bFlashFlagDownOnly"));
	
	// ------ HOTKEYS PAGE -------------------------------------------
	
	regWI(hKey, (BYTE*) &pSettings->hk_ToggleSummary, _T("hk_ToggleSummary"));
	regWI(hKey, (BYTE*) &pSettings->hk_ToggleMain, _T("hk_ToggleMain"));
	regWI(hKey, (BYTE*) &pSettings->hk_CheckNow, _T("hk_CheckNow"));
	regWI(hKey, (BYTE*) &pSettings->hk_RunClient, _T("hk_RunClient"));
	regWI(hKey, (BYTE*) &pSettings->hk_ClearFlag, _T("hk_ClearFlag"));
	regWI(hKey, (BYTE*) &pSettings->bQuickMinimize, _T("bQuickMinimize"));
	regWI(hKey, (BYTE*) &pSettings->bEnableSummaryArrowKeys, _T("bEnableSummaryArrowKeys"));
	
	// ------ MOUSE PAGE ---------------------------------------------
	
	regWI(hKey, (BYTE*) &pSettings->eDca, _T("eDca"));
	regWI(hKey, (BYTE*) &pSettings->eSDca, _T("eSDca"));
	
	// ------ SKIN PAGE ---------------------------------------------
	
	regWS(hKey, pSettings->szSelectedSkin, _T("szSelectedSkin"));
	regWI(hKey, (BYTE*) &pSettings->bEasyMove, _T("bEasyMove"));
	regWI(hKey, (BYTE*) &pSettings->nFontSize, _T("nFontSize"));
	
	RegCloseKey(hKey);
}

void
settings_Load(SETTINGS* pSettings, bool bLoadAccountInfo)
{
	HKEY hKey;
	
	if (RegOpenKeyEx(HKEY_CURRENT_USER, _T("Software\\wbiff!"),	0, KEY_ALL_ACCESS, &hKey) != ERROR_SUCCESS) {
		
		// opening the key failed, create it.
    DWORD dwDisposition;
		
		// create the wbiff! key for future reference
		RegOpenKeyEx(HKEY_CURRENT_USER, _T("Software"), 0, KEY_ALL_ACCESS, &hKey);
		RegCreateKeyEx(hKey, _T("wbiff!"), 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, &dwDisposition);
	}
	
	// ------ NON USER SETTINGS -------------------------------------
	
	REGRI(hKey, pSettings->nLastActivePage, _T("nLastActivePage"), 0)
	REGRI(hKey, pSettings->x, _T("x"), 100)
	REGRI(hKey, pSettings->y, _T("y"), 100)
	
	// ------ ACCOUNTS PAGE -----------------------------------------
	
	REGRI(hKey, pSettings->nMinLimit, _T("a_nMinLimit"), 1)
	REGRI(hKey, pSettings->nUpdateInterval, _T("a_nUpdateInterval"), 30)
	REGRI(hKey, pSettings->nAccounts, _T("a_nAccounts"), 0)
	
	pSettings->accounts = NULL;
	if (pSettings->nAccounts) {
		
		if (!(pSettings->accounts = (MAIL_PACCOUNT) wb_malloc(sizeof(MAIL_ACCOUNT) * pSettings->nAccounts)))
			return;
		else {
			
			TCHAR szHost[32], szLogin[32], szPassword[32], szProtocol[32], szDisplayName[32], szPort[32], szTimeout[32];
			int i;
			
			ASSERT(pSettings->accounts != NULL);
			for (i = 0; i < pSettings->nAccounts; i++) {
				
				wsprintf(szDisplayName, _T("a_szDisplayName%d"), i);
				wsprintf(szHost, _T("a_szHost%d"), i);
				wsprintf(szLogin, _T("a_szLogin%d"), i);
				wsprintf(szPassword, _T("a_szPassword%d"), i);
				wsprintf(szProtocol, _T("a_nType%d"), i);
				wsprintf(szPort, _T("a_nPort%d"), i);
				wsprintf(szTimeout, _T("a_nTimeout%d"), i);
				
				regRS(hKey, pSettings->accounts[i].szDisplayName, MAIL_ACCOUNT_FIELD_LENGTH, szDisplayName);
				regRS(hKey, pSettings->accounts[i].szHost, MAIL_ACCOUNT_FIELD_LENGTH, szHost);
				regRS(hKey, pSettings->accounts[i].szLogin, MAIL_ACCOUNT_FIELD_LENGTH, szLogin);
				regRS(hKey, pSettings->accounts[i].szPassword, MAIL_ACCOUNT_FIELD_LENGTH, szPassword);
				REGRI(hKey, pSettings->accounts[i].protocol, szProtocol, p_POP3)
				REGRI(hKey, pSettings->accounts[i].nPort, szPort, pSettings->accounts[i].protocol == p_POP3 ? WBIFF_DEFAULT_PROTOCOL_POP3_PORT : WBIFF_DEFAULT_PROTOCOL_IMAP4_PORT)
				REGRI(hKey, pSettings->accounts[i].nTimeout, szTimeout, -1)
				
				// if no display name is specified, create one based on server name
				if (pSettings->accounts[i].szDisplayName[0] == _T('\0'))
					lstrcpy(pSettings->accounts[i].szDisplayName, pSettings->accounts[i].szHost);
				
				crypto_Decrypt(WBIFF_WINDOWNAME, pSettings->accounts[i].szPassword);
				
				// set other members to zero values
				pSettings->accounts[i].messages = NULL;
				pSettings->accounts[i].nCount = 0;
				pSettings->accounts[i].szErrorText[0] = _T('\0');
			}
		}
		
	}
	
	// ------ GENERAL PAGE ------------------------------------------
	
	REGRI(hKey, pSettings->bFlashTray, _T("bFlashTray"), TRUE)
	REGRI(hKey, pSettings->bFlashWindow, _T("bFlashWindow"), TRUE)
	REGRI(hKey, pSettings->bRunClient, _T("bRunClient"), FALSE)
	REGRI(hKey, pSettings->bPlayWave, _T("bPlayWave"), FALSE)
	REGRI(hKey, pSettings->bPopupWindow, _T("bPopupWindow"), TRUE)
	REGRI(hKey, pSettings->bStartMinimized, _T("bStartMinimized"), FALSE)
	REGRI(hKey, pSettings->bEnableToolTips_Main, _T("bEnableToolTips_Main"), TRUE)
	regRS(hKey, pSettings->szEmailClient, MAX_PATH, _T("szEmailClient"));
	regRS(hKey, pSettings->szEmailClientArgs, MAX_PATH, _T("szEmailClientArgs"));
	regRS(hKey, pSettings->szWaveFile, MAX_PATH, _T("szWaveFile"));
	
	// ------ ADVANCED PAGE -----------------------------------------
	
	REGRI(hKey, pSettings->nPriority, _T("nPriority"), FALSE)
	REGRI(hKey, pSettings->bIgnoreErrors, _T("bIgnoreErrors"), TRUE)
	REGRI(hKey, pSettings->bCheckInParallel, _T("bCheckInParallel"), FALSE)
	REGRI(hKey, pSettings->bStartupCheck, _T("bStartupCheck"), TRUE)
	REGRI(hKey, pSettings->bStartEnabled, _T("bStartEnabled"), TRUE)
	REGRI(hKey, pSettings->bShowEnabledStatus, _T("bShowEnabledStatus"), TRUE)
	REGRI(hKey, pSettings->bAOT, _T("bAOT"), TRUE)
	REGRI(hKey, pSettings->bLoadOnWindowsStartup, _T("bLoadOnWindowsStartup"), FALSE)
	
	// ------ HEADERS PAGE -------------------------------------------
	
	REGRI(hKey, pSettings->bObtainEmailHeaders, _T("bObtainEmailHeaders"), TRUE)
	REGRI(hKey, pSettings->bAutoDisplaySummary, _T("bAutoDisplaySummary"), FALSE)
	REGRI(hKey, pSettings->bForceSummaryDisplay, _T("bForceSummaryDisplay"), FALSE)
	REGRI(hKey, pSettings->bAutoCloseSummary, _T("bAutoCloseSummary"), FALSE)
	REGRI(hKey, pSettings->bEnableToolTips_Summary, _T("bEnableToolTips_Summary"), TRUE)
	REGRI(hKey, pSettings->bDisplayNonZeroAccountFirst, _T("bDisplayNonZeroAccountFirst"), FALSE)
	REGRI(hKey, pSettings->bUseWin32Colors, _T("bUseWin32Colors"), TRUE)
	REGRI(hKey, pSettings->clrBackColor, _T("clrBackColor"), GetSysColor(COLOR_ACTIVECAPTION))
	REGRI(hKey, pSettings->clrFontColor, _T("clrFontColor"), GetSysColor(COLOR_CAPTIONTEXT))
	
	// ------ ANIMATION PAGE -----------------------------------------
	
	REGRI(hKey, pSettings->bCheckMailAnimation, _T("bCheckMailAnimation"), TRUE)
	REGRI(hKey, pSettings->bHurryCheckMailAnimation, _T("bHurryCheckMailAnimation"), TRUE);
	REGRI(hKey, pSettings->nAnimationIterations, _T("nAnimationIterations"), WBIFF_DEFAULT_ANIMATION_ITERATIONS)
	REGRI(hKey, pSettings->nAnimationDelay, _T("nAnimationDelay"), WBIFF_DEFAULT_ANIMATION_DELAY)
	REGRI(hKey, pSettings->bContinueAnimationTray, _T("bContinueAnimationTray"), FALSE)
	REGRI(hKey, pSettings->bContinueAnimationWindow, _T("bContinueAnimationWindow"), FALSE)
	REGRI(hKey, pSettings->bFlashFlagDownOnly, _T("bFlashFlagDownOnly"), TRUE)
	
	// ------ HOTKEYS PAGE -------------------------------------------
	
	REGRI(hKey, pSettings->hk_ToggleSummary, _T("hk_ToggleSummary"), 0x00000000)
	REGRI(hKey, pSettings->hk_ToggleMain, _T("hk_ToggleMain"), 0x00000000)
	REGRI(hKey, pSettings->hk_CheckNow, _T("hk_CheckNow"), 0x00000000)
	REGRI(hKey, pSettings->hk_RunClient, _T("hk_RunClient"), 0x00000000)
	REGRI(hKey, pSettings->hk_ClearFlag, _T("hk_ClearFlag"), 0x00000000)
	REGRI(hKey, pSettings->bQuickMinimize, _T("bQuickMinimize"), TRUE)
	REGRI(hKey, pSettings->bEnableSummaryArrowKeys, _T("bEnableSummaryArrowKeys"), TRUE)
	
	// ------ MOUSE PAGE ---------------------------------------------
	
	REGRI(hKey, pSettings->eDca, _T("eDca"), 4)
	REGRI(hKey, pSettings->eSDca, _T("eSDca"), 4)
	
#if 0	
	// WARNING: There is a bug that sets eDca and eSDca outside their
	//          ranges of 0-4. I can't figure out why this happens. To
	//          treat the symptom (instead of repairing the cause), ensure
	//          that these values are in the proper ranges.
	
	if (pSettings->eDca > 4 || pSettings->eDca < 0)
		pSettings->eDca = 4;
	if (pSettings->eSDca > 4 || pSettings->eSDca < 0)
		pSettings->eSDca = 4;
#endif

	// ------ SKIN PAGE ---------------------------------------------
	
	regRS(hKey, pSettings->szSelectedSkin, MAX_PATH, _T("szSelectedSkin"));
	REGRI(hKey, pSettings->bEasyMove, _T("bEasyMove"), TRUE)
	REGRI(hKey, pSettings->nFontSize, _T("nFontSize"), 8)
	RegCloseKey(hKey);
}

int
settings_Modify(HWND hwnd, bool* pbAccountsChanged)
{
	PROPSHEETPAGE psp[pageCount];
	PROPSHEETHEADER psh;
	int result;
	
	bAccountsChanged = 
	*pbAccountsChanged = false;
	
	// load temp_settings from registry
	settings_Load(&temp_settings, true);
	
	// create each page
	psp[pageAccounts].pszTemplate = MAKEINTRESOURCE(IDD_ACCOUNTS);
	psp[pageAccounts].dwSize = sizeof(PROPSHEETPAGE);
	psp[pageAccounts].dwFlags = PSP_DEFAULT | PSP_HASHELP;
	psp[pageAccounts].hInstance = hInstance;
	psp[pageAccounts].pfnDlgProc = ProcessWindowMessage_Accounts;
	psp[pageGeneral].pszTemplate = MAKEINTRESOURCE(IDD_GENERAL);
	psp[pageGeneral].dwSize = sizeof(PROPSHEETPAGE);
	psp[pageGeneral].dwFlags = PSP_DEFAULT | PSP_HASHELP;
	psp[pageGeneral].hInstance = hInstance;
	psp[pageGeneral].pfnDlgProc = ProcessWindowMessage_General;
	psp[pageAdvanced].pszTemplate = MAKEINTRESOURCE(IDD_ADVANCED);
	psp[pageAdvanced].dwSize = sizeof(PROPSHEETPAGE);
	psp[pageAdvanced].dwFlags = PSP_DEFAULT | PSP_HASHELP;
	psp[pageAdvanced].hInstance = hInstance;
	psp[pageAdvanced].pfnDlgProc = ProcessWindowMessage_Advanced;
	psp[pageHeaders].pszTemplate = MAKEINTRESOURCE(IDD_HEADERS);
	psp[pageHeaders].dwSize = sizeof(PROPSHEETPAGE);
	psp[pageHeaders].dwFlags = PSP_DEFAULT | PSP_HASHELP;
	psp[pageHeaders].hInstance = hInstance;
	psp[pageHeaders].pfnDlgProc = ProcessWindowMessage_Headers;
	psp[pageAnimation].pszTemplate = MAKEINTRESOURCE(IDD_ANIMATION);
	psp[pageAnimation].dwSize = sizeof(PROPSHEETPAGE);
	psp[pageAnimation].dwFlags = PSP_DEFAULT | PSP_HASHELP;
	psp[pageAnimation].hInstance = hInstance;
	psp[pageAnimation].pfnDlgProc = ProcessWindowMessage_Animation;
	psp[pageHotKeys].pszTemplate = MAKEINTRESOURCE(IDD_HOTKEYS);
	psp[pageHotKeys].dwSize = sizeof(PROPSHEETPAGE);
	psp[pageHotKeys].dwFlags = PSP_DEFAULT | PSP_HASHELP;
	psp[pageHotKeys].hInstance = hInstance;
	psp[pageHotKeys].pfnDlgProc = ProcessWindowMessage_HotKeys;
	psp[pageMouse].pszTemplate = MAKEINTRESOURCE(IDD_MOUSE);
	psp[pageMouse].dwSize = sizeof(PROPSHEETPAGE);
	psp[pageMouse].dwFlags = PSP_DEFAULT | PSP_HASHELP;
	psp[pageMouse].hInstance = hInstance;
	psp[pageMouse].pfnDlgProc = ProcessWindowMessage_Mouse;
	psp[pageSkin].pszTemplate = MAKEINTRESOURCE(IDD_SKIN);
	psp[pageSkin].dwSize = sizeof(PROPSHEETPAGE);
	psp[pageSkin].dwFlags = PSP_DEFAULT | PSP_HASHELP;
	psp[pageSkin].hInstance = hInstance;
	psp[pageSkin].pfnDlgProc = ProcessWindowMessage_Skin;
	
	psh.dwSize = sizeof(PROPSHEETHEADER);
	psh.dwFlags = PSH_PROPSHEETPAGE | PSH_NOAPPLYNOW | PSH_HASHELP;
	
	if (!(dwState & WBIFF_CURRENT_STATE_MAINWND_VISIBLE))
		psh.hwndParent = util_GetTrayHwnd();
	else
		psh.hwndParent = hwnd;
	
	psh.hInstance = hInstance;
	psh.pszCaption  = SETTINGS_WINDOWNAME;
	psh.nPages = sizeof(psp) / sizeof(PROPSHEETPAGE);
	psh.ppsp = (LPPROPSHEETPAGE) &psp;
	psh.nStartPage = temp_settings.nLastActivePage;
	//psh.pfnCallback = wbiff_settings_PropSheetProc;
	
	result = PropertySheet(&psh);
	
	switch (result) {
		
		case IDOK:
			*pbAccountsChanged = bAccountsChanged;
			SetLoadOnWindowsStartupFlagInRegistry(temp_settings.bLoadOnWindowsStartup);
			settings_Save(&temp_settings);
			break;
	}
	
	settings_UnLoad(&temp_settings);
	
	return result;
}
