//
//  wbiff! mailbox flag for Windows
//	Copyright  1999, 2000 F. Sadaghiani (sada@guesswork.org)
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 
//
//  59 Temple Place - Suite 330
//  Boston, MA 
//  02111-1307, USA.
//
//

#include "std_wb.h"
#include "types.h"

#include "skin.h"

#include "wb_util.h"

#pragma hdrstop

static const LPCTSTR lpSection_Skin = _T("Skin");
static const LPCTSTR lpSection_Window = _T("Window");
static const LPCTSTR lpSection_Bitmap = _T("Bitmap");
static const LPCTSTR lpDefaultSkin = _T("default.wbs");

extern TCHAR szWorkingFolder[MAX_PATH + 1];
extern SETTINGS settings;

bool 
skin_Load(PSKIN pSkin, LPCTSTR lpSkinFile)
{
	TCHAR szFullSkinFile[MAX_PATH + 1];
	TCHAR szTemp[128];
	HANDLE hFile = NULL;

	lstrcpy(szFullSkinFile, szWorkingFolder);
	lstrcat(szFullSkinFile, lpSkinFile);
		
	hFile = CreateFile(szFullSkinFile, 0, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hFile == INVALID_HANDLE_VALUE) {

		// failed to load the specified skin file; try to load the default one instead
		TRACE("skin_Load: failed to find skin file %s. Trying default skin...\r\n", lpSkinFile);

		lstrcpy(szFullSkinFile, szWorkingFolder);
		lstrcat(szFullSkinFile, lpDefaultSkin);

		hFile = CreateFile(szFullSkinFile, 0, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFile == INVALID_HANDLE_VALUE) {

			// failed to load default too!
			// our last effort will be to try to load the first .WBS file in the
			// directory
			HANDLE hFind = NULL;
			WIN32_FIND_DATA fd;
			
			lstrcpy(szFullSkinFile, szWorkingFolder);
			lstrcat(szFullSkinFile, _T("*.wbs"));
			
			hFind = FindFirstFile(szFullSkinFile, &fd);
			
			if (hFind != INVALID_HANDLE_VALUE) {

				// use the first skin file found
				lstrcpy(settings.szSelectedSkin, fd.cFileName);
				lstrcpy(szFullSkinFile, szWorkingFolder);
				lstrcat(szFullSkinFile, fd.cFileName);
				TRACE("Using first found skin in the directory \"%s\"\r\n", fd.cFileName);
				FindClose(hFind);
				
				// okay to proceed w/o returning a value
			}
			else {

				TRACE("No skin files in program directory\r\n");
				return false;
			}
		}
		else {

			TRACE("Default skin found OK\r\n");
			lstrcpy(settings.szSelectedSkin, lpDefaultSkin);
			lstrcpy(szFullSkinFile, szWorkingFolder);
			lstrcat(szFullSkinFile, lpDefaultSkin);
		}
	}

	lpSkinFile = szFullSkinFile;

	// Read in skin definitions
	RS(pSkin->Image, MAX_PATH, lpSection_Skin, _T("Image"), lpSkinFile);
	RS(pSkin->Name, 64, lpSection_Skin, _T("Name"), lpSkinFile);
	RS(pSkin->HyperLinkText, 64, lpSection_Skin, _T("HyperLinkText"), lpSkinFile);
	RS(pSkin->HyperLink, 128, lpSection_Skin, _T("HyperLink"), lpSkinFile);
	RS(pSkin->FontFaceName, 32, lpSection_Skin, _T("FontFaceName"), lpSkinFile);
	RS(szTemp, sizeof(szTemp), lpSection_Skin, _T("FontColor"), lpSkinFile);

	pSkin->FontColor = RGB(0, 0, 0);
	if (szTemp[0] != _T('\0')) {

		// parse color info (who needs strtok anyway?)
		int red = -1, green = -1, blue = -1;
		TCHAR* p = szTemp + lstrlen(szTemp);

		while (p >= szTemp && *p != _T(',')) p--;
		blue = util_atoi(p+1);

		--p;
		while (p >= szTemp && *p != _T(',')) p--;
		green = util_atoi(p+1);

		--p;
		while (p >= szTemp && *p != _T(',')) p--;
		red = util_atoi(p+1);

		pSkin->FontColor = RGB(red, green, blue);
	}

	// read icon file names
	RS(pSkin->Icon_FlagDown, sizeof(pSkin->Icon_FlagDown), lpSection_Skin, _T("Icon_FlagDown"), lpSkinFile);
	RS(pSkin->Icon_FlagDownDisabled, sizeof(pSkin->Icon_FlagDownDisabled), lpSection_Skin, _T("Icon_FlagDownDisabled"), lpSkinFile);
	RS(pSkin->Icon_FlagUp, sizeof(pSkin->Icon_FlagUp), lpSection_Skin, _T("Icon_FlagUp"), lpSkinFile);

	RS(pSkin->Icon_Broken, sizeof(pSkin->Icon_Broken), lpSection_Skin, _T("Icon_Broken"), lpSkinFile); //'ad'

	// Read window definition
	RI(pSkin->Width, lpSection_Window, _T("Width"), lpSkinFile);
	RI(pSkin->Height, lpSection_Window, _T("Height"), lpSkinFile);

	RI(pSkin->FontRect.left, lpSection_Window, _T("FontRect.left"), lpSkinFile);
	RI(pSkin->FontRect.top, lpSection_Window, _T("FontRect.top"), lpSkinFile);
	RI(pSkin->FontRect.right, lpSection_Window, _T("FontRect.right"), lpSkinFile);
	RI(pSkin->FontRect.bottom, lpSection_Window, _T("FontRect.bottom"), lpSkinFile);
	
	RI(pSkin->FlagRect.left, lpSection_Window, _T("FlagRect.left"), lpSkinFile);
	RI(pSkin->FlagRect.top, lpSection_Window, _T("FlagRect.top"), lpSkinFile);
	RI(pSkin->FlagRect.right, lpSection_Window, _T("FlagRect.right"), lpSkinFile);
	RI(pSkin->FlagRect.bottom, lpSection_Window, _T("FlagRect.bottom"), lpSkinFile);

	RI(pSkin->SysButtonRect.left, lpSection_Window, _T("SysButtonRect.left"), lpSkinFile);
	RI(pSkin->SysButtonRect.top, lpSection_Window, _T("SysButtonRect.top"), lpSkinFile);
	RI(pSkin->SysButtonRect.right, lpSection_Window, _T("SysButtonRect.right"), lpSkinFile);
	RI(pSkin->SysButtonRect.bottom, lpSection_Window, _T("SysButtonRect.bottom"), lpSkinFile);

	RI(pSkin->MinButtonRect.left, lpSection_Window, _T("MinButtonRect.left"), lpSkinFile);
	RI(pSkin->MinButtonRect.top, lpSection_Window, _T("MinButtonRect.top"), lpSkinFile);
	RI(pSkin->MinButtonRect.right, lpSection_Window, _T("MinButtonRect.right"), lpSkinFile);
	RI(pSkin->MinButtonRect.bottom, lpSection_Window, _T("MinButtonRect.bottom"), lpSkinFile);

	RI(pSkin->ClsButtonRect.left, lpSection_Window, _T("ClsButtonRect.left"), lpSkinFile);
	RI(pSkin->ClsButtonRect.top, lpSection_Window, _T("ClsButtonRect.top"), lpSkinFile);
	RI(pSkin->ClsButtonRect.right, lpSection_Window, _T("ClsButtonRect.right"), lpSkinFile);
	RI(pSkin->ClsButtonRect.bottom, lpSection_Window, _T("ClsButtonRect.bottom"), lpSkinFile);

	RI(pSkin->CaptionRect.left, lpSection_Window, _T("CaptionRect.left"), lpSkinFile);
	RI(pSkin->CaptionRect.top, lpSection_Window, _T("CaptionRect.top"), lpSkinFile);
	RI(pSkin->CaptionRect.right, lpSection_Window, _T("CaptionRect.right"), lpSkinFile);
	RI(pSkin->CaptionRect.bottom, lpSection_Window, _T("CaptionRect.bottom"), lpSkinFile);

	// Read bitmap definition
	RI(pSkin->Window_X, lpSection_Bitmap, _T("Window_X"), lpSkinFile);
	RI(pSkin->Window_Y, lpSection_Bitmap, _T("Window_Y"), lpSkinFile);

	RI(pSkin->FlagDown_X, lpSection_Bitmap, _T("FlagDown_X"), lpSkinFile);
	RI(pSkin->FlagDown_Y, lpSection_Bitmap, _T("FlagDown_Y"), lpSkinFile);

	RI(pSkin->FlagUp_X, lpSection_Bitmap, _T("FlagUp_X"), lpSkinFile);
	RI(pSkin->FlagUp_Y, lpSection_Bitmap, _T("FlagUp_Y"), lpSkinFile);

	RI(pSkin->SysButtonDown_X, lpSection_Bitmap, _T("SysButtonDown_X"), lpSkinFile);
	RI(pSkin->SysButtonDown_Y, lpSection_Bitmap, _T("SysButtonDown_Y"), lpSkinFile);

	RI(pSkin->SysButtonUp_X, lpSection_Bitmap, _T("SysButtonUp_X"), lpSkinFile);
	RI(pSkin->SysButtonUp_Y, lpSection_Bitmap, _T("SysButtonUp_Y"), lpSkinFile);

	RI(pSkin->MinButtonDown_X, lpSection_Bitmap, _T("MinButtonDown_X"), lpSkinFile);
	RI(pSkin->MinButtonDown_Y, lpSection_Bitmap, _T("MinButtonDown_Y"), lpSkinFile);

	RI(pSkin->MinButtonUp_X, lpSection_Bitmap, _T("MinButtonUp_X"), lpSkinFile);
	RI(pSkin->MinButtonUp_Y, lpSection_Bitmap, _T("MinButtonUp_Y"), lpSkinFile);

	RI(pSkin->ClsButtonDown_X, lpSection_Bitmap, _T("ClsButtonDown_X"), lpSkinFile);
	RI(pSkin->ClsButtonDown_Y, lpSection_Bitmap, _T("ClsButtonDown_Y"), lpSkinFile);

	RI(pSkin->ClsButtonUp_X, lpSection_Bitmap, _T("ClsButtonUp_X"), lpSkinFile);
	RI(pSkin->ClsButtonUp_Y, lpSection_Bitmap, _T("ClsButtonUp_Y"), lpSkinFile);

	CloseHandle(hFile);

	TRACE("skin_Load: successfully loaded skin file %s\r\n", lpSkinFile);
	return true;
}

#ifndef min
static int 
min(int a, int b)
{
	return a < b ? a : b;
}
#endif


HRGN 
skin_BitmapToRegion(HBITMAP hbmp, COLORREF cTransparentColor, COLORREF cTolerance, int xoffset, int yoffset)
{
	// This algorithm was contributed by Damian Hodgkiss of thirty4 interactive.

	HRGN hRegion = NULL;
	
	if (hbmp)	{

		// Create a memory DC inside which we will scan the bitmap content
		HDC hMemDC = CreateCompatibleDC(NULL);
		if (hMemDC)
		{
			// Get bitmap size
			HBITMAP hbm32;
			BITMAP bm;
			BITMAPINFOHEADER RGB32BITSBITMAPINFO;
			VOID * pbits32;
			GetObject(hbmp, sizeof(bm), &bm);
			
			// Create a 32 bits depth bitmap and select it into the memory DC
			RGB32BITSBITMAPINFO.biSize = sizeof(BITMAPINFOHEADER);
			RGB32BITSBITMAPINFO.biWidth = bm.bmWidth;
			RGB32BITSBITMAPINFO.biHeight = bm.bmHeight;
			RGB32BITSBITMAPINFO.biPlanes = 1;
			RGB32BITSBITMAPINFO.biBitCount = 32;
			RGB32BITSBITMAPINFO.biCompression = BI_RGB;
			RGB32BITSBITMAPINFO.biSizeImage = 0;
			RGB32BITSBITMAPINFO.biXPelsPerMeter = 0;
			RGB32BITSBITMAPINFO.biYPelsPerMeter = 0;
			RGB32BITSBITMAPINFO.biClrUsed = 0;
			RGB32BITSBITMAPINFO.biClrImportant = 0;
			
			hbm32 = CreateDIBSection(hMemDC, (BITMAPINFO *)&RGB32BITSBITMAPINFO, DIB_RGB_COLORS, &pbits32, NULL, 0);
			if (hbm32)
			{
				HBITMAP holdBmp = (HBITMAP)SelectObject(hMemDC, hbm32);
				
				// Create a DC just to copy the bitmap into the memory DC
				HDC hDC = CreateCompatibleDC(hMemDC);
				if (hDC)
				{
					// Get how many bytes per row we have for the bitmap bits (rounded up to 32 bits)
					HBITMAP holdBmp;
					BITMAP bm32;
					DWORD maxRects;
					HANDLE hData;
					RGNDATA *pData;
					HRGN h;
					BYTE *p32;
					int y, x;
					
					// Keep on hand highest and lowest values for the "transparent" pixels
					BYTE lr = GetRValue(cTransparentColor);
					BYTE lg = GetGValue(cTransparentColor);
					BYTE lb = GetBValue(cTransparentColor);
					BYTE hr = (BYTE) min(0xff, lr + GetRValue(cTolerance));
					BYTE hg = (BYTE) min(0xff, lg + GetGValue(cTolerance));
					BYTE hb = (BYTE) min(0xff, lb + GetBValue(cTolerance));
					
					GetObject(hbm32, sizeof(bm32), &bm32);
					while (bm32.bmWidthBytes % 4)
						bm32.bmWidthBytes++;
					
					// Copy the bitmap into the memory DC
					holdBmp = (HBITMAP)SelectObject(hDC, hbmp);
					BitBlt(hMemDC, 0, 0, bm.bmWidth, bm.bmHeight, hDC, 0, 0, SRCCOPY);
					
					// For better performances, we will use the ExtCreateRegion() function to create the
					// region. This function take a RGNDATA structure on entry. We will add rectangles by
					// amount of ALLOC_UNIT number in this structure.

					#define ALLOC_UNIT 100

					maxRects = ALLOC_UNIT;
					hData = GlobalAlloc(GMEM_MOVEABLE, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects));
					pData = (RGNDATA *)GlobalLock(hData);
					pData->rdh.dwSize = sizeof(RGNDATAHEADER);
					pData->rdh.iType = RDH_RECTANGLES;
					pData->rdh.nCount = pData->rdh.nRgnSize = 0;
					SetRect(&pData->rdh.rcBound, MAXLONG, MAXLONG, 0, 0);
					
					// Scan each bitmap row from bottom to top (the bitmap is inverted vertically)
					p32 = (BYTE *)bm32.bmBits + (bm32.bmHeight - 1) * bm32.bmWidthBytes;
					for (y = 0; y < bm.bmHeight; y++)
					{
						// Scan each bitmap pixel from left to right
						for (x = 0; x < bm.bmWidth; x++)
						{
							// Search for a continuous range of "non transparent pixels"
							int x0 = x;
							LONG *p = (LONG *)p32 + x;
							while (x < bm.bmWidth)
							{
								BYTE b = GetRValue(*p);
								if (b >= lr && b <= hr)
								{
									b = GetGValue(*p);
									if (b >= lg && b <= hg)
									{
										b = GetBValue(*p);
										if (b >= lb && b <= hb) // This pixel is "transparent"
											break;
									}
								}
								p++;
								x++;
							}
							
							if (x > x0)
							{
								RECT *pr;
								
								// Add the pixels (x0, y) to (x, y+1) as a new rectangle in the region
								if (pData->rdh.nCount >= maxRects)
								{
									GlobalUnlock(hData);
									maxRects += ALLOC_UNIT;
									hData = GlobalReAlloc(hData, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), GMEM_MOVEABLE);
									pData = (RGNDATA *)GlobalLock(hData);
								}

								pr = (RECT *)&pData->Buffer;
								SetRect(&pr[pData->rdh.nCount], x0+xoffset, y+yoffset, x+xoffset,y+1+yoffset);
								if (x0 < pData->rdh.rcBound.left)
									pData->rdh.rcBound.left = x0+xoffset;
								if (y < pData->rdh.rcBound.top)
									pData->rdh.rcBound.top = y+yoffset;
								if (x > pData->rdh.rcBound.right)
									pData->rdh.rcBound.right = x+xoffset;
								if (y+1 > pData->rdh.rcBound.bottom)
									pData->rdh.rcBound.bottom = y+1+yoffset;
								pData->rdh.nCount++;
								
								// On Windows98, ExtCreateRegion() may fail if the number of rectangles is too
								// large (ie: > 4000). Therefore, we have to create the region by multiple steps.
								if (pData->rdh.nCount == 2000)
								{
									HRGN h = ExtCreateRegion(NULL, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), pData);
									if (hRegion)
									{
										CombineRgn(hRegion, hRegion, h, RGN_OR);
										DeleteObject(h);
									}
									else
										hRegion = h;
									pData->rdh.nCount = 0;
									SetRect(&pData->rdh.rcBound, MAXLONG, MAXLONG, 0, 0);
								}
							}
						}
						
						// Go to next row (remember, the bitmap is inverted vertically)
						p32 -= bm32.bmWidthBytes;
					}
					
					// Create or extend the region with the remaining rectangles
					h = ExtCreateRegion(NULL, sizeof(RGNDATAHEADER) + (sizeof(RECT) * maxRects), pData);
					if (hRegion)
					{
						CombineRgn(hRegion, hRegion, h, RGN_OR);
						DeleteObject(h);
					}
					else
						hRegion = h;
					
					// Clean up
					GlobalUnlock(hData);
					GlobalFree(hData);
					SelectObject(hDC, holdBmp);
					DeleteDC(hDC);
				}

				DeleteObject(SelectObject(hMemDC, holdBmp));
			}

			DeleteDC(hMemDC);
		}
	}

	return hRegion;
}

