//
//  wbiff! mailbox flag for Windows
//	Copyright  1999, 2000 F. Sadaghiani (sada@guesswork.org)
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 
//
//  59 Temple Place - Suite 330
//  Boston, MA 
//  02111-1307, USA.
//
//

#pragma once

/////////////////////////////////////////////////////////////////////////////
//
// MSVC precompiled header file

#define STRICT				
#define WIN32_LEAN_AND_MEAN

#if 0
// Using Win2K headers forces usage of commctrl 5.0
#define WINVER 0x0500       // use Windows 2000 headers and libs
#define _WIN32_WINNT 0x0500
#else
// Non Win2K platforms must support commctrl v4.7+
#define _WIN32_IE 0x0300    // force usage of at least Commctrl 4.7
#define _WIN32_WINDOWS 0x0400
#define _WIN32_WINNT 0x0400
#endif

#ifdef _DEBUG
#define _CRTDBG_MAP_ALLOC
#include <malloc.h>
#include <stdlib.h>
#include <crtdbg.h>
#ifndef ASSERT
#define ASSERT _ASSERT
#endif
#else
#define ASSERT(x) ((void)0)
#endif

#include <windows.h>
#include <mmsystem.h>
#include <windowsx.h>
#include <lmcons.h>
#include <winbase.h>
#include <winuser.h>
#include <shellapi.h>
#include <tchar.h>
#include <commdlg.h>
#include <winuser.h>
#include <shellapi.h>
#include <tchar.h>
#include <commctrl.h>
#include <winsock2.h>
#include <svcguid.h>
#include <wincrypt.h>

#if V_21
#include <htmlhelp.h>
#pragma comment(lib, "htmlhelp.lib");
#endif

#ifdef __cplusplus
extern "C" {
#endif
extern const LPCTSTR WBIFF_WINDOWNAME; // "global" consts
extern const LPCTSTR WBIFF_CLASSNAME;
#ifdef __cplusplus
};
#endif

// define boolean type
#ifndef __cplusplus
#ifndef bool
typedef enum { false, true } bool;
#endif
#endif

#include "tracelog.h"

#ifdef _DEBUG
#define wb_free(x) _free_dbg(x, _NORMAL_BLOCK)
#define wb_malloc(x) _malloc_dbg(x, _NORMAL_BLOCK, __FILE__, __LINE__)
#define WBIFF_INLINE_DECL
#define WBIFF_INLINE_IMPL
#else
#define wb_free(x) HeapFree(GetProcessHeap(), 0, (x))
#define wb_malloc(x) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, (x))
#define WBIFF_INLINE_DECL extern __inline
#define WBIFF_INLINE_IMPL __inline
#endif

/////////////////////////////////////////////////////////////////////////////

// defaults
#define WBIFF_DEFAULT_ANIMATION_DELAY           400
#define WBIFF_DEFAULT_ANIMATION_ITERATIONS      10

#define WBIFF_DEFAULT_PROTOCOL_IMAP4_PORT       143
#define WBIFF_DEFAULT_PROTOCOL_POP3_PORT        110

// user messages
#define WBIFF_IDM_SYSTRAY_CALLBACK_MESSAGE         (WM_USER+100)
#define WBIFF_IDM_THREAD_DONE_CHECK_NOW            (WM_USER+101)
#define WBIFF_IDM_THREAD_DONE_FLASH_WINDOW         (WM_USER+102)
#define WBIFF_IDM_THREAD_DONE_FLASH_TRAY           (WM_USER+103)
#define WBIFF_IDM_THREAD_DONE_CHECK_MAIL_ANIMATION (WM_USER+104)
#define WBIFF_IDM_BEGIN_NOTIFY_ANIMATION           (WM_USER+106)

// button state
#define WBIFF_BUTTON_STATE_NONE      0x00000000
#define WBIFF_BUTTON_STATE_SYSBUTTON 0x00000001
#define WBIFF_BUTTON_STATE_MINBUTTON 0x00000002
#define WBIFF_BUTTON_STATE_CLSBUTTON 0x00000004
#define WBIFF_BUTTON_STATE_DOWN      0x00000008

// hot key ids
#define	WBIFF_IDHK_CHECK_NOW      0x00000001
#define WBIFF_IDHK_TOGGLE_MAIN    0x00000002
#define WBIFF_IDHK_TOGGLE_SUMMARY 0x00000003
#define WBIFF_IDHK_RUN_CLIENT     0x00000004
#define WBIFF_IDHK_CLEAR_FLAG     0x00000005

// timer ids
#define WBIFF_TIMER_CHECK_MAIL 0x00000001

// tool tip ids
#define WBIFF_TOOLTIP_SUMMARY_CLOSE         0x00000001
#define WBIFF_TOOLTIP_SUMMARY_NEXT_ACCOUNT  0x00000002
#define WBIFF_TOOLTIP_SUMMARY_PREV_ACCOUNT  0x00000003
#define WBIFF_TOOLTIP_SUMMARY_NEXT_MESSAGE  0x00000004
#define WBIFF_TOOLTIP_SUMMARY_PREV_MESSAGE  0x00000005
#define WBIFF_TOOLTIP_SUMMARY_CLEAR_FLAG    0x00000006
#define WBIFF_TOOLTIP_SUMMARY_RUN_CLIENT    0x00000007
#define WBIFF_TOOLTIP_MAIN_CLOSE            0x00000008
#define WBIFF_TOOLTIP_MAIN_MINIMIZE         0x00000009
#define WBIFF_TOOLTIP_MAIN_MENU             0x0000000A
#define WBIFF_TOOLTIP_MAIN_FLAG             0x0000000B

// OS capabilities
#define WBIFF_VERSION_NULL                 0x00000000
#define WBIFF_VERSION_ENABLE_SWAPPING      0x00000001
#define WBIFF_VERSION_ENABLE_EFFECTS       0x00000002
#define WBIFF_VERSION_ENABLE_MOUSE_SHADOW  0x00000004

// program state
#define WBIFF_CURRENT_STATE_NULL                0x00000000
#define WBIFF_CURRENT_STATE_ENABLED             0x00000001
#define WBIFF_CURRENT_STATE_IN_SETTINGSDLG      0x00000002
#define WBIFF_CURRENT_STATE_IN_ABOUTDLG         0x00000004
#define WBIFF_CURRENT_STATE_FLAG_UP             0x00000008
#define WBIFF_CURRENT_STATE_CHECKING_MAIL       0x00000010
#define WBIFF_CURRENT_STATE_MAINWND_VISIBLE     0x00000020
#define WBIFF_CURRENT_STATE_SUMMARYWND_VISIBLE  0x00000040

// bit manipulation helpers
#define WBIFF_BIT_IS_SET(bitfield, bit) (bitfield & bit)
#define WBIFF_BIT_SET(bitfield, bit) bitfield |= bit;
#define WBIFF_BIT_CLEAR(bitfield, bit) bitfield &= ~bit;
#define WBIFF_BIT_TOGGLE(bitfield, bit) bitfield ^= bit;

// thread routines
#ifdef _DEBUG
#include <process.h>
// use _beginthreadex to maintain CRT state through thread calls
#define wb_thread(func, param, tid) (HANDLE) _beginthreadex(NULL, 0, func, (LPVOID) param, 0, tid);
#else
// no CRT so use CreateThread instead
#define wb_thread(func, param, tid) CreateThread(NULL, 0, func, (LPVOID) param, 0, tid);
#endif

