//
//  wbiff! mailbox flag for Windows
//	Copyright  1999, 2000 F. Sadaghiani (sada@guesswork.org)
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 
//
//  59 Temple Place - Suite 330
//  Boston, MA 
//  02111-1307, USA.
//
//

//
// TODO
// ----
//
// - Optimize drawing (invalidate areas that need painting only, not entire window)
// - Too much repeated code:
//   - GetRect for window co-ordinates
// - Seperate code outside the message pump for easier maintenance
// - don't create then destroy window. Keep hidden for faster loading and
//   less overhead


#include "std_wb.h"
#include "types.h"

#include <winuser.h> //'ad'

#include "summarywnd.h"

#include "mail.general.h"

#include "wb_util.h"
#include "impl.h"
#include "res/resource.h"


#pragma intrinsic(memset)
#pragma hdrstop

const int summarywnd_CX = 250;
const int summarywnd_CY = 110;

static const LPCTSTR summarywnd_szWindowClass = _T("summarywnd_class");

#define summarywnd_ButtonSize 12

// button regions
static RECT summarywnd_rcClose = { 230, 7, 230 + summarywnd_ButtonSize, 7 + summarywnd_ButtonSize };

/*
static RECT summarywnd_rcPrevAccount = { 200, 7, 200 + summarywnd_ButtonSize, 7 + summarywnd_ButtonSize };
static RECT summarywnd_rcNextAccount = { 215, 7, 215 + summarywnd_ButtonSize, 7 + summarywnd_ButtonSize };
*/
//'ad'
static RECT summarywnd_rcPrevAccount = { 6, 7, 6 + summarywnd_ButtonSize, 7 + summarywnd_ButtonSize };
static RECT summarywnd_rcNextAccount = { 21, 7, 21 + summarywnd_ButtonSize, 7 + summarywnd_ButtonSize };

static RECT summarywnd_rcPrevMessage = { 6, 93, 6 + summarywnd_ButtonSize, 93 + summarywnd_ButtonSize };
static RECT summarywnd_rcNextMessage = { 21, 93, 21 + summarywnd_ButtonSize, 93 + summarywnd_ButtonSize };

//'ad'
static RECT summarywnd_rcClearFlag = { 
	
	250 - 54,
	110 - 20,
	250 - 54 + 15,
	110 - 20 + 16
 };
//'ad'
static RECT summarywnd_rcRunClient = {

	250 - 54 + 25,
	110 - 20,
	250 - 54 + 25 + 15,
	110 - 20 + 16
};




static HFONT summarywnd_fntBold = NULL, summarywnd_fntNormal = NULL;

static HWND hwndSummary;
static HWND hwndParent = NULL;

static HWND hwndSummaryTT = NULL;


// data pointers into account information
static MAIL_PACCOUNT accounts;
static int total_Account;
static int current_Account;
static int current_Message;

extern HINSTANCE hInstance;
extern SETTINGS settings;
extern DWORD dwState;


// function forwards
LRESULT CALLBACK summarywnd_WndProc(HWND, UINT, WPARAM, LPARAM);
void summarywnd_CreateToolTips(HWND);

void
summarywnd_Destroy(void)
{
	if (hwndSummary) 
		DestroyWindow(hwndSummary);
}

void
summarywnd_ShowWindow(bool bShow)
{
	if (dwState & WBIFF_CURRENT_STATE_SUMMARYWND_VISIBLE)
		ShowWindow(hwndSummary, bShow ? SW_SHOW : SW_HIDE);
}

void
summarywnd_UpdateWindow()
{
	if (dwState & WBIFF_CURRENT_STATE_SUMMARYWND_VISIBLE) {

		InvalidateRect(hwndSummary, NULL, TRUE);
		UpdateWindow(hwndSummary);
	}
}

void
summarywnd_ReInit(MAIL_PACCOUNT pAccounts, int nCount)
{
	accounts = pAccounts;
	total_Account = nCount;
	current_Account = 0;
	current_Message = 0;

	if (settings.bDisplayNonZeroAccountFirst) {

		// find the first account with non-zero message count

		int i;
		for (i = 0; i < nCount; ++i) {
			if (pAccounts[i].nCount > 0) {
				current_Account = i;
				break;
			}
		}
		current_Message = pAccounts[current_Account].nCount - 1; //'ad'
	}

	ShowWindow(hwndSummary, SW_SHOW);
	InvalidateRect(hwndSummary, NULL, TRUE);
	UpdateWindow(hwndSummary);
}

WBIFF_INLINE_IMPL BOOL
summarywnd_IsWindowVisible(void)
{
	return dwState & WBIFF_CURRENT_STATE_SUMMARYWND_VISIBLE;
}

bool
summarywnd_Create(HWND hwnd, int x, int y, MAIL_PACCOUNT pAccounts, int nCount)
{
	static bool bInit = false;
	
	hwndParent = hwnd;
	
	if (!bInit) {

		WNDCLASSEX wce;

		ZeroMemory(&wce, sizeof(wce));
		wce.cbSize = sizeof(WNDCLASSEX);
		wce.style = 0;
		wce.lpfnWndProc = summarywnd_WndProc;
		wce.cbClsExtra = 0;
		wce.cbWndExtra = 0;
		wce.hInstance = hInstance;
		wce.hIcon = NULL;
		wce.hIconSm = NULL;
		wce.hCursor = LoadCursor(NULL, IDC_ARROW);
		wce.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
		wce.lpszMenuName = NULL;
		wce.lpszClassName = summarywnd_szWindowClass;

		RegisterClassEx(&wce);
		bInit = true;
	}

	hwndSummary = CreateWindowEx(WS_EX_TOOLWINDOW, summarywnd_szWindowClass, 
		NULL, WS_CHILD | WS_POPUP, x, y, summarywnd_CX, summarywnd_CY, hwndParent, (HMENU) NULL, hInstance, NULL);

	if (hwndSummary) {

		if (settings.bEnableToolTips_Summary)
			summarywnd_CreateToolTips(hwndSummary);

		summarywnd_ReInit(pAccounts, nCount);
		SetForegroundWindow(hwndSummary);

		dwState |= WBIFF_CURRENT_STATE_SUMMARYWND_VISIBLE;
		return true;
	}

	return false;
}

static HFONT
summarywnd_CreateFont(HDC hdc, DWORD dwType)
{
	LPCTSTR lpFontName = _T("Tahoma");
	const int nFontSize = 8;

	return CreateFont(-MulDiv(nFontSize, GetDeviceCaps(hdc, LOGPIXELSY), 72),	0, 0,	0, dwType, FALSE,
		FALSE, FALSE, ANSI_CHARSET, OUT_TT_PRECIS,	CLIP_TT_ALWAYS,	DEFAULT_QUALITY, FF_SWISS | VARIABLE_PITCH,	lpFontName);
}

static void
summarywnd_DrawInterface(HDC hdc)
{
	// Draws the summary window buttons, lines and 3d border

	const UINT summarywnd_ButtonStyle = DFCS_FLAT | DFCS_ADJUSTRECT;

	// window interface componets
	const int summarywnd_Polylines = 3;
	const RECT summarywnd_rcBackground = { 1, 1, 249, 109 };
	const POINT summarywnd_aptsLeftSide[] =  { { 0, 109 }, { 0, 0 }, { 249, 0 } };
	const POINT summarywnd_aptsRightSide[] = { { 249, 0 }, { 249, 109 }, { -1, 109 } };

	RECT rect;
	UINT active;
	HBRUSH background = CreateSolidBrush(settings.bUseWin32Colors ? GetSysColor(COLOR_ACTIVECAPTION) : settings.clrBackColor);

	HBITMAP hbmp = NULL;

	HPEN pen0 = CreatePen(PS_SOLID, 2, GetSysColor(COLOR_3DSHADOW));
	HPEN pen1 = CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
	HPEN pen2 = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_BTNFACE));
	HPEN pen3 = CreatePen(PS_SOLID, 1, settings.bUseWin32Colors ? GetSysColor(COLOR_CAPTIONTEXT) : settings.clrFontColor);
	
	SelectObject(hdc, pen2);
	Polyline(hdc, summarywnd_aptsLeftSide, summarywnd_Polylines);

	SelectObject(hdc, pen1);
	Polyline(hdc, summarywnd_aptsRightSide, summarywnd_Polylines);
	
	SelectObject(hdc, background);
	FillRect(hdc, &summarywnd_rcBackground, background);
	
	// draw close button
	CopyRect(&rect, &summarywnd_rcClose);
	DrawFrameControl(hdc, &rect, DFC_CAPTION, DFCS_PUSHED | DFCS_CAPTIONCLOSE | summarywnd_ButtonStyle);
	
	// draw arrow controls
	if (current_Account == 0)
		active = DFCS_INACTIVE;
	else
		active = 0;
	
	CopyRect(&rect, &summarywnd_rcPrevAccount);
	DrawFrameControl(hdc, &rect, DFC_SCROLL, DFCS_SCROLLUP | summarywnd_ButtonStyle | active);

	if (current_Account == total_Account - 1)
		active = DFCS_INACTIVE;
	else
		active = 0;
	
	CopyRect(&rect, &summarywnd_rcNextAccount);
	DrawFrameControl(hdc, &rect, DFC_SCROLL, DFCS_SCROLLDOWN | summarywnd_ButtonStyle | active);
	
	// draw separating line (top line)
	SelectObject(hdc, pen3);
	MoveToEx(hdc, 5, 25, NULL);
	LineTo(hdc, summarywnd_CX-5, 25);

	// draw arrow controls
	if (current_Message <= 0)
		active = DFCS_INACTIVE;
	else
		active = 0;

	CopyRect(&rect, &summarywnd_rcPrevMessage);
	DrawFrameControl(hdc, &rect, DFC_SCROLL, DFCS_SCROLLLEFT | summarywnd_ButtonStyle | active);

	if (current_Message == accounts[current_Account].nCount - 1 || accounts[current_Account].nCount <= 0)
		active = DFCS_INACTIVE;
	else
		active = 0;
	
	CopyRect(&rect, &summarywnd_rcNextMessage);
	DrawFrameControl(hdc, &rect, DFC_SCROLL, DFCS_SCROLLRIGHT | summarywnd_ButtonStyle | active);

	// draw the bitmap in the window
	hbmp = (HBITMAP) LoadImage(hInstance, MAKEINTRESOURCE(IDB_BITMAP1), IMAGE_BITMAP, 0, 0, LR_VGACOLOR);
	
	util_DrawTransparentBitmap(hdc, hbmp, summarywnd_CX - 54, summarywnd_CY - 20, RGB(255, 0, 255));

	DeleteObject(pen0);
	DeleteObject(pen1);
	DeleteObject(pen2);
	DeleteObject(hbmp);
	DeleteObject(background);
}

static void
summarywnd_DrawSingleLine(HDC hdc, int x, int y, int cy, LPTSTR lpBold, int nBoldLen, LPTSTR lpNormal, int nNormalLen)
{
	// draws a single line of text with two components a bold & plain font

	RECT rect;
	SIZE size;
	TEXTMETRIC tm;
	
	SelectObject(hdc, summarywnd_fntBold);
	SetTextColor(hdc, settings.bUseWin32Colors ? GetSysColor(COLOR_CAPTIONTEXT) : settings.clrFontColor);
	SetBkMode(hdc, TRANSPARENT);
	
	SetRect(&rect, x, y, 0, 0); // no boundry on BOLD FIELD
	
	DrawText(hdc, lpBold, nBoldLen, &rect, DT_NOCLIP | DT_NOPREFIX | DT_SINGLELINE | DT_TOP | DT_LEFT);
	GetTextExtentPoint32(hdc, lpBold, nBoldLen, &size); 
	x += size.cx; 
	
	GetTextMetrics(hdc, &tm); // only required for non-Truetype fonts (i.e. Raster fonts)
	x -= tm.tmOverhang; 
	
	SelectObject(hdc, summarywnd_fntNormal); 
	SetRect(&rect, x, y, cy, 0); // boundry on NON-BOLD TEXT FIELD	
	DrawText(hdc, lpNormal, nNormalLen, &rect, DT_MODIFYSTRING | DT_NOCLIP | DT_END_ELLIPSIS | DT_NOPREFIX | DT_SINGLELINE | DT_TOP | DT_LEFT);
}

static void
summarywnd_DrawMultiLine(HDC hdc, int x, int y, LPTSTR lpText, int nLength)
{
	RECT rect;
	
	SelectObject(hdc, summarywnd_fntNormal);
	SetTextColor(hdc, settings.bUseWin32Colors ? GetSysColor(COLOR_CAPTIONTEXT) : settings.clrFontColor);
	SetBkMode(hdc, TRANSPARENT);
	
	SetRect(&rect, x, y, summarywnd_CX - x, 93 - 5);
	DrawText(hdc, lpText, nLength, &rect, DT_MODIFYSTRING | DT_WORDBREAK | DT_NOPREFIX | DT_TOP | DT_LEFT);
}

static RECT*
summarywnd_GetRect(const POINT* point)
{
	RECT* rect = NULL;

	if (PtInRect(&summarywnd_rcPrevAccount, *point))
		rect = &summarywnd_rcPrevAccount;
	else if (PtInRect(&summarywnd_rcNextAccount, *point))
		rect = &summarywnd_rcNextAccount;
	else if (PtInRect(&summarywnd_rcPrevMessage, *point))
		rect = &summarywnd_rcPrevMessage;
	else if (PtInRect(&summarywnd_rcNextMessage, *point))
		rect = &summarywnd_rcNextMessage;
	else if (PtInRect(&summarywnd_rcClose, *point))
		rect = &summarywnd_rcClose;
	else if (PtInRect(&summarywnd_rcClearFlag, *point))
		rect = &summarywnd_rcClearFlag;
	else if (PtInRect(&summarywnd_rcRunClient, *point))
		rect = &summarywnd_rcRunClient;
	
	return rect;
}

static bool
summarywnd_impl_NextAccount(void)
{
	if (current_Account < total_Account - 1) {
		
		++current_Account;
		//current_Message = 0;
		current_Message = accounts[current_Account].nCount - 1; //'ad'

		return true;
	}
	else
		return false;
}

static bool
summarywnd_impl_PrevAccount(void)
{
	if (current_Account > 0) {
		
		--current_Account;
		//current_Message = 0;
		current_Message = accounts[current_Account].nCount - 1; //'ad'
		return true;
	}
	else
		return false;
}

static bool
summarywnd_impl_PrevMessage(void)
{
	if (current_Message > 0) {

		--current_Message;
		return true;
	}
	else
		return false;
}

static bool
summarywnd_impl_NextMessage(void)
{
	if (current_Message < accounts[current_Account].nCount - 1) {
		
		++current_Message;
		return true;
	}
	else
		return false;
}

static LRESULT CALLBACK
summarywnd_WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {

		case WM_KILLFOCUS:
			if (settings.bAutoCloseSummary)
				DestroyWindow(hwnd);
			break;
		case WM_MOUSEWHEEL: { //'ad'
			bool bNext = (short)HIWORD(wParam) < 0;
			bool bNeedUpdate = false;
			int yPos; 
			RECT rect;
	
			GetWindowRect(hwnd, &rect);
			yPos = (short)HIWORD(lParam) - rect.top;

			if (/*yPos >= 0 && */yPos <= 25) {
				if (bNext) {
					bNeedUpdate = summarywnd_impl_NextAccount();
				}
				else {
					bNeedUpdate = summarywnd_impl_PrevAccount();
				}
			}
			else {
				if (bNext) {
					bNeedUpdate = summarywnd_impl_NextMessage();
				}
				else {
					bNeedUpdate = summarywnd_impl_PrevMessage();
				}
			}

			if (bNeedUpdate) {
			
				InvalidateRect(hwnd, NULL, TRUE);
				UpdateWindow(hwnd);
			}

			break;
		}
		case WM_DESTROY:

			DeleteObject(summarywnd_fntNormal);
			DeleteObject(summarywnd_fntBold);

			if (hwndSummaryTT)                // bounds checker reports an error here when 
				DestroyWindow(hwndSummaryTT);   // destroying a TT window

			summarywnd_fntNormal =
			summarywnd_fntBold = NULL;
			hwndSummaryTT =
			hwndSummary = NULL;

			dwState &= ~WBIFF_CURRENT_STATE_SUMMARYWND_VISIBLE;
			break;

		case WM_LBUTTONDOWN:
			{
				POINT point = { LOWORD(lParam), HIWORD(lParam) };
				RECT* rect = summarywnd_GetRect(&point);

				if (dwState & (WBIFF_CURRENT_STATE_IN_SETTINGSDLG | WBIFF_CURRENT_STATE_IN_ABOUTDLG))
					break;
				
				// make sure we're not clicking inside a button
				if (!rect)
					PostMessage(hwnd, WM_NCLBUTTONDOWN, HTCAPTION, lParam);
			}
			break;

		case WM_LBUTTONUP:
			{
				POINT point = { LOWORD(lParam), HIWORD(lParam) };
				bool bNeedUpdate = false;

				if (dwState & (WBIFF_CURRENT_STATE_IN_SETTINGSDLG | WBIFF_CURRENT_STATE_IN_ABOUTDLG))
					break;

				// clear previous animation loops
				impl_ClearAnimations();

				// check if user clicked RunClient or ClearFlag 
				// (both of these actions dismiss the window)
				if (PtInRect(&summarywnd_rcClearFlag, point)) {

					impl_ClearFlag(hwndParent);
					DestroyWindow(hwnd);
				}
				else if (PtInRect(&summarywnd_rcRunClient, point)) {

					impl_RunClient(hwndParent);
					DestroyWindow(hwnd);
				}
				else if (PtInRect(&summarywnd_rcPrevAccount, point))
					bNeedUpdate = summarywnd_impl_PrevAccount();
				else if (PtInRect(&summarywnd_rcNextAccount, point))
					bNeedUpdate = summarywnd_impl_NextAccount();
				else if (PtInRect(&summarywnd_rcPrevMessage, point))
					bNeedUpdate = summarywnd_impl_PrevMessage();
				else if (PtInRect(&summarywnd_rcNextMessage, point))
					bNeedUpdate = summarywnd_impl_NextMessage();
				else if (PtInRect(&summarywnd_rcClose, point))
					DestroyWindow(hwndSummary);

				if (bNeedUpdate) {
				
					InvalidateRect(hwnd, NULL, TRUE);
					UpdateWindow(hwnd);
				}
			}
			break;
		
		case WM_PAINT:
			{
				PAINTSTRUCT ps;

				TCHAR boldText[1024];
				TCHAR normalText[1024];

				BeginPaint(hwnd, &ps);
				SetMapMode(ps.hdc, MM_TEXT);

				// create the fonts if this is the first time painting
				if (!summarywnd_fntBold)
					summarywnd_fntBold = summarywnd_CreateFont(ps.hdc, FW_BOLD);

				if (!summarywnd_fntNormal)
					summarywnd_fntNormal = summarywnd_CreateFont(ps.hdc, FW_NORMAL);

				summarywnd_DrawInterface(ps.hdc);
				
				lstrcpy(boldText, _T("Account:  "));
				lstrcpy(normalText, accounts[current_Account].szDisplayName);
				
				//summarywnd_DrawSingleLine(ps.hdc, 6, 6, 200, boldText, lstrlen(boldText), normalText, lstrlen(normalText));
				//'ad'
				summarywnd_DrawSingleLine(ps.hdc, 39, 6, 200, boldText, lstrlen(boldText), normalText, lstrlen(normalText));

				if (accounts[current_Account].nCount < 0) {

					// display error text
					lstrcpy(boldText, _T("The following error occurred:")); 

					normalText[0] = _T('\0');
					summarywnd_DrawSingleLine(ps.hdc, 6, 35, 240, boldText, lstrlen(boldText), normalText, lstrlen(normalText));

					lstrcpy(normalText,	mail_general_GetErrorMessageFromCode(accounts[current_Account].nCount));

					if (accounts[current_Account].szErrorText[0] != _T('\0')) {

						lstrcat(normalText, _T("\n"));
						lstrcat(normalText, accounts[current_Account].szErrorText);
					}
					
					summarywnd_DrawMultiLine(ps.hdc, 6, 48, normalText, lstrlen(normalText));
				}
				else if (accounts[current_Account].nCount == 0) {

					// no messages on this server
					lstrcpy(boldText, _T("No new messages on this account"));
					normalText[0] = _T('\0');
					summarywnd_DrawSingleLine(ps.hdc, 6, 35, 240, boldText, lstrlen(boldText), normalText, lstrlen(normalText));
				}
				else {

					// draw FROM line
					lstrcpy(boldText, _T("From:  "));
					lstrcpyn(normalText, accounts[current_Account].messages[current_Message].szFrom, sizeof(normalText));
					summarywnd_DrawSingleLine(ps.hdc, 6, 35, 240, boldText, lstrlen(boldText), normalText, lstrlen(normalText));

					// draw DATE line				
					lstrcpy(boldText, _T("Date:  "));
					lstrcpyn(normalText, accounts[current_Account].messages[current_Message].szDate, sizeof(normalText));
					summarywnd_DrawSingleLine(ps.hdc, 6, 52, 240, boldText, lstrlen(boldText), normalText, lstrlen(normalText));

					// draw SUBJECT line
					lstrcpy(boldText, _T("Subject:  "));
					lstrcpyn(normalText, accounts[current_Account].messages[current_Message].szSubject, sizeof(normalText));
					summarywnd_DrawSingleLine(ps.hdc, 6, 69, 240, boldText, lstrlen(boldText), normalText, lstrlen(normalText));

					// draw MESSAGE COUNT line
					boldText[0] = _T('\0');
					wsprintf(normalText, _T("Message: %d of %d"), current_Message+1,accounts[current_Account].nCount);
					summarywnd_DrawSingleLine(ps.hdc, 40, 92, 240, boldText, lstrlen(boldText), normalText, lstrlen(normalText));
				}

				EndPaint(hwnd, &ps);
			}
			break;

		case WM_DISPLAYCHANGE:
			if (settings.bUseWin32Colors) {

				settings.clrBackColor = GetSysColor(COLOR_ACTIVECAPTION);
				settings.clrFontColor = GetSysColor(COLOR_ACTIVECAPTION);
			}
			break;
		
		case WM_ERASEBKGND:
			return TRUE;
			break;

		case WM_KEYDOWN:
		case WM_CHAR:
			{
				if (dwState & (WBIFF_CURRENT_STATE_IN_SETTINGSDLG | WBIFF_CURRENT_STATE_IN_ABOUTDLG))
					break;

				if (settings.bEnableSummaryArrowKeys) {
				
					TCHAR ch = (TCHAR) wParam;
					bool bNeedUpdate = false;

					switch (ch) {

						case VK_ESCAPE:
							DestroyWindow(hwnd);
							break;

						case VK_LEFT:
							bNeedUpdate = summarywnd_impl_PrevMessage();
							break;

						case VK_RIGHT:
							bNeedUpdate = summarywnd_impl_NextMessage();
							break;

						case VK_UP:
							bNeedUpdate = summarywnd_impl_PrevAccount();
							break;

						case VK_DOWN:
							bNeedUpdate = summarywnd_impl_NextAccount();
							break;
					}

					if (bNeedUpdate) {

						InvalidateRect(hwnd, NULL, TRUE);
						UpdateWindow(hwnd);
					}
				}
			}
			break;

		default:
			return DefWindowProc(hwnd, message, wParam, lParam);
			break;
	}
	return 0;
}


static void 
summarywnd_CreateToolTips(HWND hwnd)
{
	hwndSummaryTT = util_TT_CreateToolTip(hInstance, hwnd);

	util_TT_AddTool(hwndSummaryTT, hwnd, &summarywnd_rcClose, _T("Close"), WBIFF_TOOLTIP_SUMMARY_CLOSE);
	util_TT_AddTool(hwndSummaryTT, hwnd, &summarywnd_rcPrevAccount, _T("Previous Account"), WBIFF_TOOLTIP_SUMMARY_PREV_ACCOUNT);
	util_TT_AddTool(hwndSummaryTT, hwnd, &summarywnd_rcNextAccount, _T("Next Account"), WBIFF_TOOLTIP_SUMMARY_NEXT_ACCOUNT);
	util_TT_AddTool(hwndSummaryTT, hwnd, &summarywnd_rcPrevMessage, _T("Previous Message"), WBIFF_TOOLTIP_SUMMARY_PREV_MESSAGE);
	util_TT_AddTool(hwndSummaryTT, hwnd, &summarywnd_rcNextMessage, _T("Next Message"), WBIFF_TOOLTIP_SUMMARY_NEXT_MESSAGE);
	util_TT_AddTool(hwndSummaryTT, hwnd, &summarywnd_rcClearFlag, _T("Clear Flag"), WBIFF_TOOLTIP_SUMMARY_CLEAR_FLAG);
	util_TT_AddTool(hwndSummaryTT, hwnd, &summarywnd_rcRunClient, _T("Run Client"), WBIFF_TOOLTIP_SUMMARY_RUN_CLIENT);
}

