//
//  wbiff! mailbox flag for Windows
//	Copyright  1999, 2000 F. Sadaghiani (sada@guesswork.org)
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 
//
//  59 Temple Place - Suite 330
//  Boston, MA 
//  02111-1307, USA.
//
//

#pragma once

#ifdef  __cplusplus
extern "C" {
#endif

/////////////////////////////////////////////////////////////////////////////
//
// MESSAGE and ACCOUNT type definitions

#define MAIL_MESSAGE_FIELD_LENGTH 128

typedef struct tagMAIL_MESSAGE
{
	// defines the fields required for a single message
	TCHAR szFrom[MAIL_MESSAGE_FIELD_LENGTH];
	TCHAR szSubject[MAIL_MESSAGE_FIELD_LENGTH];
	TCHAR szDate[MAIL_MESSAGE_FIELD_LENGTH];

} MAIL_MESSAGE, *MAIL_PMESSAGE;

#define MAIL_ACCOUNT_FIELD_LENGTH 128

typedef	enum { p_POP3 = 0, p_IMAP4 } PROTOCOL;

typedef struct tagMAIL_ACCOUNT
{
	// defines the fields required for many messages on a
	// single account
	int nCount;               // number of messages
	MAIL_PMESSAGE messages;   // array of messages

	TCHAR szDisplayName[MAIL_ACCOUNT_FIELD_LENGTH];
	TCHAR szHost[MAIL_ACCOUNT_FIELD_LENGTH];
	TCHAR szLogin[MAIL_ACCOUNT_FIELD_LENGTH];
	TCHAR szPassword[MAIL_ACCOUNT_FIELD_LENGTH];

	TCHAR szErrorText[MAIL_ACCOUNT_FIELD_LENGTH];

	PROTOCOL protocol;

	short int nPort;
	int nTimeout;

	DWORD flags;

} MAIL_ACCOUNT, *MAIL_PACCOUNT;

#define MAIL_ACCOUNT_NULL       0x0    /* 0 */
#define MAIL_ACCOUNT_ENABLED    (1<<0) /* 1 */
#define MAIL_ACCOUNT_CAN_FILTER (1<<1) /* 10 */

/////////////////////////////////////////////////////////////////////////////
//
// SETTINGS type definitions

#define WBIFF_SETTINGS_GENERAL_FLASH_TRAY           0x00000001
#define WBIFF_SETTINGS_GENERAL_FLASH_WINDOW         0x00000002
#define WBIFF_SETTINGS_GENERAL_RUN_CLIENT           0x00000004
#define WBIFF_SETTINGS_GENERAL_PLAY_WAVE            0x00000008
#define WBIFF_SETTINGS_GENERAL_POPUP_WINODW         0x00000010
#define WBIFF_SETTINGS_GENERAL_START_MINIMIZED      0x00000020
#define WBIFF_SETTINGS_GENERAL_ENABLE_TOOLTIPS      0x00000040
#define WBIFF_SETTINGS_ADVANCED_LOAD_ON_STARTUP     0x00000080
#define WBIFF_SETTINGS_ADVANCED_IGNORE_ERRORS       0x00000100
#define WBIFF_SETTINGS_ADVANCED_CHECK_IN_PARALLEL   0x00000200
#define WBIFF_SETTINGS_ADVANCED_STARTUP_CHECK       0x00000400
#define WBIFF_SETTINGS_ADVANCED_SHOW_ENABLED_STATUS 0x00000800
#define WBIFF_SETTINGS_ADVANCED_START_ENABLED       0x00001000
#define WBIFF_SETTINGS_ADVANCED_AOT                 0x00002000
#define WBIFF_SETTINGS_HEADERS_OBTAIN               0x00004000
#define WBIFF_SETTINGS_HEADERS_AUTO_DISPLAY         0x00008000
#define WBIFF_SETTINGS_HEADERS_FORCE_DISPLAY        0x00010000
#define WBIFF_SETTINGS_HEADERS_AUTO_CLOSE           0x00020000
#define WBIFF_SETTINGS_HEADERS_USE_WIN32_COLORS     0x00040000
#define WBIFF_SETTINGS_HEADERS_ENABLE_TOOLTIPS      0x00080000
#define WBIFF_SETTINGS_ANIMATION_HURRY              0x00100000
#define WBIFF_SETTINGS_ANIMATION_CHECK_MAIL         0x00200000
#define WBIFF_SETTINGS_ANIMATION_CONTINUE_TRAY      0x00400000
#define WBIFF_SETTINGS_ANIMATION_CONTINUE_WINDOW    0x00800000
#define WBIFF_SETTINGS_ANIMATION_FLAG_DOWN_ONLY     0x01000000

/*
TODO
----
-use a bit field for boolean flags
*/
typedef struct tagSETTINGS
{
	// Non user settings --------------------------------------------

	int x, y;
	int nLastActivePage;

	// User settings ------------------------------------------------

	// accounts tab
	int nMinLimit;
	int nUpdateInterval;
	int nAccounts;
	MAIL_PACCOUNT accounts; // array of accounts

	// general tab
	BOOL bFlashTray;
	BOOL bFlashWindow;
	BOOL bRunClient;
	BOOL bPlayWave;
	BOOL bPopupWindow;
	BOOL bStartMinimized;
	BOOL bEnableToolTips_Main;
	TCHAR szEmailClient[MAX_PATH + 1];
	TCHAR szEmailClientArgs[MAX_PATH + 1];
	TCHAR szWaveFile[MAX_PATH + 1];

	// advanced tab
	BOOL bLoadOnWindowsStartup;
	BOOL bIgnoreErrors;
	BOOL bCheckInParallel;
	BOOL bStartupCheck;
	BOOL bShowEnabledStatus;
	BOOL bStartEnabled;
	BOOL bAOT;
	int nPriority; // 0 = normal, 1 = low

	// headers+summary tab
	BOOL bObtainEmailHeaders;
	BOOL bAutoDisplaySummary;
	BOOL bForceSummaryDisplay;
	BOOL bAutoCloseSummary;
	BOOL bUseWin32Colors;
	BOOL bEnableToolTips_Summary;
	BOOL bDisplayNonZeroAccountFirst;
	COLORREF clrFontColor;
	COLORREF clrBackColor;

	// animation tab
	BOOL bHurryCheckMailAnimation;
	BOOL bCheckMailAnimation;
	int nAnimationIterations;
	int nAnimationDelay;
	BOOL bContinueAnimationTray;
	BOOL bContinueAnimationWindow;
	BOOL bFlashFlagDownOnly;

	// hotkeys tab
	DWORD hk_CheckNow;
	DWORD hk_RunClient;
	DWORD hk_ToggleMain;
	DWORD hk_ToggleSummary;
	DWORD hk_ClearFlag;
	BOOL bQuickMinimize;
	BOOL bEnableSummaryArrowKeys;

	// mouse tab
	int eDca;
	int eSDca;

	// skin tab
	TCHAR szSelectedSkin[MAX_PATH + 1];
	BOOL bEasyMove;
	int nFontSize;

} SETTINGS, *PSETTINGS;

typedef struct tagSTATS
{
	int nTimesLoaded;
	SYSTEMTIME stFirstRun;
	int nMessages;
	int nTimesChecked;

} STATS, *PSTATS;

#ifdef  __cplusplus
}
#endif
