//
//  wbiff! mailbox flag for Windows
//	Copyright  1999, 2000 F. Sadaghiani (sada@guesswork.org)
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc.,
//
//  59 Temple Place - Suite 330
//  Boston, MA
//  02111-1307, USA.
//
//

#include "std_wb.h"

#include "wb_util.h"

#pragma hdrstop
#pragma intrinsic(memset)

int
util_ReadInt(LPCTSTR lpSectionName, LPCTSTR lpKeyName, int nDefault, LPCTSTR lpFileName)
{
	return GetPrivateProfileInt(lpSectionName, lpKeyName, nDefault, lpFileName);
}

void
util_ReadString(LPTSTR lpText, int nCount, LPCTSTR lpSectionName, LPCTSTR lpKeyName, LPCTSTR lpFileName)
{
	lpText[0] = _T('\0');
	GetPrivateProfileString(lpSectionName, lpKeyName, lpText, lpText, nCount, lpFileName);
}

#define wbiff_isspace(x) (x==_T(' ')||x==_T('\t'))
#define wbiff_isdigit(x) (x==_T('1')||x==_T('2')||x==_T('3')||x==_T('4')||x==_T('5')||x==_T('6')||x==_T('7')||x==_T('8')||x==_T('9')||x==_T('0'))
int
util_atoi(LPCTSTR nptr)
{
	int c;
	int total;
	int sign;

	while (wbiff_isspace((int)(unsigned char)*nptr))
		++nptr;

	c = (int)(unsigned char) *nptr++;
	sign = c;
	if (c == _T('-') || c == _T('+'))
		c = (int)(unsigned char) *nptr++;

	total = 0;
	while (wbiff_isdigit(c)) {

		total = 10 * total + (c - _T('0'));
		c = (int)(unsigned char) *nptr++;
	}

	if (sign == _T('-'))
		total = -total;

	return total;
}

// executes a program:
bool
util_Exec(HWND hwnd, LPCTSTR lpExecutable, LPTSTR lpCommandLine)
{
	STARTUPINFO si;
	PROCESS_INFORMATION pi;

	TCHAR szCmdLine[1024];
	LPTSTR lpFormat;

	ZeroMemory(&si, sizeof(STARTUPINFO));

	si.cb = sizeof(STARTUPINFO);
	si.wShowWindow = SW_SHOWNORMAL;
	si.dwFlags = STARTF_USESHOWWINDOW;

	if (lpExecutable[0] == _T('"') && lpExecutable[lstrlen(lpExecutable) - 1] ==  _T('"')) {

		// quotes already present
		lpFormat = _T("%s %s");
	}
	else {

		// executable file must be quoted to conform to Win32 file name
		// specs.
		lpFormat = _T("\"%s\" %s");
	}

	wsprintf(szCmdLine, lpFormat, lpExecutable, lpCommandLine);

	if (!CreateProcess(lpExecutable, szCmdLine, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi)) {

#ifdef _DEBUG
		OutputDebugString(_T("ERROR: CreateProcess failed...\n"));
#endif

		return false;
	}
	else {

		// We don't use the handles so close them now
		CloseHandle(pi.hProcess);
		CloseHandle(pi.hThread);
		return true;
	}
}

void
util_OpenObject(HWND hwnd, LPCTSTR lpObjectName)
{
	// tell the windows shell to "open" the specified object
	SHELLEXECUTEINFO sei;

	ZeroMemory(&sei, sizeof(SHELLEXECUTEINFO));

	sei.cbSize = sizeof(SHELLEXECUTEINFO);
	sei.hwnd = hwnd;
	sei.lpFile = lpObjectName;
	sei.lpVerb = _T("Open");
	sei.nShow = SW_NORMAL;

	ShellExecuteEx(&sei);
}

LPTSTR
util_strstr(LPCTSTR str1, LPCTSTR str2)
{
	if (!*str2)
		return (LPTSTR) str1;
	else {

		LPTSTR cp = (LPTSTR) str1;
		LPTSTR s1;
		LPTSTR s2;

		while (*cp) {

			s1 = cp;
			s2 = (LPTSTR) str2;

			while (*s1 && *s2 && !(*s1 - *s2))
				s1++, s2++;

			if (!*s2)
				return cp;

			cp++;
		}

		return NULL;
	}
}

LPTSTR
util_TrimBoth(LPTSTR lpBuffer)
{
	if (lpBuffer) {

		// trim left side
		LPTSTR p;

		while (*lpBuffer == _T('\r') || *lpBuffer == _T('\n') || *lpBuffer == _T(' ') || *lpBuffer == _T('\t'))
			++lpBuffer;

		// trim right side (also remove \r\n pair)
		p = lpBuffer + lstrlen(lpBuffer) - 1;
		while (*p == _T('\r') || *p == _T('\n') || *p == _T(' ') || *p == _T('\t'))
			--p;

		*(++p) = _T('\0'); // terminate the string
	}
	return lpBuffer;
}

HWND
util_GetTrayHwnd(void)
{
	// find handle to tray window
	HWND hwnd = NULL;
	TCHAR szClassName[64];

	szClassName[0] = _T('\0');
	if (hwnd = FindWindow(_T("Shell_TrayWnd"), NULL)) {

		// get the first child window
		hwnd = GetWindow(hwnd, GW_CHILD);

		while (hwnd && GetClassName(hwnd, szClassName, sizeof(szClassName))) {

			if (lstrcmp(szClassName, _T("TrayNotifyWnd")) == 0)
				return hwnd;
			else
				hwnd = GetWindow(hwnd, GW_HWNDNEXT);
		}
	}

	return NULL;
}

void // MSDN Article ID: Q79212
util_DrawTransparentBitmap(HDC hdc, HBITMAP hBitmap, int xStart, int yStart, COLORREF cTransparentColor)
{
	BITMAP     bm;
	COLORREF   cColor;
	HBITMAP    bmAndBack, bmAndObject, bmAndMem, bmSave;
	HBITMAP    bmBackOld, bmObjectOld, bmMemOld, bmSaveOld;
	HDC        hdcMem, hdcBack, hdcObject, hdcTemp, hdcSave;
	POINT      ptSize;

	hdcTemp = CreateCompatibleDC(hdc);
	SelectObject(hdcTemp, hBitmap);   // Select the bitmap

	GetObject(hBitmap, sizeof(BITMAP), (LPSTR)&bm);
	ptSize.x = bm.bmWidth;            // Get width of bitmap
	ptSize.y = bm.bmHeight;           // Get height of bitmap
	DPtoLP(hdcTemp, &ptSize, 1);      // Convert from device

	// to logical points

	// Create some DCs to hold temporary data.
	hdcBack   = CreateCompatibleDC(hdc);
	hdcObject = CreateCompatibleDC(hdc);
	hdcMem    = CreateCompatibleDC(hdc);
	hdcSave   = CreateCompatibleDC(hdc);

	// Create a bitmap for each DC. DCs are required for a number of
	// GDI functions.

	// Monochrome DC
	bmAndBack   = CreateBitmap(ptSize.x, ptSize.y, 1, 1, NULL);

	// Monochrome DC
	bmAndObject = CreateBitmap(ptSize.x, ptSize.y, 1, 1, NULL);

	bmAndMem    = CreateCompatibleBitmap(hdc, ptSize.x, ptSize.y);
	bmSave      = CreateCompatibleBitmap(hdc, ptSize.x, ptSize.y);

	// Each DC must select a bitmap object to store pixel data.
	bmBackOld   = SelectObject(hdcBack, bmAndBack);
	bmObjectOld = SelectObject(hdcObject, bmAndObject);
	bmMemOld    = SelectObject(hdcMem, bmAndMem);
	bmSaveOld   = SelectObject(hdcSave, bmSave);

	// Set proper mapping mode.
	SetMapMode(hdcTemp, GetMapMode(hdc));

	// Save the bitmap sent here, because it will be overwritten.
	BitBlt(hdcSave, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);

	// Set the background color of the source DC to the color.
	// contained in the parts of the bitmap that should be transparent
	cColor = SetBkColor(hdcTemp, cTransparentColor);

	// Create the object mask for the bitmap by performing a BitBlt
	// from the source bitmap to a monochrome bitmap.
	BitBlt(hdcObject, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0,
		SRCCOPY);

	// Set the background color of the source DC back to the original
	// color.
	SetBkColor(hdcTemp, cColor);

	// Create the inverse of the object mask.
	BitBlt(hdcBack, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0,
		NOTSRCCOPY);

	// Copy the background of the main DC to the destination.
	BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdc, xStart, yStart,
		SRCCOPY);

	// Mask out the places where the bitmap will be placed.
	BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, SRCAND);

	// Mask out the transparent colored pixels on the bitmap.
	BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcBack, 0, 0, SRCAND);

	// XOR the bitmap with the background on the destination DC.
	BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCPAINT);

	// Copy the destination to the screen.
	BitBlt(hdc, xStart, yStart, ptSize.x, ptSize.y, hdcMem, 0, 0,
		SRCCOPY);

	// Place the original bitmap back into the bitmap sent here.
	BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcSave, 0, 0, SRCCOPY);

	// Delete the memory bitmaps.
	DeleteObject(SelectObject(hdcBack, bmBackOld));
	DeleteObject(SelectObject(hdcObject, bmObjectOld));
	DeleteObject(SelectObject(hdcMem, bmMemOld));
	DeleteObject(SelectObject(hdcSave, bmSaveOld));

	// Delete the memory DCs.
	DeleteDC(hdcMem);
	DeleteDC(hdcBack);
	DeleteDC(hdcObject);
	DeleteDC(hdcSave);
	DeleteDC(hdcTemp);
}

HWND
util_TT_CreateToolTip(HINSTANCE hInstance, HWND hwnd)
{
	HWND hwndTT = CreateWindowEx(WS_EX_TOPMOST,
		TOOLTIPS_CLASS,
		NULL,
		WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		hwnd,
		NULL,
		hInstance,
		NULL
		);

	SetWindowPos(hwndTT,
		HWND_TOPMOST,
		0,
		0,
		0,
		0,
		SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);

	return hwndTT;
}

void
util_TT_AddTool(HWND hwndTT, HWND hwndParent, const RECT* rect, LPTSTR lpsz, int nID)
{
	TOOLINFO ti;

	ti.cbSize = sizeof(TOOLINFO);
	ti.uFlags = TTF_TRANSPARENT | TTF_SUBCLASS;
	ti.hwnd = hwndParent;
	ti.hinst = NULL;
	ti.uId = nID;
	ti.lpszText = lpsz == NULL ? LPSTR_TEXTCALLBACK : lpsz;

	CopyRect(&ti.rect, rect);
	SendMessage(hwndTT, TTM_ADDTOOL, 0, (LPARAM) &ti);
}

void
util_TT_DelTool(HWND hwndTT, HWND hwndParent, int nID)
{
	TOOLINFO ti;

	ti.cbSize = sizeof(TOOLINFO);
	ti.hwnd = hwndParent;
	ti.uId = nID;

	SendMessage(hwndTT, TTM_DELTOOL, 0, (LPARAM) &ti);
}

