//
//  wbiff! mailbox flag for Windows
//	Copyright  1999, 2000 F. Sadaghiani (sada@guesswork.org)
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 
//
//  59 Temple Place - Suite 330
//  Boston, MA 
//  02111-1307, USA.
//
//

/*
TODO
----
*/

#include "std_wb.h"
#include "types.h"

#include "mail.general.h"
#include "impl.h"
#include "impl.thread.h"
#include "wb_util.h"
#include "skin.h"
#include "summarywnd.h"
#include "settings.h"
#include "about.h"
#include "res/resource.h"
#include "stats.h"

#pragma intrinsic(memset)
#pragma hdrstop

// ------------------------------------------------------------------

TCHAR szWorkingFolder[MAX_PATH + 1]; // contains trailing '\'
SETTINGS settings;
STATS stats;
HINSTANCE hInstance = NULL;

SKIN skin;
TCHAR ucButtonMask = WBIFF_BUTTON_STATE_NONE;

HANDLE hThread_CheckNow = NULL;
HANDLE hThread_FlashWindow = NULL;
HANDLE hThread_FlashTray = NULL;
HANDLE hThread_CheckMailAnimation = NULL;
HANDLE hEvent_CheckMailAnimation = NULL;
HANDLE hEvent_ContinueAnimationTray = NULL;
HANDLE hEvent_ContinueAnimationWindow = NULL;

// state objects/variables
DWORD dwState = WBIFF_CURRENT_STATE_NULL;

TCHAR szWindowText[64] = {0}; // current window (and tray) text
int nCount = 0;               // batch count of waiting messages
NOTIFYICONDATA notifyIcon;    // Tray icon object

// version specific
DWORD dwOsVersion = WBIFF_VERSION_NULL;

/////////////////////////////////////////////////////////////////////////////
//
// Function forwards

void
OnDestroy(HWND hwnd);

BOOL
OnCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct);

void
OnClose(HWND hwnd);

void
OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT flags);

void
OnLButtonDblClk(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT flags);

void
OnLButtonUp(HWND hwnd, int x, int y, UINT flags);

void
OnRButtonUp(HWND hwnd, int x, int y, UINT flags);

void 
OnMouseMove(HWND hwnd, int x, int y, UINT flags);

void
OnTimer(HWND hwnd, UINT id);

void
OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);

void
OnPaint(HWND hwnd);

BOOL
OnEraseBkgnd(HWND hwnd, HDC hdc);

void 
OnTrayCommand(HWND hwnd, WPARAM wParam, LPARAM lParam);

/////////////////////////////////////////////////////////////////////////////
//
// Window message handler

LRESULT _stdcall 
mainwnd_WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {

		HANDLE_MSG(hwnd, WM_DESTROY, OnDestroy);
		HANDLE_MSG(hwnd, WM_CREATE, OnCreate);
		HANDLE_MSG(hwnd, WM_LBUTTONDOWN, OnLButtonDown);
		HANDLE_MSG(hwnd, WM_LBUTTONDBLCLK, OnLButtonDblClk);
		HANDLE_MSG(hwnd, WM_LBUTTONUP, OnLButtonUp);
		HANDLE_MSG(hwnd, WM_RBUTTONUP, OnRButtonUp);
		HANDLE_MSG(hwnd, WM_MOUSEMOVE, OnMouseMove);
		HANDLE_MSG(hwnd, WM_TIMER, OnTimer);
		HANDLE_MSG(hwnd, WM_COMMAND, OnCommand);
		HANDLE_MSG(hwnd, WM_PAINT, OnPaint);
		HANDLE_MSG(hwnd, WM_ERASEBKGND, OnEraseBkgnd);
		HANDLE_MSG(hwnd, WM_CLOSE, OnClose);
		
		case WM_NOTIFY:
			{
				LPNMHDR hdr = (LPNMHDR) lParam;
				if (hdr->code == TTN_GETDISPINFO)
					impl_SetToolTipText((LPNMTTDISPINFO) hdr);
			}
			break;	
	
		case WM_CHAR:
			if (settings.bQuickMinimize && (TCHAR) wParam == VK_ESCAPE)
				impl_ToggleMainWindow(hwnd);
			break;

		case WBIFF_IDM_SYSTRAY_CALLBACK_MESSAGE:
			OnTrayCommand(hwnd, wParam, lParam);
			break;

		case WBIFF_IDM_THREAD_DONE_FLASH_WINDOW:
			OnThreadDone_FlashWindow(hwnd, wParam, lParam);
			break;

		case WBIFF_IDM_THREAD_DONE_FLASH_TRAY:
			OnThreadDone_FlashTray(hwnd, wParam, lParam);
			break;

		case WBIFF_IDM_THREAD_DONE_CHECK_NOW:
			OnThreadDone_CheckNow(hwnd, wParam, lParam);
			break;

		case WBIFF_IDM_THREAD_DONE_CHECK_MAIL_ANIMATION:
			OnThreadDone_CheckMailAnimation(hwnd, wParam, lParam);
			break;

		case WM_HOTKEY:
			impl_ClearAnimations();
			impl_HandleHotKey(hwnd, (int) wParam);
			break;

		case WBIFF_IDM_BEGIN_NOTIFY_ANIMATION:
			impl_BeginNotificationAnimations(hwnd);
			break;

		default:
			return DefWindowProc(hwnd, message, wParam, lParam);
			break;
	}

	return 0;
}

HWND 
mainwnd_Create(HINSTANCE hInstance)
{
	WNDCLASSEX wce;
	
	ZeroMemory(&wce, sizeof(WNDCLASSEX));
	wce.cbSize = sizeof(WNDCLASSEX);
	wce.style = CS_DBLCLKS;
	wce.lpfnWndProc = mainwnd_WndProc;
	wce.cbClsExtra = 0;
	wce.cbWndExtra = 0;
	wce.hInstance = hInstance;
	wce.hIcon = NULL; 
	wce.hIconSm = NULL;
	wce.hCursor = LoadCursor(NULL, IDC_ARROW);
	wce.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
	wce.lpszMenuName = NULL;
	wce.lpszClassName = WBIFF_CLASSNAME;
	
	if (RegisterClassEx(&wce)) {
		
		HWND hwnd = CreateWindowEx(
			/*WS_EX_LAYERED |*/ WS_EX_TOOLWINDOW, 
			WBIFF_CLASSNAME, 
			WBIFF_WINDOWNAME, 
			WS_POPUP, 
			0, 
			0, 
			0, 
			0, 
			(HWND) NULL, 
			(HMENU) NULL, 
			hInstance, 
			NULL);

		//SetLayeredWindowAttributes(hwnd, 0, 175, LWA_ALPHA);
		return hwnd;
	}
	else
		return NULL;
}

int 
mainwnd_Main(HMODULE hModule)
{
	MSG msg;
	HWND hwnd = NULL;
	TCHAR* p;
	INITCOMMONCONTROLSEX iccex;
	int result = 0;

	TRACE_OPEN(_T("C:/wbiff.log.txt"))

	// Initialize 
	iccex.dwSize = sizeof(INITCOMMONCONTROLSEX);
	iccex.dwICC = ICC_HOTKEY_CLASS | ICC_LISTVIEW_CLASSES | ICC_UPDOWN_CLASS | ICC_BAR_CLASSES;
	InitCommonControlsEx(&iccex);

	hInstance = (HINSTANCE) hModule;
	szWorkingFolder[0] = _T('\0');
	
  // get CWD
	p = szWorkingFolder + GetModuleFileName(hModule, szWorkingFolder, sizeof(szWorkingFolder));
	while (p >= szWorkingFolder && *p != _T('\\'))
		p--;
	*(++p) = _T('\0');

	impl_GetVersion();
	settings_Load(&settings, true);
	stats_Load(&stats);

	if (mail_general_Startup() != 0) {
		
		// program can't run without Winsock TCP/IP stack installed
		MessageBox(NULL, _T("Failed to initialize the Windows sockets library. The program cannot continue."), 
			WBIFF_WINDOWNAME, MB_OK | MB_ICONSTOP);
		result = -1;
	}
	else {

		// Create the main window and begin message pump
		if (hwnd = mainwnd_Create(hInstance)) {

			if (!settings.bStartMinimized) {
				
				ShowWindow(hwnd, SW_SHOW);
				UpdateWindow(hwnd);

				dwState |= WBIFF_CURRENT_STATE_MAINWND_VISIBLE;
			}

			impl_RenderSettings(hwnd);

			while (GetMessage(&msg, NULL, 0, 0)) {

				TranslateMessage(&msg); 
				DispatchMessage(&msg);
			}
		}

		mail_general_Cleanup();
	
		result = msg.wParam;
	}

	settings_Save(&settings);
	settings_UnLoad(&settings);
	stats_Save(&stats);

	TRACE_CLOSE()
	
	return result;
}

#ifdef _DEBUG
int _stdcall 
WinMain(HINSTANCE h, HINSTANCE p, LPSTR c, int n) // Debug main
{
	int result;
		
	// Enable all CRT debugging options (these are off by default)
	_CrtSetDbgFlag(
		_CRTDBG_ALLOC_MEM_DF |
		_CRTDBG_DELAY_FREE_MEM_DF |
		_CRTDBG_CHECK_ALWAYS_DF |
#ifdef WBIFF_REPORT_CRT_MEMORY_LEAKS
		_CRTDBG_CHECK_CRT_DF |
#endif
		_CRTDBG_LEAK_CHECK_DF);

//	_CrtSetReportMode();

	result = mainwnd_Main(h); // call entry point explicity
	return result;
}
#else
void __cdecl 
WinMainCRTStartup(void) // Release main
{
	HANDLE hMutex = CreateMutex(NULL, TRUE, WBIFF_WINDOWNAME); // overkill?
	int result = GetLastError();

	if (result == ERROR_ALREADY_EXISTS) {

		HWND hwnd = FindWindow(WBIFF_CLASSNAME, WBIFF_WINDOWNAME);
		if (hwnd)
			MessageBox(hwnd, _T("An instance of wbiff! is already running."), WBIFF_WINDOWNAME, MB_OK | MB_ICONINFORMATION);
	}
	else {

		result = mainwnd_Main(GetModuleHandle(NULL));
	}
	
	CloseHandle(hMutex);	
	ExitProcess(result);
}
#endif

/////////////////////////////////////////////////////////////////////////////
//
// WM_* handlers

void 
OnDestroy(HWND hwnd)
{
	impl_SetTimer(hwnd, false);
	impl_DestroySkin(hwnd);
	impl_UnrenderSettings(hwnd);

	// Remove tray icon only when terminating. this avoids a "flashing"
	// effect that can result from removing then adding a tray icon
	Shell_NotifyIcon(NIM_DELETE, &notifyIcon);

	PostQuitMessage(0);
}

BOOL 
OnCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct)
{
	// (2000-06-03)
	//
	// Modified OnCreate handler so that initialization of
	// tray icon occurs in impl_InializeSkin rather than in 
	// this function.

	HICON hIcon;
	BOOL result = FALSE;

	// determine if we should start "enabled"
	if (settings.bStartEnabled)
		dwState |= WBIFF_CURRENT_STATE_ENABLED;

	// initialize the tray with the "blank" icon
	notifyIcon.cbSize = sizeof(NOTIFYICONDATA);
	notifyIcon.hWnd = hwnd;
	notifyIcon.uCallbackMessage = WBIFF_IDM_SYSTRAY_CALLBACK_MESSAGE;
	notifyIcon.uID = IDC_WBIFF_TRAYICON;

	// temporary icon 
	hIcon = (HICON) LoadImage(hInstance, MAKEINTRESOURCE(IDI_ICON6), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
	notifyIcon.hIcon = hIcon;
	notifyIcon.uFlags = NIF_ICON | NIF_MESSAGE;

	Shell_NotifyIcon(NIM_ADD, &notifyIcon);

	if (impl_InitalizeSkin(hwnd)) {

		if (settings.bStartupCheck && settings.nAccounts)
			PostMessage(hwnd, WM_COMMAND, IDM_CHECK_NOW, 0);
		else 	{
			
			// set the timer to check if necessary
			if (dwState & WBIFF_CURRENT_STATE_ENABLED)
				impl_SetTimer(hwnd, true);
		}

		result = TRUE;
	}

	// temporary icon is replaced by "real" icon after skin is initialized
	// *or*, initialization failed and we exit anyway
	DestroyIcon(hIcon);
	return result;
}


void 
OnLButtonDown(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT flags)
{
	POINT point;
	RECT* rect = NULL;
	
	point.x = x;
	point.y = y;

	impl_ClearAnimations();

	if (!(GetKeyState(VK_SHIFT) & 0x8000)) {
	
		if (rect = impl_GetRectAndSetButtonMask(hwnd, &point)) {

			SetCapture(hwnd);
			InvalidateRect(hwnd, rect, TRUE);
			UpdateWindow(hwnd);
		}
		else if (PtInRect(&skin.CaptionRect, point) || settings.bEasyMove) {
		
			PostMessage(hwnd, WM_NCLBUTTONDOWN, HTCAPTION, MAKELPARAM(x, y));
		}
	}
}

void 
OnLButtonUp(HWND hwnd, int x, int y, UINT flags)
{
	RECT* rect = NULL;
	POINT point;
	bool bKillingProgram = false;

	point.x = x;
	point.y = y;
	
	ReleaseCapture();	
	
	if (ucButtonMask & WBIFF_BUTTON_STATE_SYSBUTTON) {

		if (PtInRect(&skin.SysButtonRect, point)) {

			const unsigned offset = 3;

			point.x = skin.SysButtonRect.left + offset;
			point.y = skin.SysButtonRect.top + offset;

			ClientToScreen(hwnd, &point);
			impl_ShowMenu(hwnd, &point);
		}
		
		rect = &skin.SysButtonRect;
	}
	else if (ucButtonMask & WBIFF_BUTTON_STATE_MINBUTTON) {

		if (PtInRect(&skin.MinButtonRect, point)) {

			impl_ToggleMainWindow(hwnd);
		}

		rect = &skin.MinButtonRect;
	}
	else if (ucButtonMask & WBIFF_BUTTON_STATE_CLSBUTTON) {

		if (PtInRect(&skin.ClsButtonRect, point)) {

			DestroyWindow(hwnd);
			bKillingProgram = true;
		}

		rect = &skin.ClsButtonRect;
	}
	else if (GetKeyState(VK_SHIFT) & 0x8000)
		impl_ShiftClickAction(hwnd);

	if (!bKillingProgram) {
	
		ucButtonMask = WBIFF_BUTTON_STATE_NONE;
		InvalidateRect(hwnd, rect, TRUE);
		UpdateWindow(hwnd);
	}
}

void 
OnMouseMove(HWND hwnd, int x, int y, UINT flags)
{
	RECT* rect = NULL;
	POINT point;
	
	point.x = x;
	point.y = y;

	if (ucButtonMask & WBIFF_BUTTON_STATE_DOWN) {

		if (ucButtonMask & WBIFF_BUTTON_STATE_SYSBUTTON) {
			
			if (!PtInRect(&skin.SysButtonRect, point))
				ucButtonMask &= ~WBIFF_BUTTON_STATE_SYSBUTTON;				

			rect = &skin.SysButtonRect;
		}
		else if (ucButtonMask & WBIFF_BUTTON_STATE_MINBUTTON) {
			
			if (!PtInRect(&skin.MinButtonRect, point))
				ucButtonMask &= ~WBIFF_BUTTON_STATE_MINBUTTON;				

			rect = &skin.MinButtonRect;
		}
		else if (ucButtonMask & WBIFF_BUTTON_STATE_CLSBUTTON) {
			
			if (!PtInRect(&skin.ClsButtonRect, point))
				ucButtonMask &= ~WBIFF_BUTTON_STATE_CLSBUTTON;
				
			rect = &skin.ClsButtonRect;
		}
		else {

			// add the rectangle bit if we're entering another window
			rect = impl_GetRectAndSetButtonMask(hwnd, &point);
		}

		if (rect) {

			// only update if there is a rect defined
			InvalidateRect(hwnd, rect, TRUE);
			UpdateWindow(hwnd);
		}
	}
}

void 
OnLButtonDblClk(HWND hwnd, BOOL fDoubleClick, int x, int y, UINT flags)
{
	impl_ClearAnimations();
	impl_DoubleClickAction(hwnd);
}

void 
OnRButtonUp(HWND hwnd, int x, int y, UINT flags)
{
	POINT point;

	point.x = x;
	point.y = y;

	impl_ClearAnimations();

	ClientToScreen(hwnd, &point);
	impl_ShowMenu(hwnd, &point);
}

void 
OnTimer(HWND hwnd, UINT id)
{
	switch (id) {

		case WBIFF_TIMER_CHECK_MAIL:
			TRACE("******* OnTimer(id = %d)\r\n", id);
			impl_ClearAnimations();
			impl_CheckNow(hwnd);
			impl_SwapOutOfMemory();
			break;
	}
}

void 
OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
	switch (id) {
#if 0
    // I had, at one point, decided to implement context-sensitive help.
    // This is the message Windows sends when the users requests help.
    // The HtmlHelp API is part of latest platform SDK.
		case IDM_HELP:
			{
				HWND hwndHelp = HtmlHelp(
					GetDesktopWindow(),
					_T("wbiff.chm"),
					HH_DISPLAY_TOPIC,
					0);
			}
			break;
#endif
		case IDM_ABOUT:
			dwState |= WBIFF_CURRENT_STATE_IN_ABOUTDLG;
			about_DisplayDialog(hwnd);
			dwState &= ~WBIFF_CURRENT_STATE_IN_ABOUTDLG;
			break;

		case IDM_CLOSE:
			DestroyWindow(hwnd);
			break;

		case IDM_SETTINGS:
			dwState |= WBIFF_CURRENT_STATE_IN_SETTINGSDLG;
			impl_Settings(hwnd);
			dwState &= ~WBIFF_CURRENT_STATE_IN_SETTINGSDLG;
			break;

		case IDM_CHECK_NOW:
			TRACE("impl_CheckNow\r\n");
			impl_CheckNow(hwnd);
			break;

		case IDM_GOTO_USERHYPERLINK:
			util_OpenObject(hwnd, skin.HyperLink);
			break;

		case IDM_ENABLED:
			impl_ToggleEnabledStatus(hwnd);
			TRACE("enabled status toggled (status = %s)\r\n", (dwState & WBIFF_CURRENT_STATE_ENABLED ? "on" : "off"));
			break;

		case IDM_SUMMARY:
			impl_ToggleSummaryWindow(hwnd);
			break;
		
		case IDM_GOTO_HOME:
			util_OpenObject(hwnd, _T("http://guesswork.org/software/wbiff/"));
			break;

		case IDM_MINIMIZE:
		case IDM_RESTORE:
			impl_ToggleMainWindow(hwnd);
			break;

		case IDM_RUN_CLIENT:
			impl_RunClient(hwnd);
			break;

		case IDM_CLEAR_FLAG:
			impl_ClearFlag(hwnd);
			break;
	}
}

void 
OnPaint(HWND hwnd)
{
	//
	// Paint each region of the window separately
	
	PAINTSTRUCT ps;
	HDC cdc;
	RECT dummy;

	BeginPaint(hwnd, &ps);

	cdc = CreateCompatibleDC(ps.hdc);
	SelectObject(cdc, skin.hBitmap);

	// Window
	BitBlt(ps.hdc,
		ps.rcPaint.left,
		ps.rcPaint.top,
		ps.rcPaint.right,
		ps.rcPaint.bottom,
		cdc, 
		ps.rcPaint.left + skin.Window_X,
		ps.rcPaint.top + skin.Window_Y,
		SRCCOPY);

	// Flag
	IntersectRect(&dummy, &ps.rcPaint, &skin.FlagRect);
	if (!IsRectEmpty(&dummy)) {
		
		BitBlt(ps.hdc,
			skin.FlagRect.left,
			skin.FlagRect.top,
			skin.FlagRect.right - skin.FlagRect.left,
			skin.FlagRect.bottom - skin.FlagRect.top,
			cdc,
			(dwState & WBIFF_CURRENT_STATE_FLAG_UP) ? skin.FlagUp_X : skin.FlagDown_X,
			(dwState & WBIFF_CURRENT_STATE_FLAG_UP) ? skin.FlagUp_Y : skin.FlagDown_Y,
			SRCCOPY);
	}

	// Sys, Min, Cls buttons
	IntersectRect(&dummy, &ps.rcPaint, &skin.SysButtonRect);
	if (!IsRectEmpty(&dummy)) {

		BitBlt(ps.hdc,
			skin.SysButtonRect.left,
			skin.SysButtonRect.top,
			skin.SysButtonRect.right - skin.SysButtonRect.left,
			skin.SysButtonRect.bottom - skin.SysButtonRect.top,
			cdc,
			(ucButtonMask == (WBIFF_BUTTON_STATE_SYSBUTTON | WBIFF_BUTTON_STATE_DOWN)) ? skin.SysButtonDown_X : skin.SysButtonUp_X,
			(ucButtonMask == (WBIFF_BUTTON_STATE_SYSBUTTON | WBIFF_BUTTON_STATE_DOWN)) ? skin.SysButtonDown_Y : skin.SysButtonUp_Y,
			SRCCOPY);
	}
	
	IntersectRect(&dummy, &ps.rcPaint, &skin.MinButtonRect);
	if (!IsRectEmpty(&dummy)) {
		
		BitBlt(ps.hdc,
			skin.MinButtonRect.left,
			skin.MinButtonRect.top,
			skin.MinButtonRect.right - skin.MinButtonRect.left,
			skin.MinButtonRect.bottom - skin.MinButtonRect.top,
			cdc,
			(ucButtonMask == (WBIFF_BUTTON_STATE_MINBUTTON | WBIFF_BUTTON_STATE_DOWN)) ? skin.MinButtonDown_X : skin.MinButtonUp_X,
			(ucButtonMask == (WBIFF_BUTTON_STATE_MINBUTTON | WBIFF_BUTTON_STATE_DOWN)) ? skin.MinButtonDown_Y : skin.MinButtonUp_Y,
			SRCCOPY);
	}

	IntersectRect(&dummy, &ps.rcPaint, &skin.ClsButtonRect);
	if (!IsRectEmpty(&dummy)) {
		
		BitBlt(ps.hdc,
			skin.ClsButtonRect.left,
			skin.ClsButtonRect.top,
			skin.ClsButtonRect.right - skin.ClsButtonRect.left,
			skin.ClsButtonRect.bottom - skin.ClsButtonRect.top,
			cdc,
			(ucButtonMask == (WBIFF_BUTTON_STATE_CLSBUTTON | WBIFF_BUTTON_STATE_DOWN)) ? skin.ClsButtonDown_X : skin.ClsButtonUp_X,
			(ucButtonMask == (WBIFF_BUTTON_STATE_CLSBUTTON | WBIFF_BUTTON_STATE_DOWN)) ? skin.ClsButtonDown_Y : skin.ClsButtonUp_Y,
			SRCCOPY);
	}

	// Font output
	IntersectRect(&dummy, &ps.rcPaint, &skin.FontRect);
	if (!IsRectEmpty(&dummy) && szWindowText[0] != _T('\0')) {

		// create the font if neccessary
		if (!skin.hFont) {

			// for compatibility with old (ugly-fonted) skins we need to determine
			// which type of font to CREATE and which MODE we should draw in
			int height;
			int weight;

			if (skin.FontFaceName[0] != _T('\0')) {

				// font name specified, use new-style mode: face name and point size
				height = -MulDiv(settings.nFontSize, GetDeviceCaps(ps.hdc, LOGPIXELSY), 72);
				weight = FW_NORMAL;
			}
			else {

				// old-style ugly font (yes, these values were hardcoded)
				height = 11;
				weight = FW_DONTCARE;
			}

			SetMapMode(ps.hdc, MM_TEXT);
			skin.hFont = CreateFont(height,	0, 0,	0, weight, FALSE, FALSE, FALSE, ANSI_CHARSET, OUT_TT_PRECIS, CLIP_TT_ALWAYS, DEFAULT_QUALITY, FF_SWISS | VARIABLE_PITCH, skin.FontFaceName);
		}
		
		SelectObject(ps.hdc, skin.hFont);
		SetTextColor(ps.hdc, skin.FontColor);
		SetBkMode(ps.hdc, TRANSPARENT);
		DrawText(ps.hdc, szWindowText, lstrlen(szWindowText), &skin.FontRect, DT_WORDBREAK | DT_CENTER | DT_MODIFYSTRING | DT_NOCLIP | DT_END_ELLIPSIS | DT_NOPREFIX);
	}
	
	DeleteDC(cdc);
	EndPaint(hwnd, &ps);	
}

BOOL 
OnEraseBkgnd(HWND hwnd, HDC hdc)
{
	return TRUE;
}

void
OnClose(HWND hwnd)
{
	DestroyWindow(hwnd);
}

void 
OnTrayCommand(HWND hwnd, WPARAM wParam, LPARAM lParam)
{
  // only allow tray interaction if no dialogs are raised
	if (wParam == notifyIcon.uID && !(dwState & (WBIFF_CURRENT_STATE_IN_ABOUTDLG | WBIFF_CURRENT_STATE_IN_SETTINGSDLG))) {

		if (LOWORD(lParam) == WM_RBUTTONUP) {

			POINT point;
			GetCursorPos(&point);

			impl_ClearAnimations();
			impl_ShowMenu(hwnd, &point);
		}
		/*
		else if (LOWORD(lParam) == WM_LBUTTONDBLCLK) {

			impl_ClearAnimations();
			impl_ToggleMainWindow(hwnd);
		}
		*/
		else if (LOWORD(lParam) == WM_LBUTTONUP) {

			// activate the window
			impl_ClearAnimations();

			// check if any modifiers were used (i.e. shift)
			if (GetKeyState(VK_SHIFT) & 0x8000)
				impl_ShiftClickAction(hwnd);
			else
				//SetForegroundWindow(hwnd);
				impl_ToggleSummaryWindow(hwnd); //'ad'

		}
	}
}


